# Marpa::R3 is Copyright (C) 2016, Jeffrey Kegler.
#
# This module is free software; you can redistribute it and/or modify it
# under the same terms as Perl 5.10.1. For more details, see the full text
# of the licenses in the directory LICENSES.
#
# This program is distributed in the hope that it will be
# useful, but it is provided “as is” and without any express
# or implied warranties. For details, see the full text of
# of the licenses in the directory LICENSES.

package Marpa::R3::MetaG;

use 5.010001;
use strict;
use warnings;

use vars qw($VERSION $STRING_VERSION);
$VERSION        = '4.001_024';
$STRING_VERSION = $VERSION;
## no critic(BuiltinFunctions::ProhibitStringyEval)
$VERSION = eval $VERSION;
## use critic

package Marpa::R3::Internal::MetaG;

use English qw( -no_match_vars );

sub hashed_grammar {
    my $hashed_metag;

## no critic(RegularExpressions::RequireDotMatchAnything)
## no critic(RegularExpressions::RequireExtendedFormatting)
## no critic(RegularExpressions::RequireLineBoundaryMatching)
## no critic(RegularExpressions::ProhibitEscapedMetacharacters)
## no critic(RegularExpressions::ProhibitComplexRegexes)

## The code after this line was automatically generated by sl_to_hash.pl
$hashed_metag = {
                  'character_classes' => {
                                           '[[\']]' => [
                                                         '[\']',
                                                         ''
                                                       ],
                                           '[[+-]]' => [
                                                         '[+-]',
                                                         ''
                                                       ],
                                           '[[,]]' => [
                                                        '[,]',
                                                        ''
                                                      ],
                                           '[[01]]' => [
                                                         '[01]',
                                                         ''
                                                       ],
                                           '[[1]]' => [
                                                        '[1]',
                                                        ''
                                                      ],
                                           '[[[:alnum:]]]' => [
                                                                '[[:alnum:]]',
                                                                ''
                                                              ],
                                           '[[\\#]]' => [
                                                          '[\\#]',
                                                          ''
                                                        ],
                                           '[[\\(]]' => [
                                                          '[\\(]',
                                                          ''
                                                        ],
                                           '[[\\)]]' => [
                                                          '[\\)]',
                                                          ''
                                                        ],
                                           '[[\\*]]' => [
                                                          '[\\*]',
                                                          ''
                                                        ],
                                           '[[\\+]]' => [
                                                          '[\\+]',
                                                          ''
                                                        ],
                                           '[[\\,]]' => [
                                                          '[\\,]',
                                                          ''
                                                        ],
                                           '[[\\-]]' => [
                                                          '[\\-]',
                                                          ''
                                                        ],
                                           '[[\\:]]' => [
                                                          '[\\:]',
                                                          ''
                                                        ],
                                           '[[\\;]]' => [
                                                          '[\\;]',
                                                          ''
                                                        ],
                                           '[[\\<]]' => [
                                                          '[\\<]',
                                                          ''
                                                        ],
                                           '[[\\=]]' => [
                                                          '[\\=]',
                                                          ''
                                                        ],
                                           '[[\\>]]' => [
                                                          '[\\>]',
                                                          ''
                                                        ],
                                           '[[\\[]]' => [
                                                          '[\\[]',
                                                          ''
                                                        ],
                                           '[[\\\\]]' => [
                                                           '[\\\\]',
                                                           ''
                                                         ],
                                           '[[\\]]]' => [
                                                          '[\\]]',
                                                          ''
                                                        ],
                                           '[[\\^]]' => [
                                                          '[\\^]',
                                                          ''
                                                        ],
                                           '[[\\d]]' => [
                                                          '[\\d]',
                                                          ''
                                                        ],
                                           '[[\\s\\w]]' => [
                                                             '[\\s\\w]',
                                                             ''
                                                           ],
                                           '[[\\s]]' => [
                                                          '[\\s]',
                                                          ''
                                                        ],
                                           '[[\\w]]' => [
                                                          '[\\w]',
                                                          ''
                                                        ],
                                           '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => [
                                                                                                 '[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]',
                                                                                                 ''
                                                                                               ],
                                           '[[\\{]]' => [
                                                          '[\\{]',
                                                          ''
                                                        ],
                                           '[[\\|]]' => [
                                                          '[\\|]',
                                                          ''
                                                        ],
                                           '[[\\}]]' => [
                                                          '[\\}]',
                                                          ''
                                                        ],
                                           '[[\\~]]' => [
                                                          '[\\~]',
                                                          ''
                                                        ],
                                           '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => [
                                                                                                                 '[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]',
                                                                                                                 ''
                                                                                                               ],
                                           '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => [
                                                                                                                      '[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]',
                                                                                                                      ''
                                                                                                                    ],
                                           '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => [
                                                                                                  '[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]',
                                                                                                  ''
                                                                                                ],
                                           '[[a-zA-Z]]' => [
                                                             '[a-zA-Z]',
                                                             ''
                                                           ],
                                           '[[a]]' => [
                                                        '[a]',
                                                        ''
                                                      ],
                                           '[[b]]' => [
                                                        '[b]',
                                                        ''
                                                      ],
                                           '[[c]]' => [
                                                        '[c]',
                                                        ''
                                                      ],
                                           '[[d]]' => [
                                                        '[d]',
                                                        ''
                                                      ],
                                           '[[e]]' => [
                                                        '[e]',
                                                        ''
                                                      ],
                                           '[[f]]' => [
                                                        '[f]',
                                                        ''
                                                      ],
                                           '[[g]]' => [
                                                        '[g]',
                                                        ''
                                                      ],
                                           '[[h]]' => [
                                                        '[h]',
                                                        ''
                                                      ],
                                           '[[i]]' => [
                                                        '[i]',
                                                        ''
                                                      ],
                                           '[[k]]' => [
                                                        '[k]',
                                                        ''
                                                      ],
                                           '[[l]]' => [
                                                        '[l]',
                                                        ''
                                                      ],
                                           '[[m]]' => [
                                                        '[m]',
                                                        ''
                                                      ],
                                           '[[n]]' => [
                                                        '[n]',
                                                        ''
                                                      ],
                                           '[[o]]' => [
                                                        '[o]',
                                                        ''
                                                      ],
                                           '[[p]]' => [
                                                        '[p]',
                                                        ''
                                                      ],
                                           '[[r]]' => [
                                                        '[r]',
                                                        ''
                                                      ],
                                           '[[s]]' => [
                                                        '[s]',
                                                        ''
                                                      ],
                                           '[[t]]' => [
                                                        '[t]',
                                                        ''
                                                      ],
                                           '[[u]]' => [
                                                        '[u]',
                                                        ''
                                                      ],
                                           '[[v]]' => [
                                                        '[v]',
                                                        ''
                                                      ],
                                           '[[w]]' => [
                                                        '[w]',
                                                        ''
                                                      ],
                                           '[[x]]' => [
                                                        '[x]',
                                                        ''
                                                      ],
                                           '[[y]]' => [
                                                        '[y]',
                                                        ''
                                                      ]
                                         },
                  'discard_default_adverbs' => undef,
                  'first_lhs' => 'statements',
                  'lexeme_default_adverbs' => {
                                                'action' => '[start,length,value]',
                                                'bless' => '::name'
                                              },
                  'rules' => {
                               'G1' => [
                                         {
                                           'action' => '::first',
                                           'lhs' => '[:start]',
                                           'rhs' => [
                                                      'statements'
                                                    ],
                                           'xbnfid' => '[:start],statements'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'action',
                                           'rhs' => [
                                                      '[Lex-33]',
                                                      '[Lex-34]',
                                                      'action name'
                                                    ],
                                           'xbnfid' => 'action,[Lex-33],[Lex-34],action name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'action name',
                                           'rhs' => [
                                                      'Perl name'
                                                    ],
                                           'xbnfid' => 'action name,Perl name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'action name',
                                           'rhs' => [
                                                      'array descriptor'
                                                    ],
                                           'xbnfid' => 'action name,array descriptor'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'action name',
                                           'rhs' => [
                                                      'reserved action name'
                                                    ],
                                           'xbnfid' => 'action name,reserved action name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'action'
                                                    ],
                                           'xbnfid' => 'adverb item,action'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'blessing'
                                                    ],
                                           'xbnfid' => 'adverb item,blessing'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'event specification'
                                                    ],
                                           'xbnfid' => 'adverb item,event specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'group association'
                                                    ],
                                           'xbnfid' => 'adverb item,group association'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'left association'
                                                    ],
                                           'xbnfid' => 'adverb item,left association'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'naming'
                                                    ],
                                           'xbnfid' => 'adverb item,naming'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'null adverb'
                                                    ],
                                           'xbnfid' => 'adverb item,null adverb'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'null ranking specification'
                                                    ],
                                           'xbnfid' => 'adverb item,null ranking specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'pause specification'
                                                    ],
                                           'xbnfid' => 'adverb item,pause specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'priority specification'
                                                    ],
                                           'xbnfid' => 'adverb item,priority specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'proper specification'
                                                    ],
                                           'xbnfid' => 'adverb item,proper specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'rank specification'
                                                    ],
                                           'xbnfid' => 'adverb item,rank specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'right association'
                                                    ],
                                           'xbnfid' => 'adverb item,right association'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb item',
                                           'rhs' => [
                                                      'separator specification'
                                                    ],
                                           'xbnfid' => 'adverb item,separator specification'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb list',
                                           'rhs' => [
                                                      'adverb list items'
                                                    ],
                                           'xbnfid' => 'adverb list,adverb list items'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'adverb list items',
                                           'min' => 0,
                                           'rhs' => [
                                                      'adverb item'
                                                    ],
                                           'xbnfid' => 'adverb list items,adverb item'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'alternative',
                                           'rhs' => [
                                                      'rhs',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'alternative,rhs,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'alternative name',
                                           'rhs' => [
                                                      'single quoted name'
                                                    ],
                                           'xbnfid' => 'alternative name,single quoted name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'alternative name',
                                           'rhs' => [
                                                      'standard name'
                                                    ],
                                           'xbnfid' => 'alternative name,standard name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'alternatives',
                                           'min' => 1,
                                           'proper' => '1',
                                           'rhs' => [
                                                      'alternative'
                                                    ],
                                           'separator' => 'op equal priority',
                                           'xbnfid' => 'alternatives,alternative'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'blessing',
                                           'rhs' => [
                                                      '[Lex-66]',
                                                      '[Lex-67]',
                                                      'blessing name'
                                                    ],
                                           'xbnfid' => 'blessing,[Lex-66],[Lex-67],blessing name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'blessing name',
                                           'rhs' => [
                                                      'reserved blessing name'
                                                    ],
                                           'xbnfid' => 'blessing name,reserved blessing name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'blessing name',
                                           'rhs' => [
                                                      'standard name'
                                                    ],
                                           'xbnfid' => 'blessing name,standard name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'completion event declaration',
                                           'rhs' => [
                                                      '[Lex-16]',
                                                      'event initialization',
                                                      '[Lex-17]',
                                                      '[Lex-18]',
                                                      'symbol name'
                                                    ],
                                           'xbnfid' => 'completion event declaration,[Lex-16],event initialization,[Lex-17],[Lex-18],symbol name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'default rule',
                                           'rhs' => [
                                                      '[Lex-7]',
                                                      'op declare bnf',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'default rule,[Lex-7],op declare bnf,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'discard default statement',
                                           'rhs' => [
                                                      '[Lex-11]',
                                                      '[Lex-12]',
                                                      '[Lex-13]',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'discard default statement,[Lex-11],[Lex-12],[Lex-13],adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'discard rule',
                                           'rhs' => [
                                                      '[Lex-14]',
                                                      'op declare match',
                                                      'single symbol',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'discard rule,[Lex-14],op declare match,single symbol,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'empty rule',
                                           'rhs' => [
                                                      'lhs',
                                                      'op declare',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'empty rule,lhs,op declare,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event initialization',
                                           'rhs' => [
                                                      'event name',
                                                      'event initializer'
                                                    ],
                                           'xbnfid' => 'event initialization,event name,event initializer'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event initializer',
                                           'rhs' => [],
                                           'xbnfid' => 'event initializer'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event initializer',
                                           'rhs' => [
                                                      '[Lex-63]',
                                                      'on or off'
                                                    ],
                                           'xbnfid' => 'event initializer,[Lex-63],on or off'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event name',
                                           'rhs' => [
                                                      'reserved event name'
                                                    ],
                                           'xbnfid' => 'event name,reserved event name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event name',
                                           'rhs' => [
                                                      'single quoted name'
                                                    ],
                                           'xbnfid' => 'event name,single quoted name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event name',
                                           'rhs' => [
                                                      'standard name'
                                                    ],
                                           'xbnfid' => 'event name,standard name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'event specification',
                                           'rhs' => [
                                                      '[Lex-61]',
                                                      '[Lex-62]',
                                                      'event initialization'
                                                    ],
                                           'xbnfid' => 'event specification,[Lex-61],[Lex-62],event initialization'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'group association',
                                           'rhs' => [
                                                      '[Lex-41]',
                                                      '[Lex-42]',
                                                      '[Lex-43]'
                                                    ],
                                           'xbnfid' => 'group association,[Lex-41],[Lex-42],[Lex-43]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'inaccessible statement',
                                           'rhs' => [
                                                      '[Lex-25]',
                                                      '[Lex-26]',
                                                      'inaccessible treatment',
                                                      '[Lex-27]',
                                                      '[Lex-28]'
                                                    ],
                                           'xbnfid' => 'inaccessible statement,[Lex-25],[Lex-26],inaccessible treatment,[Lex-27],[Lex-28]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'inaccessible treatment',
                                           'rhs' => [
                                                      '[Lex-29]'
                                                    ],
                                           'xbnfid' => 'inaccessible treatment,[Lex-29]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'inaccessible treatment',
                                           'rhs' => [
                                                      '[Lex-30]'
                                                    ],
                                           'xbnfid' => 'inaccessible treatment,[Lex-30]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'inaccessible treatment',
                                           'rhs' => [
                                                      '[Lex-31]'
                                                    ],
                                           'xbnfid' => 'inaccessible treatment,[Lex-31]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'left association',
                                           'rhs' => [
                                                      '[Lex-35]',
                                                      '[Lex-36]',
                                                      '[Lex-37]'
                                                    ],
                                           'xbnfid' => 'left association,[Lex-35],[Lex-36],[Lex-37]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'lexeme default statement',
                                           'rhs' => [
                                                      '[Lex-8]',
                                                      '[Lex-9]',
                                                      '[Lex-10]',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'lexeme default statement,[Lex-8],[Lex-9],[Lex-10],adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'lexeme rule',
                                           'rhs' => [
                                                      '[Lex-15]',
                                                      'op declare match',
                                                      'symbol',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'lexeme rule,[Lex-15],op declare match,symbol,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'lhs',
                                           'rhs' => [
                                                      'symbol name'
                                                    ],
                                           'xbnfid' => 'lhs,symbol name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'naming',
                                           'rhs' => [
                                                      '[Lex-68]',
                                                      '[Lex-69]',
                                                      'alternative name'
                                                    ],
                                           'xbnfid' => 'naming,[Lex-68],[Lex-69],alternative name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null adverb',
                                           'rhs' => [
                                                      '[Lex-32]'
                                                    ],
                                           'xbnfid' => 'null adverb,[Lex-32]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null ranking constant',
                                           'rhs' => [
                                                      '[Lex-55]'
                                                    ],
                                           'xbnfid' => 'null ranking constant,[Lex-55]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null ranking constant',
                                           'rhs' => [
                                                      '[Lex-56]'
                                                    ],
                                           'xbnfid' => 'null ranking constant,[Lex-56]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null ranking specification',
                                           'rhs' => [
                                                      '[Lex-50]',
                                                      '[Lex-51]',
                                                      'null ranking constant'
                                                    ],
                                           'xbnfid' => 'null ranking specification,[Lex-50],[Lex-51],null ranking constant'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null ranking specification',
                                           'rhs' => [
                                                      '[Lex-52]',
                                                      '[Lex-53]',
                                                      '[Lex-54]',
                                                      'null ranking constant'
                                                    ],
                                           'xbnfid' => 'null ranking specification,[Lex-52],[Lex-53],[Lex-54],null ranking constant'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'null statement',
                                           'rhs' => [
                                                      '[Lex-0]'
                                                    ],
                                           'xbnfid' => 'null statement,[Lex-0]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'nulled event declaration',
                                           'rhs' => [
                                                      '[Lex-19]',
                                                      'event initialization',
                                                      '[Lex-20]',
                                                      '[Lex-21]',
                                                      'symbol name'
                                                    ],
                                           'xbnfid' => 'nulled event declaration,[Lex-19],event initialization,[Lex-20],[Lex-21],symbol name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'on or off',
                                           'rhs' => [
                                                      '[Lex-64]'
                                                    ],
                                           'xbnfid' => 'on or off,[Lex-64]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'on or off',
                                           'rhs' => [
                                                      '[Lex-65]'
                                                    ],
                                           'xbnfid' => 'on or off,[Lex-65]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'op declare',
                                           'rhs' => [
                                                      'op declare bnf'
                                                    ],
                                           'xbnfid' => 'op declare,op declare bnf'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'op declare',
                                           'rhs' => [
                                                      'op declare match'
                                                    ],
                                           'xbnfid' => 'op declare,op declare match'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'parenthesized rhs primary list',
                                           'rhs' => [
                                                      '[Lex-70]',
                                                      'rhs primary list',
                                                      '[Lex-71]'
                                                    ],
                                           'xbnfid' => 'parenthesized rhs primary list,[Lex-70],rhs primary list,[Lex-71]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'pause specification',
                                           'rhs' => [
                                                      '[Lex-59]',
                                                      '[Lex-60]',
                                                      'before or after'
                                                    ],
                                           'xbnfid' => 'pause specification,[Lex-59],[Lex-60],before or after'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'prediction event declaration',
                                           'rhs' => [
                                                      '[Lex-22]',
                                                      'event initialization',
                                                      '[Lex-23]',
                                                      '[Lex-24]',
                                                      'symbol name'
                                                    ],
                                           'xbnfid' => 'prediction event declaration,[Lex-22],event initialization,[Lex-23],[Lex-24],symbol name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'priorities',
                                           'min' => 1,
                                           'proper' => '1',
                                           'rhs' => [
                                                      'alternatives'
                                                    ],
                                           'separator' => 'op loosen',
                                           'xbnfid' => 'priorities,alternatives'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'priority rule',
                                           'rhs' => [
                                                      'lhs',
                                                      'op declare',
                                                      'priorities'
                                                    ],
                                           'xbnfid' => 'priority rule,lhs,op declare,priorities'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'priority specification',
                                           'rhs' => [
                                                      '[Lex-57]',
                                                      '[Lex-58]',
                                                      'signed integer'
                                                    ],
                                           'xbnfid' => 'priority specification,[Lex-57],[Lex-58],signed integer'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'proper specification',
                                           'rhs' => [
                                                      '[Lex-46]',
                                                      '[Lex-47]',
                                                      'boolean'
                                                    ],
                                           'xbnfid' => 'proper specification,[Lex-46],[Lex-47],boolean'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'quantified rule',
                                           'rhs' => [
                                                      'lhs',
                                                      'op declare',
                                                      'single symbol',
                                                      'quantifier',
                                                      'adverb list'
                                                    ],
                                           'xbnfid' => 'quantified rule,lhs,op declare,single symbol,quantifier,adverb list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'quantifier',
                                           'rhs' => [
                                                      '[Lex-72]'
                                                    ],
                                           'xbnfid' => 'quantifier,[Lex-72]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'quantifier',
                                           'rhs' => [
                                                      '[Lex-73]'
                                                    ],
                                           'xbnfid' => 'quantifier,[Lex-73]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rank specification',
                                           'rhs' => [
                                                      '[Lex-48]',
                                                      '[Lex-49]',
                                                      'signed integer'
                                                    ],
                                           'xbnfid' => 'rank specification,[Lex-48],[Lex-49],signed integer'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rhs',
                                           'min' => 1,
                                           'rhs' => [
                                                      'rhs primary'
                                                    ],
                                           'xbnfid' => 'rhs,rhs primary'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rhs primary',
                                           'rhs' => [
                                                      'parenthesized rhs primary list'
                                                    ],
                                           'xbnfid' => 'rhs primary,parenthesized rhs primary list'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rhs primary',
                                           'rhs' => [
                                                      'single quoted string'
                                                    ],
                                           'xbnfid' => 'rhs primary,single quoted string'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rhs primary',
                                           'rhs' => [
                                                      'single symbol'
                                                    ],
                                           'xbnfid' => 'rhs primary,single symbol'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'rhs primary list',
                                           'min' => 1,
                                           'rhs' => [
                                                      'rhs primary'
                                                    ],
                                           'xbnfid' => 'rhs primary list,rhs primary'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'right association',
                                           'rhs' => [
                                                      '[Lex-38]',
                                                      '[Lex-39]',
                                                      '[Lex-40]'
                                                    ],
                                           'xbnfid' => 'right association,[Lex-38],[Lex-39],[Lex-40]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'separator specification',
                                           'rhs' => [
                                                      '[Lex-44]',
                                                      '[Lex-45]',
                                                      'single symbol'
                                                    ],
                                           'xbnfid' => 'separator specification,[Lex-44],[Lex-45],single symbol'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'single symbol',
                                           'rhs' => [
                                                      'character class'
                                                    ],
                                           'xbnfid' => 'single symbol,character class'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'single symbol',
                                           'rhs' => [
                                                      'symbol'
                                                    ],
                                           'xbnfid' => 'single symbol,symbol'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'start rule',
                                           'rhs' => [
                                                      '[Lex-3]',
                                                      'op declare bnf',
                                                      'symbol'
                                                    ],
                                           'xbnfid' => 'start rule,[Lex-3],op declare bnf,symbol'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'start rule',
                                           'rhs' => [
                                                      '[Lex-4]',
                                                      '[Lex-5]',
                                                      '[Lex-6]',
                                                      'symbol'
                                                    ],
                                           'xbnfid' => 'start rule,[Lex-4],[Lex-5],[Lex-6],symbol'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'completion event declaration'
                                                    ],
                                           'xbnfid' => 'statement,completion event declaration'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'default rule'
                                                    ],
                                           'xbnfid' => 'statement,default rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'discard default statement'
                                                    ],
                                           'xbnfid' => 'statement,discard default statement'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'discard rule'
                                                    ],
                                           'xbnfid' => 'statement,discard rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'empty rule'
                                                    ],
                                           'xbnfid' => 'statement,empty rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'inaccessible statement'
                                                    ],
                                           'xbnfid' => 'statement,inaccessible statement'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'lexeme default statement'
                                                    ],
                                           'xbnfid' => 'statement,lexeme default statement'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'lexeme rule'
                                                    ],
                                           'xbnfid' => 'statement,lexeme rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'null statement'
                                                    ],
                                           'xbnfid' => 'statement,null statement'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'nulled event declaration'
                                                    ],
                                           'xbnfid' => 'statement,nulled event declaration'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'prediction event declaration'
                                                    ],
                                           'xbnfid' => 'statement,prediction event declaration'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'priority rule'
                                                    ],
                                           'xbnfid' => 'statement,priority rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'quantified rule'
                                                    ],
                                           'xbnfid' => 'statement,quantified rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'start rule'
                                                    ],
                                           'xbnfid' => 'statement,start rule'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement',
                                           'rhs' => [
                                                      'statement group'
                                                    ],
                                           'xbnfid' => 'statement,statement group'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statement group',
                                           'rhs' => [
                                                      '[Lex-1]',
                                                      'statements',
                                                      '[Lex-2]'
                                                    ],
                                           'xbnfid' => 'statement group,[Lex-1],statements,[Lex-2]'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'statements',
                                           'min' => 1,
                                           'rhs' => [
                                                      'statement'
                                                    ],
                                           'xbnfid' => 'statements,statement'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'symbol',
                                           'rhs' => [
                                                      'symbol name'
                                                    ],
                                           'xbnfid' => 'symbol,symbol name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'symbol name',
                                           'rhs' => [
                                                      'bare name'
                                                    ],
                                           'xbnfid' => 'symbol name,bare name'
                                         },
                                         {
                                           'action' => '[start,length,values]',
                                           'lhs' => 'symbol name',
                                           'rhs' => [
                                                      'bracketed name'
                                                    ],
                                           'xbnfid' => 'symbol name,bracketed name'
                                         }
                                       ],
                               'L0' => [
                                         {
                                           'lhs' => 'Perl identifier',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\w]]'
                                                    ],
                                           'xbnfid' => 'Perl identifier,[[\\w]]'
                                         },
                                         {
                                           'lhs' => 'Perl name',
                                           'min' => 1,
                                           'proper' => '1',
                                           'rhs' => [
                                                      'Perl identifier'
                                                    ],
                                           'separator' => 'double colon',
                                           'xbnfid' => 'Perl name,Perl identifier'
                                         },
                                         {
                                           'lhs' => '[:discard]',
                                           'rhs' => [
                                                      'hash comment'
                                                    ],
                                           'xbnfid' => '[:discard],hash comment'
                                         },
                                         {
                                           'lhs' => '[:discard]',
                                           'rhs' => [
                                                      'whitespace'
                                                    ],
                                           'xbnfid' => '[:discard],whitespace'
                                         },
                                         {
                                           'lhs' => '[Lex-0]',
                                           'rhs' => [
                                                      '[[\\;]]'
                                                    ],
                                           'xbnfid' => '[Lex-0],[[\\;]]'
                                         },
                                         {
                                           'lhs' => '[Lex-10]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-10],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-11]',
                                           'rhs' => [
                                                      '[[d]]',
                                                      '[[i]]',
                                                      '[[s]]',
                                                      '[[c]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[d]]'
                                                    ],
                                           'xbnfid' => '[Lex-11],[[d]],[[i]],[[s]],[[c]],[[a]],[[r]],[[d]]'
                                         },
                                         {
                                           'lhs' => '[Lex-12]',
                                           'rhs' => [
                                                      '[[d]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[a]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-12],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-13]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-13],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-14]',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[d]]',
                                                      '[[i]]',
                                                      '[[s]]',
                                                      '[[c]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[d]]'
                                                    ],
                                           'xbnfid' => '[Lex-14],[[\\:]],[[d]],[[i]],[[s]],[[c]],[[a]],[[r]],[[d]]'
                                         },
                                         {
                                           'lhs' => '[Lex-15]',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[x]]',
                                                      '[[e]]',
                                                      '[[m]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => '[Lex-15],[[\\:]],[[l]],[[e]],[[x]],[[e]],[[m]],[[e]]'
                                         },
                                         {
                                           'lhs' => '[Lex-16]',
                                           'rhs' => [
                                                      '[[e]]',
                                                      '[[v]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-16],[[e]],[[v]],[[e]],[[n]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-17]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-17],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-18]',
                                           'rhs' => [
                                                      '[[c]]',
                                                      '[[o]]',
                                                      '[[m]]',
                                                      '[[p]]',
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[t]]',
                                                      '[[e]]',
                                                      '[[d]]'
                                                    ],
                                           'xbnfid' => '[Lex-18],[[c]],[[o]],[[m]],[[p]],[[l]],[[e]],[[t]],[[e]],[[d]]'
                                         },
                                         {
                                           'lhs' => '[Lex-19]',
                                           'rhs' => [
                                                      '[[e]]',
                                                      '[[v]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-19],[[e]],[[v]],[[e]],[[n]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-1]',
                                           'rhs' => [
                                                      '[[\\{]]'
                                                    ],
                                           'xbnfid' => '[Lex-1],[[\\{]]'
                                         },
                                         {
                                           'lhs' => '[Lex-20]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-20],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-21]',
                                           'rhs' => [
                                                      '[[n]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[d]]'
                                                    ],
                                           'xbnfid' => '[Lex-21],[[n]],[[u]],[[l]],[[l]],[[e]],[[d]]'
                                         },
                                         {
                                           'lhs' => '[Lex-22]',
                                           'rhs' => [
                                                      '[[e]]',
                                                      '[[v]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-22],[[e]],[[v]],[[e]],[[n]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-23]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-23],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-24]',
                                           'rhs' => [
                                                      '[[p]]',
                                                      '[[r]]',
                                                      '[[e]]',
                                                      '[[d]]',
                                                      '[[i]]',
                                                      '[[c]]',
                                                      '[[t]]',
                                                      '[[e]]',
                                                      '[[d]]'
                                                    ],
                                           'xbnfid' => '[Lex-24],[[p]],[[r]],[[e]],[[d]],[[i]],[[c]],[[t]],[[e]],[[d]]'
                                         },
                                         {
                                           'lhs' => '[Lex-25]',
                                           'rhs' => [
                                                      '[[i]]',
                                                      '[[n]]',
                                                      '[[a]]',
                                                      '[[c]]',
                                                      '[[c]]',
                                                      '[[e]]',
                                                      '[[s]]',
                                                      '[[s]]',
                                                      '[[i]]',
                                                      '[[b]]',
                                                      '[[l]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => '[Lex-25],[[i]],[[n]],[[a]],[[c]],[[c]],[[e]],[[s]],[[s]],[[i]],[[b]],[[l]],[[e]]'
                                         },
                                         {
                                           'lhs' => '[Lex-26]',
                                           'rhs' => [
                                                      '[[i]]',
                                                      '[[s]]'
                                                    ],
                                           'xbnfid' => '[Lex-26],[[i]],[[s]]'
                                         },
                                         {
                                           'lhs' => '[Lex-27]',
                                           'rhs' => [
                                                      '[[b]]',
                                                      '[[y]]'
                                                    ],
                                           'xbnfid' => '[Lex-27],[[b]],[[y]]'
                                         },
                                         {
                                           'lhs' => '[Lex-28]',
                                           'rhs' => [
                                                      '[[d]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[a]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-28],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-29]',
                                           'rhs' => [
                                                      '[[w]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[n]]'
                                                    ],
                                           'xbnfid' => '[Lex-29],[[w]],[[a]],[[r]],[[n]]'
                                         },
                                         {
                                           'lhs' => '[Lex-2]',
                                           'rhs' => [
                                                      '[[\\}]]'
                                                    ],
                                           'xbnfid' => '[Lex-2],[[\\}]]'
                                         },
                                         {
                                           'lhs' => '[Lex-30]',
                                           'rhs' => [
                                                      '[[o]]',
                                                      '[[k]]'
                                                    ],
                                           'xbnfid' => '[Lex-30],[[o]],[[k]]'
                                         },
                                         {
                                           'lhs' => '[Lex-31]',
                                           'rhs' => [
                                                      '[[f]]',
                                                      '[[a]]',
                                                      '[[t]]',
                                                      '[[a]]',
                                                      '[[l]]'
                                                    ],
                                           'xbnfid' => '[Lex-31],[[f]],[[a]],[[t]],[[a]],[[l]]'
                                         },
                                         {
                                           'lhs' => '[Lex-32]',
                                           'rhs' => [
                                                      '[[\\,]]'
                                                    ],
                                           'xbnfid' => '[Lex-32],[[\\,]]'
                                         },
                                         {
                                           'lhs' => '[Lex-33]',
                                           'rhs' => [
                                                      '[[a]]',
                                                      '[[c]]',
                                                      '[[t]]',
                                                      '[[i]]',
                                                      '[[o]]',
                                                      '[[n]]'
                                                    ],
                                           'xbnfid' => '[Lex-33],[[a]],[[c]],[[t]],[[i]],[[o]],[[n]]'
                                         },
                                         {
                                           'lhs' => '[Lex-34]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-34],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-35]',
                                           'rhs' => [
                                                      '[[a]]',
                                                      '[[s]]',
                                                      '[[s]]',
                                                      '[[o]]',
                                                      '[[c]]'
                                                    ],
                                           'xbnfid' => '[Lex-35],[[a]],[[s]],[[s]],[[o]],[[c]]'
                                         },
                                         {
                                           'lhs' => '[Lex-36]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-36],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-37]',
                                           'rhs' => [
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-37],[[l]],[[e]],[[f]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-38]',
                                           'rhs' => [
                                                      '[[a]]',
                                                      '[[s]]',
                                                      '[[s]]',
                                                      '[[o]]',
                                                      '[[c]]'
                                                    ],
                                           'xbnfid' => '[Lex-38],[[a]],[[s]],[[s]],[[o]],[[c]]'
                                         },
                                         {
                                           'lhs' => '[Lex-39]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-39],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-3]',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[s]]',
                                                      '[[t]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-3],[[\\:]],[[s]],[[t]],[[a]],[[r]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-40]',
                                           'rhs' => [
                                                      '[[r]]',
                                                      '[[i]]',
                                                      '[[g]]',
                                                      '[[h]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-40],[[r]],[[i]],[[g]],[[h]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-41]',
                                           'rhs' => [
                                                      '[[a]]',
                                                      '[[s]]',
                                                      '[[s]]',
                                                      '[[o]]',
                                                      '[[c]]'
                                                    ],
                                           'xbnfid' => '[Lex-41],[[a]],[[s]],[[s]],[[o]],[[c]]'
                                         },
                                         {
                                           'lhs' => '[Lex-42]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-42],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-43]',
                                           'rhs' => [
                                                      '[[g]]',
                                                      '[[r]]',
                                                      '[[o]]',
                                                      '[[u]]',
                                                      '[[p]]'
                                                    ],
                                           'xbnfid' => '[Lex-43],[[g]],[[r]],[[o]],[[u]],[[p]]'
                                         },
                                         {
                                           'lhs' => '[Lex-44]',
                                           'rhs' => [
                                                      '[[s]]',
                                                      '[[e]]',
                                                      '[[p]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[a]]',
                                                      '[[t]]',
                                                      '[[o]]',
                                                      '[[r]]'
                                                    ],
                                           'xbnfid' => '[Lex-44],[[s]],[[e]],[[p]],[[a]],[[r]],[[a]],[[t]],[[o]],[[r]]'
                                         },
                                         {
                                           'lhs' => '[Lex-45]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-45],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-46]',
                                           'rhs' => [
                                                      '[[p]]',
                                                      '[[r]]',
                                                      '[[o]]',
                                                      '[[p]]',
                                                      '[[e]]',
                                                      '[[r]]'
                                                    ],
                                           'xbnfid' => '[Lex-46],[[p]],[[r]],[[o]],[[p]],[[e]],[[r]]'
                                         },
                                         {
                                           'lhs' => '[Lex-47]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-47],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-48]',
                                           'rhs' => [
                                                      '[[r]]',
                                                      '[[a]]',
                                                      '[[n]]',
                                                      '[[k]]'
                                                    ],
                                           'xbnfid' => '[Lex-48],[[r]],[[a]],[[n]],[[k]]'
                                         },
                                         {
                                           'lhs' => '[Lex-49]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-49],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-4]',
                                           'rhs' => [
                                                      '[[s]]',
                                                      '[[t]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-4],[[s]],[[t]],[[a]],[[r]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-50]',
                                           'rhs' => [
                                                      '[[n]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[l]]',
                                                      '[[\\-]]',
                                                      '[[r]]',
                                                      '[[a]]',
                                                      '[[n]]',
                                                      '[[k]]',
                                                      '[[i]]',
                                                      '[[n]]',
                                                      '[[g]]'
                                                    ],
                                           'xbnfid' => '[Lex-50],[[n]],[[u]],[[l]],[[l]],[[\\-]],[[r]],[[a]],[[n]],[[k]],[[i]],[[n]],[[g]]'
                                         },
                                         {
                                           'lhs' => '[Lex-51]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-51],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-52]',
                                           'rhs' => [
                                                      '[[n]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[l]]'
                                                    ],
                                           'xbnfid' => '[Lex-52],[[n]],[[u]],[[l]],[[l]]'
                                         },
                                         {
                                           'lhs' => '[Lex-53]',
                                           'rhs' => [
                                                      '[[r]]',
                                                      '[[a]]',
                                                      '[[n]]',
                                                      '[[k]]'
                                                    ],
                                           'xbnfid' => '[Lex-53],[[r]],[[a]],[[n]],[[k]]'
                                         },
                                         {
                                           'lhs' => '[Lex-54]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-54],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-55]',
                                           'rhs' => [
                                                      '[[l]]',
                                                      '[[o]]',
                                                      '[[w]]'
                                                    ],
                                           'xbnfid' => '[Lex-55],[[l]],[[o]],[[w]]'
                                         },
                                         {
                                           'lhs' => '[Lex-56]',
                                           'rhs' => [
                                                      '[[h]]',
                                                      '[[i]]',
                                                      '[[g]]',
                                                      '[[h]]'
                                                    ],
                                           'xbnfid' => '[Lex-56],[[h]],[[i]],[[g]],[[h]]'
                                         },
                                         {
                                           'lhs' => '[Lex-57]',
                                           'rhs' => [
                                                      '[[p]]',
                                                      '[[r]]',
                                                      '[[i]]',
                                                      '[[o]]',
                                                      '[[r]]',
                                                      '[[i]]',
                                                      '[[t]]',
                                                      '[[y]]'
                                                    ],
                                           'xbnfid' => '[Lex-57],[[p]],[[r]],[[i]],[[o]],[[r]],[[i]],[[t]],[[y]]'
                                         },
                                         {
                                           'lhs' => '[Lex-58]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-58],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-59]',
                                           'rhs' => [
                                                      '[[p]]',
                                                      '[[a]]',
                                                      '[[u]]',
                                                      '[[s]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => '[Lex-59],[[p]],[[a]],[[u]],[[s]],[[e]]'
                                         },
                                         {
                                           'lhs' => '[Lex-5]',
                                           'rhs' => [
                                                      '[[s]]',
                                                      '[[y]]',
                                                      '[[m]]',
                                                      '[[b]]',
                                                      '[[o]]',
                                                      '[[l]]'
                                                    ],
                                           'xbnfid' => '[Lex-5],[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]'
                                         },
                                         {
                                           'lhs' => '[Lex-60]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-60],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-61]',
                                           'rhs' => [
                                                      '[[e]]',
                                                      '[[v]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-61],[[e]],[[v]],[[e]],[[n]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-62]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-62],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-63]',
                                           'rhs' => [
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => '[Lex-63],[[\\=]]'
                                         },
                                         {
                                           'lhs' => '[Lex-64]',
                                           'rhs' => [
                                                      '[[o]]',
                                                      '[[n]]'
                                                    ],
                                           'xbnfid' => '[Lex-64],[[o]],[[n]]'
                                         },
                                         {
                                           'lhs' => '[Lex-65]',
                                           'rhs' => [
                                                      '[[o]]',
                                                      '[[f]]',
                                                      '[[f]]'
                                                    ],
                                           'xbnfid' => '[Lex-65],[[o]],[[f]],[[f]]'
                                         },
                                         {
                                           'lhs' => '[Lex-66]',
                                           'rhs' => [
                                                      '[[b]]',
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[s]]',
                                                      '[[s]]'
                                                    ],
                                           'xbnfid' => '[Lex-66],[[b]],[[l]],[[e]],[[s]],[[s]]'
                                         },
                                         {
                                           'lhs' => '[Lex-67]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-67],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-68]',
                                           'rhs' => [
                                                      '[[n]]',
                                                      '[[a]]',
                                                      '[[m]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => '[Lex-68],[[n]],[[a]],[[m]],[[e]]'
                                         },
                                         {
                                           'lhs' => '[Lex-69]',
                                           'rhs' => [
                                                      '[[\\=]]',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => '[Lex-69],[[\\=]],[[\\>]]'
                                         },
                                         {
                                           'lhs' => '[Lex-6]',
                                           'rhs' => [
                                                      '[[i]]',
                                                      '[[s]]'
                                                    ],
                                           'xbnfid' => '[Lex-6],[[i]],[[s]]'
                                         },
                                         {
                                           'lhs' => '[Lex-70]',
                                           'rhs' => [
                                                      '[[\\(]]'
                                                    ],
                                           'xbnfid' => '[Lex-70],[[\\(]]'
                                         },
                                         {
                                           'lhs' => '[Lex-71]',
                                           'rhs' => [
                                                      '[[\\)]]'
                                                    ],
                                           'xbnfid' => '[Lex-71],[[\\)]]'
                                         },
                                         {
                                           'lhs' => '[Lex-72]',
                                           'rhs' => [
                                                      '[[\\*]]'
                                                    ],
                                           'xbnfid' => '[Lex-72],[[\\*]]'
                                         },
                                         {
                                           'lhs' => '[Lex-73]',
                                           'rhs' => [
                                                      '[[\\+]]'
                                                    ],
                                           'xbnfid' => '[Lex-73],[[\\+]]'
                                         },
                                         {
                                           'lhs' => '[Lex-7]',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[d]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[a]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-7],[[\\:]],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]'
                                         },
                                         {
                                           'lhs' => '[Lex-8]',
                                           'rhs' => [
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[x]]',
                                                      '[[e]]',
                                                      '[[m]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => '[Lex-8],[[l]],[[e]],[[x]],[[e]],[[m]],[[e]]'
                                         },
                                         {
                                           'lhs' => '[Lex-9]',
                                           'rhs' => [
                                                      '[[d]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[a]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => '[Lex-9],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]'
                                         },
                                         {
                                           'lhs' => 'array descriptor',
                                           'rhs' => [
                                                      'array descriptor left bracket',
                                                      'result item descriptor list',
                                                      'array descriptor right bracket'
                                                    ],
                                           'xbnfid' => 'array descriptor,array descriptor left bracket,result item descriptor list,array descriptor right bracket'
                                         },
                                         {
                                           'lhs' => 'array descriptor left bracket',
                                           'rhs' => [
                                                      '[[\\[]]'
                                                    ],
                                           'xbnfid' => 'array descriptor left bracket,[[\\[]]'
                                         },
                                         {
                                           'lhs' => 'array descriptor left bracket',
                                           'rhs' => [
                                                      '[[\\[]]',
                                                      'whitespace'
                                                    ],
                                           'xbnfid' => 'array descriptor left bracket,[[\\[]],whitespace'
                                         },
                                         {
                                           'lhs' => 'array descriptor right bracket',
                                           'rhs' => [
                                                      '[[\\]]]'
                                                    ],
                                           'xbnfid' => 'array descriptor right bracket,[[\\]]]'
                                         },
                                         {
                                           'lhs' => 'array descriptor right bracket',
                                           'rhs' => [
                                                      'whitespace',
                                                      '[[\\]]]'
                                                    ],
                                           'xbnfid' => 'array descriptor right bracket,whitespace,[[\\]]]'
                                         },
                                         {
                                           'lhs' => 'bare name',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\w]]'
                                                    ],
                                           'xbnfid' => 'bare name,[[\\w]]'
                                         },
                                         {
                                           'lhs' => 'before or after',
                                           'rhs' => [
                                                      '[[a]]',
                                                      '[[f]]',
                                                      '[[t]]',
                                                      '[[e]]',
                                                      '[[r]]'
                                                    ],
                                           'xbnfid' => 'before or after,[[a]],[[f]],[[t]],[[e]],[[r]]'
                                         },
                                         {
                                           'lhs' => 'before or after',
                                           'rhs' => [
                                                      '[[b]]',
                                                      '[[e]]',
                                                      '[[f]]',
                                                      '[[o]]',
                                                      '[[r]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => 'before or after,[[b]],[[e]],[[f]],[[o]],[[r]],[[e]]'
                                         },
                                         {
                                           'lhs' => 'boolean',
                                           'rhs' => [
                                                      '[[01]]'
                                                    ],
                                           'xbnfid' => 'boolean,[[01]]'
                                         },
                                         {
                                           'lhs' => 'bracketed name',
                                           'rhs' => [
                                                      '[[\\<]]',
                                                      'bracketed name string',
                                                      '[[\\>]]'
                                                    ],
                                           'xbnfid' => 'bracketed name,[[\\<]],bracketed name string,[[\\>]]'
                                         },
                                         {
                                           'lhs' => 'bracketed name string',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\s\\w]]'
                                                    ],
                                           'xbnfid' => 'bracketed name string,[[\\s\\w]]'
                                         },
                                         {
                                           'lhs' => 'cc element',
                                           'rhs' => [
                                                      'escaped cc character'
                                                    ],
                                           'xbnfid' => 'cc element,escaped cc character'
                                         },
                                         {
                                           'lhs' => 'cc element',
                                           'rhs' => [
                                                      'negated posix char class'
                                                    ],
                                           'xbnfid' => 'cc element,negated posix char class'
                                         },
                                         {
                                           'lhs' => 'cc element',
                                           'rhs' => [
                                                      'posix char class'
                                                    ],
                                           'xbnfid' => 'cc element,posix char class'
                                         },
                                         {
                                           'lhs' => 'cc element',
                                           'rhs' => [
                                                      'safe cc character'
                                                    ],
                                           'xbnfid' => 'cc element,safe cc character'
                                         },
                                         {
                                           'lhs' => 'cc elements',
                                           'min' => 1,
                                           'rhs' => [
                                                      'cc element'
                                                    ],
                                           'xbnfid' => 'cc elements,cc element'
                                         },
                                         {
                                           'lhs' => 'character class',
                                           'rhs' => [
                                                      '[[\\[]]',
                                                      'cc elements',
                                                      '[[\\]]]',
                                                      'character class modifiers'
                                                    ],
                                           'xbnfid' => 'character class,[[\\[]],cc elements,[[\\]]],character class modifiers'
                                         },
                                         {
                                           'lhs' => 'character class modifier',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[i]]'
                                                    ],
                                           'xbnfid' => 'character class modifier,[[\\:]],[[i]]'
                                         },
                                         {
                                           'lhs' => 'character class modifier',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[i]]',
                                                      '[[c]]'
                                                    ],
                                           'xbnfid' => 'character class modifier,[[\\:]],[[i]],[[c]]'
                                         },
                                         {
                                           'lhs' => 'character class modifiers',
                                           'min' => 0,
                                           'rhs' => [
                                                      'character class modifier'
                                                    ],
                                           'xbnfid' => 'character class modifiers,character class modifier'
                                         },
                                         {
                                           'lhs' => 'double colon',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[\\:]]'
                                                    ],
                                           'xbnfid' => 'double colon,[[\\:]],[[\\:]]'
                                         },
                                         {
                                           'lhs' => 'escaped cc character',
                                           'rhs' => [
                                                      '[[\\\\]]',
                                                      'horizontal character'
                                                    ],
                                           'xbnfid' => 'escaped cc character,[[\\\\]],horizontal character'
                                         },
                                         {
                                           'lhs' => 'hash comment',
                                           'rhs' => [
                                                      'terminated hash comment'
                                                    ],
                                           'xbnfid' => 'hash comment,terminated hash comment'
                                         },
                                         {
                                           'lhs' => 'hash comment',
                                           'rhs' => [
                                                      'unterminated final hash comment'
                                                    ],
                                           'xbnfid' => 'hash comment,unterminated final hash comment'
                                         },
                                         {
                                           'lhs' => 'hash comment body',
                                           'min' => 0,
                                           'rhs' => [
                                                      'hash comment char'
                                                    ],
                                           'xbnfid' => 'hash comment body,hash comment char'
                                         },
                                         {
                                           'lhs' => 'hash comment char',
                                           'rhs' => [
                                                      '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                    ],
                                           'xbnfid' => 'hash comment char,[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                         },
                                         {
                                           'lhs' => 'horizontal character',
                                           'rhs' => [
                                                      '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                    ],
                                           'xbnfid' => 'horizontal character,[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                         },
                                         {
                                           'lhs' => 'integer',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\d]]'
                                                    ],
                                           'xbnfid' => 'integer,[[\\d]]'
                                         },
                                         {
                                           'lhs' => 'negated posix char class',
                                           'rhs' => [
                                                      '[[\\[]]',
                                                      '[[\\:]]',
                                                      '[[\\^]]',
                                                      'posix char class name',
                                                      '[[\\:]]',
                                                      '[[\\]]]'
                                                    ],
                                           'xbnfid' => 'negated posix char class,[[\\[]],[[\\:]],[[\\^]],posix char class name,[[\\:]],[[\\]]]'
                                         },
                                         {
                                           'lhs' => 'one or more word characters',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\w]]'
                                                    ],
                                           'xbnfid' => 'one or more word characters,[[\\w]]'
                                         },
                                         {
                                           'lhs' => 'op declare bnf',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[\\:]]',
                                                      '[[\\=]]'
                                                    ],
                                           'xbnfid' => 'op declare bnf,[[\\:]],[[\\:]],[[\\=]]'
                                         },
                                         {
                                           'lhs' => 'op declare match',
                                           'rhs' => [
                                                      '[[\\~]]'
                                                    ],
                                           'xbnfid' => 'op declare match,[[\\~]]'
                                         },
                                         {
                                           'lhs' => 'op equal priority',
                                           'rhs' => [
                                                      '[[\\|]]'
                                                    ],
                                           'xbnfid' => 'op equal priority,[[\\|]]'
                                         },
                                         {
                                           'lhs' => 'op loosen',
                                           'rhs' => [
                                                      '[[\\|]]',
                                                      '[[\\|]]'
                                                    ],
                                           'xbnfid' => 'op loosen,[[\\|]],[[\\|]]'
                                         },
                                         {
                                           'lhs' => 'posix char class',
                                           'rhs' => [
                                                      '[[\\[]]',
                                                      '[[\\:]]',
                                                      'posix char class name',
                                                      '[[\\:]]',
                                                      '[[\\]]]'
                                                    ],
                                           'xbnfid' => 'posix char class,[[\\[]],[[\\:]],posix char class name,[[\\:]],[[\\]]]'
                                         },
                                         {
                                           'lhs' => 'posix char class name',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[[:alnum:]]]'
                                                    ],
                                           'xbnfid' => 'posix char class name,[[[:alnum:]]]'
                                         },
                                         {
                                           'lhs' => 'reserved action name',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[\\:]]',
                                                      'one or more word characters'
                                                    ],
                                           'xbnfid' => 'reserved action name,[[\\:]],[[\\:]],one or more word characters'
                                         },
                                         {
                                           'lhs' => 'reserved blessing name',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[\\:]]',
                                                      'one or more word characters'
                                                    ],
                                           'xbnfid' => 'reserved blessing name,[[\\:]],[[\\:]],one or more word characters'
                                         },
                                         {
                                           'lhs' => 'reserved event name',
                                           'rhs' => [
                                                      '[[\\:]]',
                                                      '[[s]]',
                                                      '[[y]]',
                                                      '[[m]]',
                                                      '[[b]]',
                                                      '[[o]]',
                                                      '[[l]]'
                                                    ],
                                           'xbnfid' => 'reserved event name,[[\\:]],[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[l]]',
                                                      '[[h]]',
                                                      '[[s]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[l]],[[h]],[[s]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[n]]',
                                                      '[[a]]',
                                                      '[[m]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[n]],[[a]],[[m]],[[e]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[r]]',
                                                      '[[u]]',
                                                      '[[l]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[r]],[[u]],[[l]],[[e]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[s]]',
                                                      '[[t]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[s]],[[t]],[[a]],[[r]],[[t]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[v]]',
                                                      '[[a]]',
                                                      '[[l]]',
                                                      '[[u]]',
                                                      '[[e]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[v]],[[a]],[[l]],[[u]],[[e]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[g]]',
                                                      '[[t]]',
                                                      '[[h]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[l]],[[e]],[[n]],[[g]],[[t]],[[h]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[s]]',
                                                      '[[y]]',
                                                      '[[m]]',
                                                      '[[b]]',
                                                      '[[o]]',
                                                      '[[l]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[v]]',
                                                      '[[a]]',
                                                      '[[l]]',
                                                      '[[u]]',
                                                      '[[e]]',
                                                      '[[s]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[v]],[[a]],[[l]],[[u]],[[e]],[[s]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[g]]',
                                                      '[[1]]',
                                                      '[[s]]',
                                                      '[[t]]',
                                                      '[[a]]',
                                                      '[[r]]',
                                                      '[[t]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[g]],[[1]],[[s]],[[t]],[[a]],[[r]],[[t]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor',
                                           'rhs' => [
                                                      '[[g]]',
                                                      '[[1]]',
                                                      '[[l]]',
                                                      '[[e]]',
                                                      '[[n]]',
                                                      '[[g]]',
                                                      '[[t]]',
                                                      '[[h]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor,[[g]],[[1]],[[l]],[[e]],[[n]],[[g]],[[t]],[[h]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor list',
                                           'min' => 0,
                                           'rhs' => [
                                                      'result item descriptor'
                                                    ],
                                           'separator' => 'result item descriptor separator',
                                           'xbnfid' => 'result item descriptor list,result item descriptor'
                                         },
                                         {
                                           'lhs' => 'result item descriptor separator',
                                           'rhs' => [
                                                      '[[,]]'
                                                    ],
                                           'xbnfid' => 'result item descriptor separator,[[,]]'
                                         },
                                         {
                                           'lhs' => 'result item descriptor separator',
                                           'rhs' => [
                                                      '[[,]]',
                                                      'whitespace'
                                                    ],
                                           'xbnfid' => 'result item descriptor separator,[[,]],whitespace'
                                         },
                                         {
                                           'lhs' => 'safe cc character',
                                           'rhs' => [
                                                      '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                    ],
                                           'xbnfid' => 'safe cc character,[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                         },
                                         {
                                           'lhs' => 'sign',
                                           'rhs' => [
                                                      '[[+-]]'
                                                    ],
                                           'xbnfid' => 'sign,[[+-]]'
                                         },
                                         {
                                           'lhs' => 'signed integer',
                                           'rhs' => [
                                                      'integer'
                                                    ],
                                           'xbnfid' => 'signed integer,integer'
                                         },
                                         {
                                           'lhs' => 'signed integer',
                                           'rhs' => [
                                                      'sign',
                                                      'integer'
                                                    ],
                                           'xbnfid' => 'signed integer,sign,integer'
                                         },
                                         {
                                           'lhs' => 'single quoted name',
                                           'rhs' => [
                                                      '[[\']]',
                                                      'string without single quote or vertical space',
                                                      '[[\']]'
                                                    ],
                                           'xbnfid' => 'single quoted name,[[\']],string without single quote or vertical space,[[\']]'
                                         },
                                         {
                                           'lhs' => 'single quoted string',
                                           'rhs' => [
                                                      '[[\']]',
                                                      'string without single quote or vertical space',
                                                      '[[\']]',
                                                      'character class modifiers'
                                                    ],
                                           'xbnfid' => 'single quoted string,[[\']],string without single quote or vertical space,[[\']],character class modifiers'
                                         },
                                         {
                                           'lhs' => 'standard name',
                                           'rhs' => [
                                                      '[[a-zA-Z]]',
                                                      'zero or more word characters'
                                                    ],
                                           'xbnfid' => 'standard name,[[a-zA-Z]],zero or more word characters'
                                         },
                                         {
                                           'lhs' => 'string without single quote or vertical space',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                    ],
                                           'xbnfid' => 'string without single quote or vertical space,[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                         },
                                         {
                                           'lhs' => 'terminated hash comment',
                                           'rhs' => [
                                                      '[[\\#]]',
                                                      'hash comment body',
                                                      'vertical space char'
                                                    ],
                                           'xbnfid' => 'terminated hash comment,[[\\#]],hash comment body,vertical space char'
                                         },
                                         {
                                           'lhs' => 'unterminated final hash comment',
                                           'rhs' => [
                                                      '[[\\#]]',
                                                      'hash comment body'
                                                    ],
                                           'xbnfid' => 'unterminated final hash comment,[[\\#]],hash comment body'
                                         },
                                         {
                                           'lhs' => 'vertical space char',
                                           'rhs' => [
                                                      '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                    ],
                                           'xbnfid' => 'vertical space char,[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                         },
                                         {
                                           'lhs' => 'whitespace',
                                           'min' => 1,
                                           'rhs' => [
                                                      '[[\\s]]'
                                                    ],
                                           'xbnfid' => 'whitespace,[[\\s]]'
                                         },
                                         {
                                           'lhs' => 'zero or more word characters',
                                           'min' => 0,
                                           'rhs' => [
                                                      '[[\\w]]'
                                                    ],
                                           'xbnfid' => 'zero or more word characters,[[\\w]]'
                                         }
                                       ]
                             },
                  'start_lhs' => 'statements',
                  'symbols' => {
                                 'G1' => {
                                           'Perl name' => {
                                                            'wsyid' => 648,
                                                            'xsy' => 'Perl name'
                                                          },
                                           '[:start]' => {},
                                           '[Lex-0]' => {
                                                          'wsyid' => 1,
                                                          'xsy' => '[Lex-0]'
                                                        },
                                           '[Lex-10]' => {
                                                           'wsyid' => 64,
                                                           'xsy' => '[Lex-10]'
                                                         },
                                           '[Lex-11]' => {
                                                           'wsyid' => 73,
                                                           'xsy' => '[Lex-11]'
                                                         },
                                           '[Lex-12]' => {
                                                           'wsyid' => 82,
                                                           'xsy' => '[Lex-12]'
                                                         },
                                           '[Lex-13]' => {
                                                           'wsyid' => 85,
                                                           'xsy' => '[Lex-13]'
                                                         },
                                           '[Lex-14]' => {
                                                           'wsyid' => 95,
                                                           'xsy' => '[Lex-14]'
                                                         },
                                           '[Lex-15]' => {
                                                           'wsyid' => 104,
                                                           'xsy' => '[Lex-15]'
                                                         },
                                           '[Lex-16]' => {
                                                           'wsyid' => 111,
                                                           'xsy' => '[Lex-16]'
                                                         },
                                           '[Lex-17]' => {
                                                           'wsyid' => 114,
                                                           'xsy' => '[Lex-17]'
                                                         },
                                           '[Lex-18]' => {
                                                           'wsyid' => 125,
                                                           'xsy' => '[Lex-18]'
                                                         },
                                           '[Lex-19]' => {
                                                           'wsyid' => 132,
                                                           'xsy' => '[Lex-19]'
                                                         },
                                           '[Lex-1]' => {
                                                          'wsyid' => 4,
                                                          'xsy' => '[Lex-1]'
                                                        },
                                           '[Lex-20]' => {
                                                           'wsyid' => 135,
                                                           'xsy' => '[Lex-20]'
                                                         },
                                           '[Lex-21]' => {
                                                           'wsyid' => 143,
                                                           'xsy' => '[Lex-21]'
                                                         },
                                           '[Lex-22]' => {
                                                           'wsyid' => 150,
                                                           'xsy' => '[Lex-22]'
                                                         },
                                           '[Lex-23]' => {
                                                           'wsyid' => 153,
                                                           'xsy' => '[Lex-23]'
                                                         },
                                           '[Lex-24]' => {
                                                           'wsyid' => 164,
                                                           'xsy' => '[Lex-24]'
                                                         },
                                           '[Lex-25]' => {
                                                           'wsyid' => 178,
                                                           'xsy' => '[Lex-25]'
                                                         },
                                           '[Lex-26]' => {
                                                           'wsyid' => 182,
                                                           'xsy' => '[Lex-26]'
                                                         },
                                           '[Lex-27]' => {
                                                           'wsyid' => 186,
                                                           'xsy' => '[Lex-27]'
                                                         },
                                           '[Lex-28]' => {
                                                           'wsyid' => 195,
                                                           'xsy' => '[Lex-28]'
                                                         },
                                           '[Lex-29]' => {
                                                           'wsyid' => 201,
                                                           'xsy' => '[Lex-29]'
                                                         },
                                           '[Lex-2]' => {
                                                          'wsyid' => 7,
                                                          'xsy' => '[Lex-2]'
                                                        },
                                           '[Lex-30]' => {
                                                           'wsyid' => 205,
                                                           'xsy' => '[Lex-30]'
                                                         },
                                           '[Lex-31]' => {
                                                           'wsyid' => 212,
                                                           'xsy' => '[Lex-31]'
                                                         },
                                           '[Lex-32]' => {
                                                           'wsyid' => 215,
                                                           'xsy' => '[Lex-32]'
                                                         },
                                           '[Lex-33]' => {
                                                           'wsyid' => 223,
                                                           'xsy' => '[Lex-33]'
                                                         },
                                           '[Lex-34]' => {
                                                           'wsyid' => 227,
                                                           'xsy' => '[Lex-34]'
                                                         },
                                           '[Lex-35]' => {
                                                           'wsyid' => 234,
                                                           'xsy' => '[Lex-35]'
                                                         },
                                           '[Lex-36]' => {
                                                           'wsyid' => 238,
                                                           'xsy' => '[Lex-36]'
                                                         },
                                           '[Lex-37]' => {
                                                           'wsyid' => 244,
                                                           'xsy' => '[Lex-37]'
                                                         },
                                           '[Lex-38]' => {
                                                           'wsyid' => 251,
                                                           'xsy' => '[Lex-38]'
                                                         },
                                           '[Lex-39]' => {
                                                           'wsyid' => 255,
                                                           'xsy' => '[Lex-39]'
                                                         },
                                           '[Lex-3]' => {
                                                          'wsyid' => 15,
                                                          'xsy' => '[Lex-3]'
                                                        },
                                           '[Lex-40]' => {
                                                           'wsyid' => 262,
                                                           'xsy' => '[Lex-40]'
                                                         },
                                           '[Lex-41]' => {
                                                           'wsyid' => 269,
                                                           'xsy' => '[Lex-41]'
                                                         },
                                           '[Lex-42]' => {
                                                           'wsyid' => 273,
                                                           'xsy' => '[Lex-42]'
                                                         },
                                           '[Lex-43]' => {
                                                           'wsyid' => 280,
                                                           'xsy' => '[Lex-43]'
                                                         },
                                           '[Lex-44]' => {
                                                           'wsyid' => 291,
                                                           'xsy' => '[Lex-44]'
                                                         },
                                           '[Lex-45]' => {
                                                           'wsyid' => 295,
                                                           'xsy' => '[Lex-45]'
                                                         },
                                           '[Lex-46]' => {
                                                           'wsyid' => 303,
                                                           'xsy' => '[Lex-46]'
                                                         },
                                           '[Lex-47]' => {
                                                           'wsyid' => 307,
                                                           'xsy' => '[Lex-47]'
                                                         },
                                           '[Lex-48]' => {
                                                           'wsyid' => 313,
                                                           'xsy' => '[Lex-48]'
                                                         },
                                           '[Lex-49]' => {
                                                           'wsyid' => 317,
                                                           'xsy' => '[Lex-49]'
                                                         },
                                           '[Lex-4]' => {
                                                          'wsyid' => 22,
                                                          'xsy' => '[Lex-4]'
                                                        },
                                           '[Lex-50]' => {
                                                           'wsyid' => 331,
                                                           'xsy' => '[Lex-50]'
                                                         },
                                           '[Lex-51]' => {
                                                           'wsyid' => 335,
                                                           'xsy' => '[Lex-51]'
                                                         },
                                           '[Lex-52]' => {
                                                           'wsyid' => 341,
                                                           'xsy' => '[Lex-52]'
                                                         },
                                           '[Lex-53]' => {
                                                           'wsyid' => 347,
                                                           'xsy' => '[Lex-53]'
                                                         },
                                           '[Lex-54]' => {
                                                           'wsyid' => 351,
                                                           'xsy' => '[Lex-54]'
                                                         },
                                           '[Lex-55]' => {
                                                           'wsyid' => 356,
                                                           'xsy' => '[Lex-55]'
                                                         },
                                           '[Lex-56]' => {
                                                           'wsyid' => 362,
                                                           'xsy' => '[Lex-56]'
                                                         },
                                           '[Lex-57]' => {
                                                           'wsyid' => 372,
                                                           'xsy' => '[Lex-57]'
                                                         },
                                           '[Lex-58]' => {
                                                           'wsyid' => 376,
                                                           'xsy' => '[Lex-58]'
                                                         },
                                           '[Lex-59]' => {
                                                           'wsyid' => 383,
                                                           'xsy' => '[Lex-59]'
                                                         },
                                           '[Lex-5]' => {
                                                          'wsyid' => 30,
                                                          'xsy' => '[Lex-5]'
                                                        },
                                           '[Lex-60]' => {
                                                           'wsyid' => 387,
                                                           'xsy' => '[Lex-60]'
                                                         },
                                           '[Lex-61]' => {
                                                           'wsyid' => 394,
                                                           'xsy' => '[Lex-61]'
                                                         },
                                           '[Lex-62]' => {
                                                           'wsyid' => 398,
                                                           'xsy' => '[Lex-62]'
                                                         },
                                           '[Lex-63]' => {
                                                           'wsyid' => 401,
                                                           'xsy' => '[Lex-63]'
                                                         },
                                           '[Lex-64]' => {
                                                           'wsyid' => 405,
                                                           'xsy' => '[Lex-64]'
                                                         },
                                           '[Lex-65]' => {
                                                           'wsyid' => 410,
                                                           'xsy' => '[Lex-65]'
                                                         },
                                           '[Lex-66]' => {
                                                           'wsyid' => 417,
                                                           'xsy' => '[Lex-66]'
                                                         },
                                           '[Lex-67]' => {
                                                           'wsyid' => 421,
                                                           'xsy' => '[Lex-67]'
                                                         },
                                           '[Lex-68]' => {
                                                           'wsyid' => 427,
                                                           'xsy' => '[Lex-68]'
                                                         },
                                           '[Lex-69]' => {
                                                           'wsyid' => 431,
                                                           'xsy' => '[Lex-69]'
                                                         },
                                           '[Lex-6]' => {
                                                          'wsyid' => 34,
                                                          'xsy' => '[Lex-6]'
                                                        },
                                           '[Lex-70]' => {
                                                           'wsyid' => 441,
                                                           'xsy' => '[Lex-70]'
                                                         },
                                           '[Lex-71]' => {
                                                           'wsyid' => 444,
                                                           'xsy' => '[Lex-71]'
                                                         },
                                           '[Lex-72]' => {
                                                           'wsyid' => 461,
                                                           'xsy' => '[Lex-72]'
                                                         },
                                           '[Lex-73]' => {
                                                           'wsyid' => 464,
                                                           'xsy' => '[Lex-73]'
                                                         },
                                           '[Lex-7]' => {
                                                          'wsyid' => 44,
                                                          'xsy' => '[Lex-7]'
                                                        },
                                           '[Lex-8]' => {
                                                          'wsyid' => 52,
                                                          'xsy' => '[Lex-8]'
                                                        },
                                           '[Lex-9]' => {
                                                          'wsyid' => 61,
                                                          'xsy' => '[Lex-9]'
                                                        },
                                           'action' => {
                                                         'wsyid' => 605,
                                                         'xsy' => 'action'
                                                       },
                                           'action name' => {
                                                              'wsyid' => 633,
                                                              'xsy' => 'action name'
                                                            },
                                           'adverb item' => {
                                                              'wsyid' => 603,
                                                              'xsy' => 'adverb item'
                                                            },
                                           'adverb list' => {
                                                              'wsyid' => 601,
                                                              'xsy' => 'adverb list'
                                                            },
                                           'adverb list items' => {
                                                                    'wsyid' => 602,
                                                                    'xsy' => 'adverb list items'
                                                                  },
                                           'alternative' => {
                                                              'wsyid' => 600,
                                                              'xsy' => 'alternative'
                                                            },
                                           'alternative name' => {
                                                                   'wsyid' => 622,
                                                                   'xsy' => 'alternative name'
                                                                 },
                                           'alternatives' => {
                                                               'wsyid' => 599,
                                                               'xsy' => 'alternatives'
                                                             },
                                           'array descriptor' => {
                                                                   'wsyid' => 650,
                                                                   'xsy' => 'array descriptor'
                                                                 },
                                           'bare name' => {
                                                            'wsyid' => 646,
                                                            'xsy' => 'bare name'
                                                          },
                                           'before or after' => {
                                                                  'wsyid' => 639,
                                                                  'xsy' => 'before or after'
                                                                },
                                           'blessing' => {
                                                           'wsyid' => 620,
                                                           'xsy' => 'blessing'
                                                         },
                                           'blessing name' => {
                                                                'wsyid' => 624,
                                                                'xsy' => 'blessing name'
                                                              },
                                           'boolean' => {
                                                          'wsyid' => 637,
                                                          'xsy' => 'boolean'
                                                        },
                                           'bracketed name' => {
                                                                 'wsyid' => 647,
                                                                 'xsy' => 'bracketed name'
                                                               },
                                           'character class' => {
                                                                  'wsyid' => 645,
                                                                  'xsy' => 'character class'
                                                                },
                                           'completion event declaration' => {
                                                                               'wsyid' => 592,
                                                                               'xsy' => 'completion event declaration'
                                                                             },
                                           'default rule' => {
                                                               'wsyid' => 584,
                                                               'xsy' => 'default rule'
                                                             },
                                           'discard default statement' => {
                                                                            'wsyid' => 586,
                                                                            'xsy' => 'discard default statement'
                                                                          },
                                           'discard rule' => {
                                                               'wsyid' => 590,
                                                               'xsy' => 'discard rule'
                                                             },
                                           'empty rule' => {
                                                             'wsyid' => 588,
                                                             'xsy' => 'empty rule'
                                                           },
                                           'event initialization' => {
                                                                       'wsyid' => 617,
                                                                       'xsy' => 'event initialization'
                                                                     },
                                           'event initializer' => {
                                                                    'wsyid' => 618,
                                                                    'xsy' => 'event initializer'
                                                                  },
                                           'event name' => {
                                                             'wsyid' => 623,
                                                             'xsy' => 'event name'
                                                           },
                                           'event specification' => {
                                                                      'wsyid' => 616,
                                                                      'xsy' => 'event specification'
                                                                    },
                                           'group association' => {
                                                                    'wsyid' => 608,
                                                                    'xsy' => 'group association'
                                                                  },
                                           'inaccessible statement' => {
                                                                         'wsyid' => 595,
                                                                         'xsy' => 'inaccessible statement'
                                                                       },
                                           'inaccessible treatment' => {
                                                                         'wsyid' => 596,
                                                                         'xsy' => 'inaccessible treatment'
                                                                       },
                                           'left association' => {
                                                                   'wsyid' => 606,
                                                                   'xsy' => 'left association'
                                                                 },
                                           'lexeme default statement' => {
                                                                           'wsyid' => 585,
                                                                           'xsy' => 'lexeme default statement'
                                                                         },
                                           'lexeme rule' => {
                                                              'wsyid' => 591,
                                                              'xsy' => 'lexeme rule'
                                                            },
                                           'lhs' => {
                                                      'wsyid' => 625,
                                                      'xsy' => 'lhs'
                                                    },
                                           'naming' => {
                                                         'wsyid' => 621,
                                                         'xsy' => 'naming'
                                                       },
                                           'null adverb' => {
                                                              'wsyid' => 604,
                                                              'xsy' => 'null adverb'
                                                            },
                                           'null ranking constant' => {
                                                                        'wsyid' => 613,
                                                                        'xsy' => 'null ranking constant'
                                                                      },
                                           'null ranking specification' => {
                                                                             'wsyid' => 612,
                                                                             'xsy' => 'null ranking specification'
                                                                           },
                                           'null statement' => {
                                                                 'wsyid' => 581,
                                                                 'xsy' => 'null statement'
                                                               },
                                           'nulled event declaration' => {
                                                                           'wsyid' => 593,
                                                                           'xsy' => 'nulled event declaration'
                                                                         },
                                           'on or off' => {
                                                            'wsyid' => 619,
                                                            'xsy' => 'on or off'
                                                          },
                                           'op declare' => {
                                                             'wsyid' => 597,
                                                             'xsy' => 'op declare'
                                                           },
                                           'op declare bnf' => {
                                                                 'wsyid' => 635,
                                                                 'xsy' => 'op declare bnf'
                                                               },
                                           'op declare match' => {
                                                                   'wsyid' => 636,
                                                                   'xsy' => 'op declare match'
                                                                 },
                                           'parenthesized rhs primary list' => {
                                                                                 'wsyid' => 628,
                                                                                 'xsy' => 'parenthesized rhs primary list'
                                                                               },
                                           'pause specification' => {
                                                                      'wsyid' => 615,
                                                                      'xsy' => 'pause specification'
                                                                    },
                                           'prediction event declaration' => {
                                                                               'wsyid' => 594,
                                                                               'xsy' => 'prediction event declaration'
                                                                             },
                                           'priorities' => {
                                                             'wsyid' => 598,
                                                             'xsy' => 'priorities'
                                                           },
                                           'priority rule' => {
                                                                'wsyid' => 587,
                                                                'xsy' => 'priority rule'
                                                              },
                                           'priority specification' => {
                                                                         'wsyid' => 614,
                                                                         'xsy' => 'priority specification'
                                                                       },
                                           'proper specification' => {
                                                                       'wsyid' => 610,
                                                                       'xsy' => 'proper specification'
                                                                     },
                                           'quantified rule' => {
                                                                  'wsyid' => 589,
                                                                  'xsy' => 'quantified rule'
                                                                },
                                           'quantifier' => {
                                                             'wsyid' => 634,
                                                             'xsy' => 'quantifier'
                                                           },
                                           'rank specification' => {
                                                                     'wsyid' => 611,
                                                                     'xsy' => 'rank specification'
                                                                   },
                                           'reserved action name' => {
                                                                       'wsyid' => 649,
                                                                       'xsy' => 'reserved action name'
                                                                     },
                                           'reserved blessing name' => {
                                                                         'wsyid' => 643,
                                                                         'xsy' => 'reserved blessing name'
                                                                       },
                                           'reserved event name' => {
                                                                      'wsyid' => 642,
                                                                      'xsy' => 'reserved event name'
                                                                    },
                                           'rhs' => {
                                                      'wsyid' => 626,
                                                      'xsy' => 'rhs'
                                                    },
                                           'rhs primary' => {
                                                              'wsyid' => 627,
                                                              'xsy' => 'rhs primary'
                                                            },
                                           'rhs primary list' => {
                                                                   'wsyid' => 629,
                                                                   'xsy' => 'rhs primary list'
                                                                 },
                                           'right association' => {
                                                                    'wsyid' => 607,
                                                                    'xsy' => 'right association'
                                                                  },
                                           'separator specification' => {
                                                                          'wsyid' => 609,
                                                                          'xsy' => 'separator specification'
                                                                        },
                                           'signed integer' => {
                                                                 'wsyid' => 638,
                                                                 'xsy' => 'signed integer'
                                                               },
                                           'single quoted name' => {
                                                                     'wsyid' => 641,
                                                                     'xsy' => 'single quoted name'
                                                                   },
                                           'single quoted string' => {
                                                                       'wsyid' => 644,
                                                                       'xsy' => 'single quoted string'
                                                                     },
                                           'single symbol' => {
                                                                'wsyid' => 630,
                                                                'xsy' => 'single symbol'
                                                              },
                                           'standard name' => {
                                                                'wsyid' => 640,
                                                                'xsy' => 'standard name'
                                                              },
                                           'start rule' => {
                                                             'wsyid' => 583,
                                                             'xsy' => 'start rule'
                                                           },
                                           'statement' => {
                                                            'wsyid' => 580,
                                                            'xsy' => 'statement'
                                                          },
                                           'statement group' => {
                                                                  'wsyid' => 582,
                                                                  'xsy' => 'statement group'
                                                                },
                                           'statements' => {
                                                             'wsyid' => 579,
                                                             'xsy' => 'statements'
                                                           },
                                           'symbol' => {
                                                         'wsyid' => 631,
                                                         'xsy' => 'symbol'
                                                       },
                                           'symbol name' => {
                                                              'wsyid' => 632,
                                                              'xsy' => 'symbol name'
                                                            }
                                         },
                                 'L0' => {
                                           'Perl identifier' => {
                                                                  'wsyid' => 672,
                                                                  'xsy' => 'Perl identifier'
                                                                },
                                           'Perl name' => {
                                                            'wsyid' => 674,
                                                            'xsy' => 'Perl name'
                                                          },
                                           '[:discard]' => {
                                                             'wsyid' => 448
                                                           },
                                           '[Lex-0]' => {
                                                          'wsyid' => 2,
                                                          'xsy' => '[Lex-0]'
                                                        },
                                           '[Lex-10]' => {
                                                           'wsyid' => 65,
                                                           'xsy' => '[Lex-10]'
                                                         },
                                           '[Lex-11]' => {
                                                           'wsyid' => 74,
                                                           'xsy' => '[Lex-11]'
                                                         },
                                           '[Lex-12]' => {
                                                           'wsyid' => 83,
                                                           'xsy' => '[Lex-12]'
                                                         },
                                           '[Lex-13]' => {
                                                           'wsyid' => 86,
                                                           'xsy' => '[Lex-13]'
                                                         },
                                           '[Lex-14]' => {
                                                           'wsyid' => 96,
                                                           'xsy' => '[Lex-14]'
                                                         },
                                           '[Lex-15]' => {
                                                           'wsyid' => 105,
                                                           'xsy' => '[Lex-15]'
                                                         },
                                           '[Lex-16]' => {
                                                           'wsyid' => 112,
                                                           'xsy' => '[Lex-16]'
                                                         },
                                           '[Lex-17]' => {
                                                           'wsyid' => 115,
                                                           'xsy' => '[Lex-17]'
                                                         },
                                           '[Lex-18]' => {
                                                           'wsyid' => 126,
                                                           'xsy' => '[Lex-18]'
                                                         },
                                           '[Lex-19]' => {
                                                           'wsyid' => 133,
                                                           'xsy' => '[Lex-19]'
                                                         },
                                           '[Lex-1]' => {
                                                          'wsyid' => 5,
                                                          'xsy' => '[Lex-1]'
                                                        },
                                           '[Lex-20]' => {
                                                           'wsyid' => 136,
                                                           'xsy' => '[Lex-20]'
                                                         },
                                           '[Lex-21]' => {
                                                           'wsyid' => 144,
                                                           'xsy' => '[Lex-21]'
                                                         },
                                           '[Lex-22]' => {
                                                           'wsyid' => 151,
                                                           'xsy' => '[Lex-22]'
                                                         },
                                           '[Lex-23]' => {
                                                           'wsyid' => 154,
                                                           'xsy' => '[Lex-23]'
                                                         },
                                           '[Lex-24]' => {
                                                           'wsyid' => 165,
                                                           'xsy' => '[Lex-24]'
                                                         },
                                           '[Lex-25]' => {
                                                           'wsyid' => 179,
                                                           'xsy' => '[Lex-25]'
                                                         },
                                           '[Lex-26]' => {
                                                           'wsyid' => 183,
                                                           'xsy' => '[Lex-26]'
                                                         },
                                           '[Lex-27]' => {
                                                           'wsyid' => 187,
                                                           'xsy' => '[Lex-27]'
                                                         },
                                           '[Lex-28]' => {
                                                           'wsyid' => 196,
                                                           'xsy' => '[Lex-28]'
                                                         },
                                           '[Lex-29]' => {
                                                           'wsyid' => 202,
                                                           'xsy' => '[Lex-29]'
                                                         },
                                           '[Lex-2]' => {
                                                          'wsyid' => 8,
                                                          'xsy' => '[Lex-2]'
                                                        },
                                           '[Lex-30]' => {
                                                           'wsyid' => 206,
                                                           'xsy' => '[Lex-30]'
                                                         },
                                           '[Lex-31]' => {
                                                           'wsyid' => 213,
                                                           'xsy' => '[Lex-31]'
                                                         },
                                           '[Lex-32]' => {
                                                           'wsyid' => 216,
                                                           'xsy' => '[Lex-32]'
                                                         },
                                           '[Lex-33]' => {
                                                           'wsyid' => 224,
                                                           'xsy' => '[Lex-33]'
                                                         },
                                           '[Lex-34]' => {
                                                           'wsyid' => 228,
                                                           'xsy' => '[Lex-34]'
                                                         },
                                           '[Lex-35]' => {
                                                           'wsyid' => 235,
                                                           'xsy' => '[Lex-35]'
                                                         },
                                           '[Lex-36]' => {
                                                           'wsyid' => 239,
                                                           'xsy' => '[Lex-36]'
                                                         },
                                           '[Lex-37]' => {
                                                           'wsyid' => 245,
                                                           'xsy' => '[Lex-37]'
                                                         },
                                           '[Lex-38]' => {
                                                           'wsyid' => 252,
                                                           'xsy' => '[Lex-38]'
                                                         },
                                           '[Lex-39]' => {
                                                           'wsyid' => 256,
                                                           'xsy' => '[Lex-39]'
                                                         },
                                           '[Lex-3]' => {
                                                          'wsyid' => 16,
                                                          'xsy' => '[Lex-3]'
                                                        },
                                           '[Lex-40]' => {
                                                           'wsyid' => 263,
                                                           'xsy' => '[Lex-40]'
                                                         },
                                           '[Lex-41]' => {
                                                           'wsyid' => 270,
                                                           'xsy' => '[Lex-41]'
                                                         },
                                           '[Lex-42]' => {
                                                           'wsyid' => 274,
                                                           'xsy' => '[Lex-42]'
                                                         },
                                           '[Lex-43]' => {
                                                           'wsyid' => 281,
                                                           'xsy' => '[Lex-43]'
                                                         },
                                           '[Lex-44]' => {
                                                           'wsyid' => 292,
                                                           'xsy' => '[Lex-44]'
                                                         },
                                           '[Lex-45]' => {
                                                           'wsyid' => 296,
                                                           'xsy' => '[Lex-45]'
                                                         },
                                           '[Lex-46]' => {
                                                           'wsyid' => 304,
                                                           'xsy' => '[Lex-46]'
                                                         },
                                           '[Lex-47]' => {
                                                           'wsyid' => 308,
                                                           'xsy' => '[Lex-47]'
                                                         },
                                           '[Lex-48]' => {
                                                           'wsyid' => 314,
                                                           'xsy' => '[Lex-48]'
                                                         },
                                           '[Lex-49]' => {
                                                           'wsyid' => 318,
                                                           'xsy' => '[Lex-49]'
                                                         },
                                           '[Lex-4]' => {
                                                          'wsyid' => 23,
                                                          'xsy' => '[Lex-4]'
                                                        },
                                           '[Lex-50]' => {
                                                           'wsyid' => 332,
                                                           'xsy' => '[Lex-50]'
                                                         },
                                           '[Lex-51]' => {
                                                           'wsyid' => 336,
                                                           'xsy' => '[Lex-51]'
                                                         },
                                           '[Lex-52]' => {
                                                           'wsyid' => 342,
                                                           'xsy' => '[Lex-52]'
                                                         },
                                           '[Lex-53]' => {
                                                           'wsyid' => 348,
                                                           'xsy' => '[Lex-53]'
                                                         },
                                           '[Lex-54]' => {
                                                           'wsyid' => 352,
                                                           'xsy' => '[Lex-54]'
                                                         },
                                           '[Lex-55]' => {
                                                           'wsyid' => 357,
                                                           'xsy' => '[Lex-55]'
                                                         },
                                           '[Lex-56]' => {
                                                           'wsyid' => 363,
                                                           'xsy' => '[Lex-56]'
                                                         },
                                           '[Lex-57]' => {
                                                           'wsyid' => 373,
                                                           'xsy' => '[Lex-57]'
                                                         },
                                           '[Lex-58]' => {
                                                           'wsyid' => 377,
                                                           'xsy' => '[Lex-58]'
                                                         },
                                           '[Lex-59]' => {
                                                           'wsyid' => 384,
                                                           'xsy' => '[Lex-59]'
                                                         },
                                           '[Lex-5]' => {
                                                          'wsyid' => 31,
                                                          'xsy' => '[Lex-5]'
                                                        },
                                           '[Lex-60]' => {
                                                           'wsyid' => 388,
                                                           'xsy' => '[Lex-60]'
                                                         },
                                           '[Lex-61]' => {
                                                           'wsyid' => 395,
                                                           'xsy' => '[Lex-61]'
                                                         },
                                           '[Lex-62]' => {
                                                           'wsyid' => 399,
                                                           'xsy' => '[Lex-62]'
                                                         },
                                           '[Lex-63]' => {
                                                           'wsyid' => 402,
                                                           'xsy' => '[Lex-63]'
                                                         },
                                           '[Lex-64]' => {
                                                           'wsyid' => 406,
                                                           'xsy' => '[Lex-64]'
                                                         },
                                           '[Lex-65]' => {
                                                           'wsyid' => 411,
                                                           'xsy' => '[Lex-65]'
                                                         },
                                           '[Lex-66]' => {
                                                           'wsyid' => 418,
                                                           'xsy' => '[Lex-66]'
                                                         },
                                           '[Lex-67]' => {
                                                           'wsyid' => 422,
                                                           'xsy' => '[Lex-67]'
                                                         },
                                           '[Lex-68]' => {
                                                           'wsyid' => 428,
                                                           'xsy' => '[Lex-68]'
                                                         },
                                           '[Lex-69]' => {
                                                           'wsyid' => 432,
                                                           'xsy' => '[Lex-69]'
                                                         },
                                           '[Lex-6]' => {
                                                          'wsyid' => 35,
                                                          'xsy' => '[Lex-6]'
                                                        },
                                           '[Lex-70]' => {
                                                           'wsyid' => 442,
                                                           'xsy' => '[Lex-70]'
                                                         },
                                           '[Lex-71]' => {
                                                           'wsyid' => 445,
                                                           'xsy' => '[Lex-71]'
                                                         },
                                           '[Lex-72]' => {
                                                           'wsyid' => 462,
                                                           'xsy' => '[Lex-72]'
                                                         },
                                           '[Lex-73]' => {
                                                           'wsyid' => 465,
                                                           'xsy' => '[Lex-73]'
                                                         },
                                           '[Lex-7]' => {
                                                          'wsyid' => 45,
                                                          'xsy' => '[Lex-7]'
                                                        },
                                           '[Lex-8]' => {
                                                          'wsyid' => 53,
                                                          'xsy' => '[Lex-8]'
                                                        },
                                           '[Lex-9]' => {
                                                          'wsyid' => 62,
                                                          'xsy' => '[Lex-9]'
                                                        },
                                           '[[\']]' => {
                                                         'wsyid' => 557,
                                                         'xsy' => '[[\']]'
                                                       },
                                           '[[+-]]' => {
                                                         'wsyid' => 477,
                                                         'xsy' => '[[+-]]'
                                                       },
                                           '[[,]]' => {
                                                        'wsyid' => 499,
                                                        'xsy' => '[[,]]'
                                                      },
                                           '[[01]]' => {
                                                         'wsyid' => 479,
                                                         'xsy' => '[[01]]'
                                                       },
                                           '[[1]]' => {
                                                        'wsyid' => 519,
                                                        'xsy' => '[[1]]'
                                                      },
                                           '[[[:alnum:]]]' => {
                                                                'wsyid' => 577,
                                                                'xsy' => '[[[:alnum:]]]'
                                                              },
                                           '[[\\#]]' => {
                                                          'wsyid' => 450,
                                                          'xsy' => '[[\\#]]'
                                                        },
                                           '[[\\(]]' => {
                                                          'wsyid' => 440,
                                                          'xsy' => '[[\\(]]'
                                                        },
                                           '[[\\)]]' => {
                                                          'wsyid' => 443,
                                                          'xsy' => '[[\\)]]'
                                                        },
                                           '[[\\*]]' => {
                                                          'wsyid' => 460,
                                                          'xsy' => '[[\\*]]'
                                                        },
                                           '[[\\+]]' => {
                                                          'wsyid' => 463,
                                                          'xsy' => '[[\\+]]'
                                                        },
                                           '[[\\,]]' => {
                                                          'wsyid' => 214,
                                                          'xsy' => '[[\\,]]'
                                                        },
                                           '[[\\-]]' => {
                                                          'wsyid' => 323,
                                                          'xsy' => '[[\\-]]'
                                                        },
                                           '[[\\:]]' => {
                                                          'wsyid' => 575,
                                                          'xsy' => '[[\\:]]'
                                                        },
                                           '[[\\;]]' => {
                                                          'wsyid' => 0,
                                                          'xsy' => '[[\\;]]'
                                                        },
                                           '[[\\<]]' => {
                                                          'wsyid' => 491,
                                                          'xsy' => '[[\\<]]'
                                                        },
                                           '[[\\=]]' => {
                                                          'wsyid' => 455,
                                                          'xsy' => '[[\\=]]'
                                                        },
                                           '[[\\>]]' => {
                                                          'wsyid' => 492,
                                                          'xsy' => '[[\\>]]'
                                                        },
                                           '[[\\[]]' => {
                                                          'wsyid' => 572,
                                                          'xsy' => '[[\\[]]'
                                                        },
                                           '[[\\\\]]' => {
                                                           'wsyid' => 562,
                                                           'xsy' => '[[\\\\]]'
                                                         },
                                           '[[\\]]]' => {
                                                          'wsyid' => 576,
                                                          'xsy' => '[[\\]]]'
                                                        },
                                           '[[\\^]]' => {
                                                          'wsyid' => 574,
                                                          'xsy' => '[[\\^]]'
                                                        },
                                           '[[\\d]]' => {
                                                          'wsyid' => 478,
                                                          'xsy' => '[[\\d]]'
                                                        },
                                           '[[\\s\\w]]' => {
                                                             'wsyid' => 493,
                                                             'xsy' => '[[\\s\\w]]'
                                                           },
                                           '[[\\s]]' => {
                                                          'wsyid' => 447,
                                                          'xsy' => '[[\\s]]'
                                                        },
                                           '[[\\w]]' => {
                                                          'wsyid' => 489,
                                                          'xsy' => '[[\\w]]'
                                                        },
                                           '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                                 'wsyid' => 451,
                                                                                                 'xsy' => '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                                                               },
                                           '[[\\{]]' => {
                                                          'wsyid' => 3,
                                                          'xsy' => '[[\\{]]'
                                                        },
                                           '[[\\|]]' => {
                                                          'wsyid' => 459,
                                                          'xsy' => '[[\\|]]'
                                                        },
                                           '[[\\}]]' => {
                                                          'wsyid' => 6,
                                                          'xsy' => '[[\\}]]'
                                                        },
                                           '[[\\~]]' => {
                                                          'wsyid' => 456,
                                                          'xsy' => '[[\\~]]'
                                                        },
                                           '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                                 'wsyid' => 558,
                                                                                                                 'xsy' => '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                                                                               },
                                           '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                                      'wsyid' => 561,
                                                                                                                      'xsy' => '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                                                                                    },
                                           '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                                  'wsyid' => 578,
                                                                                                  'xsy' => '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                                                                },
                                           '[[a-zA-Z]]' => {
                                                             'wsyid' => 490,
                                                             'xsy' => '[[a-zA-Z]]'
                                                           },
                                           '[[a]]' => {
                                                        'wsyid' => 549,
                                                        'xsy' => '[[a]]'
                                                      },
                                           '[[b]]' => {
                                                        'wsyid' => 536,
                                                        'xsy' => '[[b]]'
                                                      },
                                           '[[c]]' => {
                                                        'wsyid' => 565,
                                                        'xsy' => '[[c]]'
                                                      },
                                           '[[d]]' => {
                                                        'wsyid' => 188,
                                                        'xsy' => '[[d]]'
                                                      },
                                           '[[e]]' => {
                                                        'wsyid' => 552,
                                                        'xsy' => '[[e]]'
                                                      },
                                           '[[f]]' => {
                                                        'wsyid' => 473,
                                                        'xsy' => '[[f]]'
                                                      },
                                           '[[g]]' => {
                                                        'wsyid' => 523,
                                                        'xsy' => '[[g]]'
                                                      },
                                           '[[h]]' => {
                                                        'wsyid' => 531,
                                                        'xsy' => '[[h]]'
                                                      },
                                           '[[i]]' => {
                                                        'wsyid' => 567,
                                                        'xsy' => '[[i]]'
                                                      },
                                           '[[k]]' => {
                                                        'wsyid' => 346,
                                                        'xsy' => '[[k]]'
                                                      },
                                           '[[l]]' => {
                                                        'wsyid' => 550,
                                                        'xsy' => '[[l]]'
                                                      },
                                           '[[m]]' => {
                                                        'wsyid' => 535,
                                                        'xsy' => '[[m]]'
                                                      },
                                           '[[n]]' => {
                                                        'wsyid' => 526,
                                                        'xsy' => '[[n]]'
                                                      },
                                           '[[o]]' => {
                                                        'wsyid' => 537,
                                                        'xsy' => '[[o]]'
                                                      },
                                           '[[p]]' => {
                                                        'wsyid' => 378,
                                                        'xsy' => '[[p]]'
                                                      },
                                           '[[r]]' => {
                                                        'wsyid' => 539,
                                                        'xsy' => '[[r]]'
                                                      },
                                           '[[s]]' => {
                                                        'wsyid' => 553,
                                                        'xsy' => '[[s]]'
                                                      },
                                           '[[t]]' => {
                                                        'wsyid' => 524,
                                                        'xsy' => '[[t]]'
                                                      },
                                           '[[u]]' => {
                                                        'wsyid' => 551,
                                                        'xsy' => '[[u]]'
                                                      },
                                           '[[v]]' => {
                                                        'wsyid' => 548,
                                                        'xsy' => '[[v]]'
                                                      },
                                           '[[w]]' => {
                                                        'wsyid' => 355,
                                                        'xsy' => '[[w]]'
                                                      },
                                           '[[x]]' => {
                                                        'wsyid' => 100,
                                                        'xsy' => '[[x]]'
                                                      },
                                           '[[y]]' => {
                                                        'wsyid' => 534,
                                                        'xsy' => '[[y]]'
                                                      },
                                           'array descriptor' => {
                                                                   'wsyid' => 679,
                                                                   'xsy' => 'array descriptor'
                                                                 },
                                           'array descriptor left bracket' => {
                                                                                'wsyid' => 680,
                                                                                'xsy' => 'array descriptor left bracket'
                                                                              },
                                           'array descriptor right bracket' => {
                                                                                 'wsyid' => 681,
                                                                                 'xsy' => 'array descriptor right bracket'
                                                                               },
                                           'bare name' => {
                                                            'wsyid' => 675,
                                                            'xsy' => 'bare name'
                                                          },
                                           'before or after' => {
                                                                  'wsyid' => 663,
                                                                  'xsy' => 'before or after'
                                                                },
                                           'boolean' => {
                                                          'wsyid' => 667,
                                                          'xsy' => 'boolean'
                                                        },
                                           'bracketed name' => {
                                                                 'wsyid' => 677,
                                                                 'xsy' => 'bracketed name'
                                                               },
                                           'bracketed name string' => {
                                                                        'wsyid' => 678,
                                                                        'xsy' => 'bracketed name string'
                                                                      },
                                           'cc element' => {
                                                             'wsyid' => 690,
                                                             'xsy' => 'cc element'
                                                           },
                                           'cc elements' => {
                                                              'wsyid' => 689,
                                                              'xsy' => 'cc elements'
                                                            },
                                           'character class' => {
                                                                  'wsyid' => 688,
                                                                  'xsy' => 'character class'
                                                                },
                                           'character class modifier' => {
                                                                           'wsyid' => 694,
                                                                           'xsy' => 'character class modifier'
                                                                         },
                                           'character class modifiers' => {
                                                                            'wsyid' => 693,
                                                                            'xsy' => 'character class modifiers'
                                                                          },
                                           'double colon' => {
                                                               'wsyid' => 673,
                                                               'xsy' => 'double colon'
                                                             },
                                           'escaped cc character' => {
                                                                       'wsyid' => 692,
                                                                       'xsy' => 'escaped cc character'
                                                                     },
                                           'hash comment' => {
                                                               'wsyid' => 653,
                                                               'xsy' => 'hash comment'
                                                             },
                                           'hash comment body' => {
                                                                    'wsyid' => 656,
                                                                    'xsy' => 'hash comment body'
                                                                  },
                                           'hash comment char' => {
                                                                    'wsyid' => 658,
                                                                    'xsy' => 'hash comment char'
                                                                  },
                                           'horizontal character' => {
                                                                       'wsyid' => 698,
                                                                       'xsy' => 'horizontal character'
                                                                     },
                                           'integer' => {
                                                          'wsyid' => 666,
                                                          'xsy' => 'integer'
                                                        },
                                           'negated posix char class' => {
                                                                           'wsyid' => 696,
                                                                           'xsy' => 'negated posix char class'
                                                                         },
                                           'one or more word characters' => {
                                                                              'wsyid' => 670,
                                                                              'xsy' => 'one or more word characters'
                                                                            },
                                           'op declare bnf' => {
                                                                 'wsyid' => 659,
                                                                 'xsy' => 'op declare bnf'
                                                               },
                                           'op declare match' => {
                                                                   'wsyid' => 660,
                                                                   'xsy' => 'op declare match'
                                                                 },
                                           'op equal priority' => {
                                                                    'wsyid' => 662,
                                                                    'xsy' => 'op equal priority'
                                                                  },
                                           'op loosen' => {
                                                            'wsyid' => 661,
                                                            'xsy' => 'op loosen'
                                                          },
                                           'posix char class' => {
                                                                   'wsyid' => 695,
                                                                   'xsy' => 'posix char class'
                                                                 },
                                           'posix char class name' => {
                                                                        'wsyid' => 697,
                                                                        'xsy' => 'posix char class name'
                                                                      },
                                           'reserved action name' => {
                                                                       'wsyid' => 668,
                                                                       'xsy' => 'reserved action name'
                                                                     },
                                           'reserved blessing name' => {
                                                                         'wsyid' => 669,
                                                                         'xsy' => 'reserved blessing name'
                                                                       },
                                           'reserved event name' => {
                                                                      'wsyid' => 651,
                                                                      'xsy' => 'reserved event name'
                                                                    },
                                           'result item descriptor' => {
                                                                         'wsyid' => 684,
                                                                         'xsy' => 'result item descriptor'
                                                                       },
                                           'result item descriptor list' => {
                                                                              'wsyid' => 682,
                                                                              'xsy' => 'result item descriptor list'
                                                                            },
                                           'result item descriptor separator' => {
                                                                                   'wsyid' => 683,
                                                                                   'xsy' => 'result item descriptor separator'
                                                                                 },
                                           'safe cc character' => {
                                                                    'wsyid' => 691,
                                                                    'xsy' => 'safe cc character'
                                                                  },
                                           'sign' => {
                                                       'wsyid' => 665,
                                                       'xsy' => 'sign'
                                                     },
                                           'signed integer' => {
                                                                 'wsyid' => 664,
                                                                 'xsy' => 'signed integer'
                                                               },
                                           'single quoted name' => {
                                                                     'wsyid' => 686,
                                                                     'xsy' => 'single quoted name'
                                                                   },
                                           'single quoted string' => {
                                                                       'wsyid' => 685,
                                                                       'xsy' => 'single quoted string'
                                                                     },
                                           'standard name' => {
                                                                'wsyid' => 676,
                                                                'xsy' => 'standard name'
                                                              },
                                           'string without single quote or vertical space' => {
                                                                                                'wsyid' => 687,
                                                                                                'xsy' => 'string without single quote or vertical space'
                                                                                              },
                                           'terminated hash comment' => {
                                                                          'wsyid' => 654,
                                                                          'xsy' => 'terminated hash comment'
                                                                        },
                                           'unterminated final hash comment' => {
                                                                                  'wsyid' => 655,
                                                                                  'xsy' => 'unterminated final hash comment'
                                                                                },
                                           'vertical space char' => {
                                                                      'wsyid' => 657,
                                                                      'xsy' => 'vertical space char'
                                                                    },
                                           'whitespace' => {
                                                             'wsyid' => 652,
                                                             'xsy' => 'whitespace'
                                                           },
                                           'zero or more word characters' => {
                                                                               'wsyid' => 671,
                                                                               'xsy' => 'zero or more word characters'
                                                                             }
                                         }
                               },
                  'xbnf' => {
                              'G1' => {
                                        '[:start],statements' => {
                                                                   'action' => '::first',
                                                                   'length' => 0,
                                                                   'lhs' => '[:start]',
                                                                   'rhs' => [
                                                                              'statements'
                                                                            ],
                                                                   'start' => 0,
                                                                   'subkey' => 0
                                                                 },
                                        'action name,Perl name' => {
                                                                     'action' => '[start,length,values]',
                                                                     'bless' => 'action_name',
                                                                     'length' => 29,
                                                                     'lhs' => 'action name',
                                                                     'mask' => [
                                                                                 1
                                                                               ],
                                                                     'name' => 'action name',
                                                                     'rhs' => [
                                                                                'Perl name'
                                                                              ],
                                                                     'start' => 4685,
                                                                     'subkey' => 1,
                                                                     'xrlid' => 'action name@4685+29'
                                                                   },
                                        'action name,array descriptor' => {
                                                                            'action' => '[start,length,values]',
                                                                            'bless' => 'action_name',
                                                                            'length' => 36,
                                                                            'lhs' => 'action name',
                                                                            'mask' => [
                                                                                        1
                                                                                      ],
                                                                            'name' => 'action name',
                                                                            'rhs' => [
                                                                                       'array descriptor'
                                                                                     ],
                                                                            'start' => 4756,
                                                                            'subkey' => 1,
                                                                            'xrlid' => 'action name@4756+36'
                                                                          },
                                        'action name,reserved action name' => {
                                                                                'action' => '[start,length,values]',
                                                                                'bless' => 'action_name',
                                                                                'length' => 40,
                                                                                'lhs' => 'action name',
                                                                                'mask' => [
                                                                                            1
                                                                                          ],
                                                                                'name' => 'action name',
                                                                                'rhs' => [
                                                                                           'reserved action name'
                                                                                         ],
                                                                                'start' => 4715,
                                                                                'subkey' => 1,
                                                                                'xrlid' => 'action name@4715+40'
                                                                              },
                                        'action,[Lex-33],[Lex-34],action name' => {
                                                                                    'action' => '[start,length,values]',
                                                                                    'bless' => 'action',
                                                                                    'length' => 40,
                                                                                    'lhs' => 'action',
                                                                                    'mask' => [
                                                                                                0,
                                                                                                0,
                                                                                                1
                                                                                              ],
                                                                                    'name' => 'action',
                                                                                    'rhs' => [
                                                                                               '[Lex-33]',
                                                                                               '[Lex-34]',
                                                                                               'action name'
                                                                                             ],
                                                                                    'start' => 2897,
                                                                                    'subkey' => 1,
                                                                                    'xrlid' => 'action@2897+40'
                                                                                  },
                                        'adverb item,action' => {
                                                                  'action' => '[start,length,values]',
                                                                  'bless' => 'adverb_item',
                                                                  'length' => 362,
                                                                  'lhs' => 'adverb item',
                                                                  'mask' => [
                                                                              1
                                                                            ],
                                                                  'name' => 'adverb item',
                                                                  'rhs' => [
                                                                             'action'
                                                                           ],
                                                                  'start' => 2511,
                                                                  'subkey' => 1,
                                                                  'xrlid' => 'adverb item@2511+362'
                                                                },
                                        'adverb item,blessing' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'adverb_item',
                                                                    'length' => 8,
                                                                    'lhs' => 'adverb item',
                                                                    'mask' => [
                                                                                1
                                                                              ],
                                                                    'name' => 'adverb item',
                                                                    'rhs' => [
                                                                               'blessing'
                                                                             ],
                                                                    'start' => 2840,
                                                                    'subkey' => 12,
                                                                    'xrlid' => 'adverb item@2511+362'
                                                                  },
                                        'adverb item,event specification' => {
                                                                               'action' => '[start,length,values]',
                                                                               'bless' => 'adverb_item',
                                                                               'length' => 21,
                                                                               'lhs' => 'adverb item',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'adverb item',
                                                                               'rhs' => [
                                                                                          'event specification'
                                                                                        ],
                                                                               'start' => 2783,
                                                                               'subkey' => 11,
                                                                               'xrlid' => 'adverb item@2511+362'
                                                                             },
                                        'adverb item,group association' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'adverb_item',
                                                                             'length' => 19,
                                                                             'lhs' => 'adverb item',
                                                                             'mask' => [
                                                                                         1
                                                                                       ],
                                                                             'name' => 'adverb item',
                                                                             'rhs' => [
                                                                                        'group association'
                                                                                      ],
                                                                             'start' => 2591,
                                                                             'subkey' => 4,
                                                                             'xrlid' => 'adverb item@2511+362'
                                                                           },
                                        'adverb item,left association' => {
                                                                            'action' => '[start,length,values]',
                                                                            'bless' => 'adverb_item',
                                                                            'length' => 18,
                                                                            'lhs' => 'adverb item',
                                                                            'mask' => [
                                                                                        1
                                                                                      ],
                                                                            'name' => 'adverb item',
                                                                            'rhs' => [
                                                                                       'left association'
                                                                                     ],
                                                                            'start' => 2548,
                                                                            'subkey' => 2,
                                                                            'xrlid' => 'adverb item@2511+362'
                                                                          },
                                        'adverb item,naming' => {
                                                                  'action' => '[start,length,values]',
                                                                  'bless' => 'adverb_item',
                                                                  'length' => 6,
                                                                  'lhs' => 'adverb item',
                                                                  'mask' => [
                                                                              1
                                                                            ],
                                                                  'name' => 'adverb item',
                                                                  'rhs' => [
                                                                             'naming'
                                                                           ],
                                                                  'start' => 2851,
                                                                  'subkey' => 13,
                                                                  'xrlid' => 'adverb item@2511+362'
                                                                },
                                        'adverb item,null adverb' => {
                                                                       'action' => '[start,length,values]',
                                                                       'bless' => 'adverb_item',
                                                                       'length' => 13,
                                                                       'lhs' => 'adverb item',
                                                                       'mask' => [
                                                                                   1
                                                                                 ],
                                                                       'name' => 'adverb item',
                                                                       'rhs' => [
                                                                                  'null adverb'
                                                                                ],
                                                                       'start' => 2860,
                                                                       'subkey' => 14,
                                                                       'xrlid' => 'adverb item@2511+362'
                                                                     },
                                        'adverb item,null ranking specification' => {
                                                                                      'action' => '[start,length,values]',
                                                                                      'bless' => 'adverb_item',
                                                                                      'length' => 28,
                                                                                      'lhs' => 'adverb item',
                                                                                      'mask' => [
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'adverb item',
                                                                                      'rhs' => [
                                                                                                 'null ranking specification'
                                                                                               ],
                                                                                      'start' => 2697,
                                                                                      'subkey' => 8,
                                                                                      'xrlid' => 'adverb item@2511+362'
                                                                                    },
                                        'adverb item,pause specification' => {
                                                                               'action' => '[start,length,values]',
                                                                               'bless' => 'adverb_item',
                                                                               'length' => 21,
                                                                               'lhs' => 'adverb item',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'adverb item',
                                                                               'rhs' => [
                                                                                          'pause specification'
                                                                                        ],
                                                                               'start' => 2759,
                                                                               'subkey' => 10,
                                                                               'xrlid' => 'adverb item@2511+362'
                                                                             },
                                        'adverb item,priority specification' => {
                                                                                  'action' => '[start,length,values]',
                                                                                  'bless' => 'adverb_item',
                                                                                  'length' => 24,
                                                                                  'lhs' => 'adverb item',
                                                                                  'mask' => [
                                                                                              1
                                                                                            ],
                                                                                  'name' => 'adverb item',
                                                                                  'rhs' => [
                                                                                             'priority specification'
                                                                                           ],
                                                                                  'start' => 2732,
                                                                                  'subkey' => 9,
                                                                                  'xrlid' => 'adverb item@2511+362'
                                                                                },
                                        'adverb item,proper specification' => {
                                                                                'action' => '[start,length,values]',
                                                                                'bless' => 'adverb_item',
                                                                                'length' => 22,
                                                                                'lhs' => 'adverb item',
                                                                                'mask' => [
                                                                                            1
                                                                                          ],
                                                                                'name' => 'adverb item',
                                                                                'rhs' => [
                                                                                           'proper specification'
                                                                                         ],
                                                                                'start' => 2645,
                                                                                'subkey' => 6,
                                                                                'xrlid' => 'adverb item@2511+362'
                                                                              },
                                        'adverb item,rank specification' => {
                                                                              'action' => '[start,length,values]',
                                                                              'bless' => 'adverb_item',
                                                                              'length' => 20,
                                                                              'lhs' => 'adverb item',
                                                                              'mask' => [
                                                                                          1
                                                                                        ],
                                                                              'name' => 'adverb item',
                                                                              'rhs' => [
                                                                                         'rank specification'
                                                                                       ],
                                                                              'start' => 2674,
                                                                              'subkey' => 7,
                                                                              'xrlid' => 'adverb item@2511+362'
                                                                            },
                                        'adverb item,right association' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'adverb_item',
                                                                             'length' => 19,
                                                                             'lhs' => 'adverb item',
                                                                             'mask' => [
                                                                                         1
                                                                                       ],
                                                                             'name' => 'adverb item',
                                                                             'rhs' => [
                                                                                        'right association'
                                                                                      ],
                                                                             'start' => 2569,
                                                                             'subkey' => 3,
                                                                             'xrlid' => 'adverb item@2511+362'
                                                                           },
                                        'adverb item,separator specification' => {
                                                                                   'action' => '[start,length,values]',
                                                                                   'bless' => 'adverb_item',
                                                                                   'length' => 25,
                                                                                   'lhs' => 'adverb item',
                                                                                   'mask' => [
                                                                                               1
                                                                                             ],
                                                                                   'name' => 'adverb item',
                                                                                   'rhs' => [
                                                                                              'separator specification'
                                                                                            ],
                                                                                   'start' => 2617,
                                                                                   'subkey' => 5,
                                                                                   'xrlid' => 'adverb item@2511+362'
                                                                                 },
                                        'adverb list items,adverb item' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'adverb_list_items',
                                                                             'length' => 38,
                                                                             'lhs' => 'adverb list items',
                                                                             'min' => 0,
                                                                             'name' => 'adverb list items',
                                                                             'rhs' => [
                                                                                        'adverb item'
                                                                                      ],
                                                                             'start' => 2472,
                                                                             'subkey' => 0,
                                                                             'xrlid' => 'adverb list items@2472+38'
                                                                           },
                                        'adverb list,adverb list items' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'adverb_list',
                                                                             'length' => 37,
                                                                             'lhs' => 'adverb list',
                                                                             'mask' => [
                                                                                         1
                                                                                       ],
                                                                             'name' => 'adverb list',
                                                                             'rhs' => [
                                                                                        'adverb list items'
                                                                                      ],
                                                                             'start' => 2434,
                                                                             'subkey' => 1,
                                                                             'xrlid' => 'adverb list@2434+37'
                                                                           },
                                        'alternative name,single quoted name' => {
                                                                                   'action' => '[start,length,values]',
                                                                                   'bless' => 'alternative_name',
                                                                                   'length' => 20,
                                                                                   'lhs' => 'alternative name',
                                                                                   'mask' => [
                                                                                               1
                                                                                             ],
                                                                                   'name' => 'alternative name',
                                                                                   'rhs' => [
                                                                                              'single quoted name'
                                                                                            ],
                                                                                   'start' => 4038,
                                                                                   'subkey' => 2,
                                                                                   'xrlid' => 'alternative name@3997+61'
                                                                                 },
                                        'alternative name,standard name' => {
                                                                              'action' => '[start,length,values]',
                                                                              'bless' => 'alternative_name',
                                                                              'length' => 61,
                                                                              'lhs' => 'alternative name',
                                                                              'mask' => [
                                                                                          1
                                                                                        ],
                                                                              'name' => 'alternative name',
                                                                              'rhs' => [
                                                                                         'standard name'
                                                                                       ],
                                                                              'start' => 3997,
                                                                              'subkey' => 1,
                                                                              'xrlid' => 'alternative name@3997+61'
                                                                            },
                                        'alternative,rhs,adverb list' => {
                                                                           'action' => '[start,length,values]',
                                                                           'bless' => 'alternative',
                                                                           'length' => 33,
                                                                           'lhs' => 'alternative',
                                                                           'mask' => [
                                                                                       1,
                                                                                       1
                                                                                     ],
                                                                           'name' => 'alternative',
                                                                           'rhs' => [
                                                                                      'rhs',
                                                                                      'adverb list'
                                                                                    ],
                                                                           'start' => 2399,
                                                                           'subkey' => 1,
                                                                           'xrlid' => 'alternative@2399+33'
                                                                         },
                                        'alternatives,alternative' => {
                                                                        'action' => '[start,length,values]',
                                                                        'bless' => 'alternatives',
                                                                        'length' => 78,
                                                                        'lhs' => 'alternatives',
                                                                        'min' => 1,
                                                                        'name' => 'alternatives',
                                                                        'proper' => '1',
                                                                        'rhs' => [
                                                                                   'alternative'
                                                                                 ],
                                                                        'separator' => 'op equal priority',
                                                                        'start' => 2320,
                                                                        'subkey' => 0,
                                                                        'xrlid' => 'alternatives@2320+78'
                                                                      },
                                        'blessing name,reserved blessing name' => {
                                                                                    'action' => '[start,length,values]',
                                                                                    'bless' => 'blessing_name',
                                                                                    'length' => 44,
                                                                                    'lhs' => 'blessing name',
                                                                                    'mask' => [
                                                                                                1
                                                                                              ],
                                                                                    'name' => 'blessing name',
                                                                                    'rhs' => [
                                                                                               'reserved blessing name'
                                                                                             ],
                                                                                    'start' => 4219,
                                                                                    'subkey' => 1,
                                                                                    'xrlid' => 'blessing name@4219+44'
                                                                                  },
                                        'blessing name,standard name' => {
                                                                           'action' => '[start,length,values]',
                                                                           'bless' => 'blessing_name',
                                                                           'length' => 35,
                                                                           'lhs' => 'blessing name',
                                                                           'mask' => [
                                                                                       1
                                                                                     ],
                                                                           'name' => 'blessing name',
                                                                           'rhs' => [
                                                                                      'standard name'
                                                                                    ],
                                                                           'start' => 4183,
                                                                           'subkey' => 1,
                                                                           'xrlid' => 'blessing name@4183+35'
                                                                         },
                                        'blessing,[Lex-66],[Lex-67],blessing name' => {
                                                                                        'action' => '[start,length,values]',
                                                                                        'bless' => 'blessing',
                                                                                        'length' => 45,
                                                                                        'lhs' => 'blessing',
                                                                                        'mask' => [
                                                                                                    0,
                                                                                                    0,
                                                                                                    1
                                                                                                  ],
                                                                                        'name' => 'blessing',
                                                                                        'rhs' => [
                                                                                                   '[Lex-66]',
                                                                                                   '[Lex-67]',
                                                                                                   'blessing name'
                                                                                                 ],
                                                                                        'start' => 3904,
                                                                                        'subkey' => 1,
                                                                                        'xrlid' => 'blessing@3904+45'
                                                                                      },
                                        'completion event declaration,[Lex-16],event initialization,[Lex-17],[Lex-18],symbol name' => {
                                                                                                                                        'action' => '[start,length,values]',
                                                                                                                                        'bless' => 'completion_event_declaration',
                                                                                                                                        'length' => 99,
                                                                                                                                        'lhs' => 'completion event declaration',
                                                                                                                                        'mask' => [
                                                                                                                                                    0,
                                                                                                                                                    1,
                                                                                                                                                    0,
                                                                                                                                                    0,
                                                                                                                                                    1
                                                                                                                                                  ],
                                                                                                                                        'name' => 'completion event declaration',
                                                                                                                                        'rhs' => [
                                                                                                                                                   '[Lex-16]',
                                                                                                                                                   'event initialization',
                                                                                                                                                   '[Lex-17]',
                                                                                                                                                   '[Lex-18]',
                                                                                                                                                   'symbol name'
                                                                                                                                                 ],
                                                                                                                                        'start' => 1754,
                                                                                                                                        'subkey' => 1,
                                                                                                                                        'xrlid' => 'completion event declaration@1754+99'
                                                                                                                                      },
                                        'default rule,[Lex-7],op declare bnf,adverb list' => {
                                                                                               'action' => '[start,length,values]',
                                                                                               'bless' => 'default_rule',
                                                                                               'length' => 60,
                                                                                               'lhs' => 'default rule',
                                                                                               'mask' => [
                                                                                                           1,
                                                                                                           1,
                                                                                                           1
                                                                                                         ],
                                                                                               'name' => 'default rule',
                                                                                               'rhs' => [
                                                                                                          '[Lex-7]',
                                                                                                          'op declare bnf',
                                                                                                          'adverb list'
                                                                                                        ],
                                                                                               'start' => 1224,
                                                                                               'subkey' => 1,
                                                                                               'xrlid' => 'default rule@1224+60'
                                                                                             },
                                        'discard default statement,[Lex-11],[Lex-12],[Lex-13],adverb list' => {
                                                                                                                'action' => '[start,length,values]',
                                                                                                                'bless' => 'discard_default_statement',
                                                                                                                'length' => 71,
                                                                                                                'lhs' => 'discard default statement',
                                                                                                                'mask' => [
                                                                                                                            0,
                                                                                                                            0,
                                                                                                                            0,
                                                                                                                            1
                                                                                                                          ],
                                                                                                                'name' => 'discard default statement',
                                                                                                                'rhs' => [
                                                                                                                           '[Lex-11]',
                                                                                                                           '[Lex-12]',
                                                                                                                           '[Lex-13]',
                                                                                                                           'adverb list'
                                                                                                                         ],
                                                                                                                'start' => 1355,
                                                                                                                'subkey' => 1,
                                                                                                                'xrlid' => 'discard default statement@1355+71'
                                                                                                              },
                                        'discard rule,[Lex-14],op declare match,single symbol,adverb list' => {
                                                                                                                'action' => '[start,length,values]',
                                                                                                                'bless' => 'discard_rule',
                                                                                                                'length' => 80,
                                                                                                                'lhs' => 'discard rule',
                                                                                                                'mask' => [
                                                                                                                            0,
                                                                                                                            0,
                                                                                                                            1,
                                                                                                                            1
                                                                                                                          ],
                                                                                                                'name' => 'discard rule',
                                                                                                                'rhs' => [
                                                                                                                           '[Lex-14]',
                                                                                                                           'op declare match',
                                                                                                                           'single symbol',
                                                                                                                           'adverb list'
                                                                                                                         ],
                                                                                                                'start' => 1603,
                                                                                                                'subkey' => 1,
                                                                                                                'xrlid' => 'discard rule@1603+80'
                                                                                                              },
                                        'empty rule,lhs,op declare,adverb list' => {
                                                                                     'action' => '[start,length,values]',
                                                                                     'bless' => 'empty_rule',
                                                                                     'length' => 47,
                                                                                     'lhs' => 'empty rule',
                                                                                     'mask' => [
                                                                                                 1,
                                                                                                 1,
                                                                                                 1
                                                                                               ],
                                                                                     'name' => 'empty rule',
                                                                                     'rhs' => [
                                                                                                'lhs',
                                                                                                'op declare',
                                                                                                'adverb list'
                                                                                              ],
                                                                                     'start' => 1475,
                                                                                     'subkey' => 1,
                                                                                     'xrlid' => 'empty rule@1475+47'
                                                                                   },
                                        'event initialization,event name,event initializer' => {
                                                                                                 'action' => '[start,length,values]',
                                                                                                 'bless' => 'event_initialization',
                                                                                                 'length' => 59,
                                                                                                 'lhs' => 'event initialization',
                                                                                                 'mask' => [
                                                                                                             1,
                                                                                                             1
                                                                                                           ],
                                                                                                 'name' => 'event initialization',
                                                                                                 'rhs' => [
                                                                                                            'event name',
                                                                                                            'event initializer'
                                                                                                          ],
                                                                                                 'start' => 3637,
                                                                                                 'subkey' => 1,
                                                                                                 'xrlid' => 'event initialization@3637+59'
                                                                                               },
                                        'event initializer' => {
                                                                 'action' => '[start,length,values]',
                                                                 'bless' => 'event_initializer',
                                                                 'length' => 23,
                                                                 'lhs' => 'event initializer',
                                                                 'name' => 'event initializer',
                                                                 'rhs' => [],
                                                                 'start' => 3768,
                                                                 'subkey' => 0,
                                                                 'xrlid' => 'event initializer@3768+23'
                                                               },
                                        'event initializer,[Lex-63],on or off' => {
                                                                                    'action' => '[start,length,values]',
                                                                                    'bless' => 'event_initializer',
                                                                                    'length' => 41,
                                                                                    'lhs' => 'event initializer',
                                                                                    'mask' => [
                                                                                                0,
                                                                                                1
                                                                                              ],
                                                                                    'name' => 'event initializer',
                                                                                    'rhs' => [
                                                                                               '[Lex-63]',
                                                                                               'on or off'
                                                                                             ],
                                                                                    'start' => 3697,
                                                                                    'subkey' => 1,
                                                                                    'xrlid' => 'event initializer@3697+41'
                                                                                  },
                                        'event name,reserved event name' => {
                                                                              'action' => '[start,length,values]',
                                                                              'bless' => 'event_name',
                                                                              'length' => 21,
                                                                              'lhs' => 'event name',
                                                                              'mask' => [
                                                                                          1
                                                                                        ],
                                                                              'name' => 'event name',
                                                                              'rhs' => [
                                                                                         'reserved event name'
                                                                                       ],
                                                                              'start' => 4126,
                                                                              'subkey' => 3,
                                                                              'xrlid' => 'event name@4060+87'
                                                                            },
                                        'event name,single quoted name' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'event_name',
                                                                             'length' => 20,
                                                                             'lhs' => 'event name',
                                                                             'mask' => [
                                                                                         1
                                                                                       ],
                                                                             'name' => 'event name',
                                                                             'rhs' => [
                                                                                        'single quoted name'
                                                                                      ],
                                                                             'start' => 4099,
                                                                             'subkey' => 2,
                                                                             'xrlid' => 'event name@4060+87'
                                                                           },
                                        'event name,standard name' => {
                                                                        'action' => '[start,length,values]',
                                                                        'bless' => 'event_name',
                                                                        'length' => 87,
                                                                        'lhs' => 'event name',
                                                                        'mask' => [
                                                                                    1
                                                                                  ],
                                                                        'name' => 'event name',
                                                                        'rhs' => [
                                                                                   'standard name'
                                                                                 ],
                                                                        'start' => 4060,
                                                                        'subkey' => 1,
                                                                        'xrlid' => 'event name@4060+87'
                                                                      },
                                        'event specification,[Lex-61],[Lex-62],event initialization' => {
                                                                                                          'action' => '[start,length,values]',
                                                                                                          'bless' => 'event_specification',
                                                                                                          'length' => 63,
                                                                                                          'lhs' => 'event specification',
                                                                                                          'mask' => [
                                                                                                                      0,
                                                                                                                      0,
                                                                                                                      1
                                                                                                                    ],
                                                                                                          'name' => 'event specification',
                                                                                                          'rhs' => [
                                                                                                                     '[Lex-61]',
                                                                                                                     '[Lex-62]',
                                                                                                                     'event initialization'
                                                                                                                   ],
                                                                                                          'start' => 3573,
                                                                                                          'subkey' => 1,
                                                                                                          'xrlid' => 'event specification@3573+63'
                                                                                                        },
                                        'group association,[Lex-41],[Lex-42],[Lex-43]' => {
                                                                                            'action' => '[start,length,values]',
                                                                                            'bless' => 'group_association',
                                                                                            'length' => 46,
                                                                                            'lhs' => 'group association',
                                                                                            'mask' => [
                                                                                                        0,
                                                                                                        0,
                                                                                                        0
                                                                                                      ],
                                                                                            'name' => 'group association',
                                                                                            'rhs' => [
                                                                                                       '[Lex-41]',
                                                                                                       '[Lex-42]',
                                                                                                       '[Lex-43]'
                                                                                                     ],
                                                                                            'start' => 3030,
                                                                                            'subkey' => 1,
                                                                                            'xrlid' => 'group association@3030+46'
                                                                                          },
                                        'inaccessible statement,[Lex-25],[Lex-26],inaccessible treatment,[Lex-27],[Lex-28]' => {
                                                                                                                                 'action' => '[start,length,values]',
                                                                                                                                 'bless' => 'inaccessible_statement',
                                                                                                                                 'length' => 92,
                                                                                                                                 'lhs' => 'inaccessible statement',
                                                                                                                                 'mask' => [
                                                                                                                                             0,
                                                                                                                                             0,
                                                                                                                                             1,
                                                                                                                                             0,
                                                                                                                                             0
                                                                                                                                           ],
                                                                                                                                 'name' => 'inaccessible statement',
                                                                                                                                 'rhs' => [
                                                                                                                                            '[Lex-25]',
                                                                                                                                            '[Lex-26]',
                                                                                                                                            'inaccessible treatment',
                                                                                                                                            '[Lex-27]',
                                                                                                                                            '[Lex-28]'
                                                                                                                                          ],
                                                                                                                                 'start' => 2048,
                                                                                                                                 'subkey' => 1,
                                                                                                                                 'xrlid' => 'inaccessible statement@2048+92'
                                                                                                                               },
                                        'inaccessible treatment,[Lex-29]' => {
                                                                               'action' => '[start,length,values]',
                                                                               'bless' => 'inaccessible_treatment',
                                                                               'length' => 52,
                                                                               'lhs' => 'inaccessible treatment',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'inaccessible treatment',
                                                                               'rhs' => [
                                                                                          '[Lex-29]'
                                                                                        ],
                                                                               'start' => 2141,
                                                                               'subkey' => 1,
                                                                               'xrlid' => 'inaccessible treatment@2141+52'
                                                                             },
                                        'inaccessible treatment,[Lex-30]' => {
                                                                               'action' => '[start,length,values]',
                                                                               'bless' => 'inaccessible_treatment',
                                                                               'length' => 4,
                                                                               'lhs' => 'inaccessible treatment',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'inaccessible treatment',
                                                                               'rhs' => [
                                                                                          '[Lex-30]'
                                                                                        ],
                                                                               'start' => 2179,
                                                                               'subkey' => 2,
                                                                               'xrlid' => 'inaccessible treatment@2141+52'
                                                                             },
                                        'inaccessible treatment,[Lex-31]' => {
                                                                               'action' => '[start,length,values]',
                                                                               'bless' => 'inaccessible_treatment',
                                                                               'length' => 7,
                                                                               'lhs' => 'inaccessible treatment',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'inaccessible treatment',
                                                                               'rhs' => [
                                                                                          '[Lex-31]'
                                                                                        ],
                                                                               'start' => 2186,
                                                                               'subkey' => 3,
                                                                               'xrlid' => 'inaccessible treatment@2141+52'
                                                                             },
                                        'left association,[Lex-35],[Lex-36],[Lex-37]' => {
                                                                                           'action' => '[start,length,values]',
                                                                                           'bless' => 'left_association',
                                                                                           'length' => 44,
                                                                                           'lhs' => 'left association',
                                                                                           'mask' => [
                                                                                                       0,
                                                                                                       0,
                                                                                                       0
                                                                                                     ],
                                                                                           'name' => 'left association',
                                                                                           'rhs' => [
                                                                                                      '[Lex-35]',
                                                                                                      '[Lex-36]',
                                                                                                      '[Lex-37]'
                                                                                                    ],
                                                                                           'start' => 2938,
                                                                                           'subkey' => 1,
                                                                                           'xrlid' => 'left association@2938+44'
                                                                                         },
                                        'lexeme default statement,[Lex-8],[Lex-9],[Lex-10],adverb list' => {
                                                                                                             'action' => '[start,length,values]',
                                                                                                             'bless' => 'lexeme_default_statement',
                                                                                                             'length' => 69,
                                                                                                             'lhs' => 'lexeme default statement',
                                                                                                             'mask' => [
                                                                                                                         0,
                                                                                                                         0,
                                                                                                                         0,
                                                                                                                         1
                                                                                                                       ],
                                                                                                             'name' => 'lexeme default statement',
                                                                                                             'rhs' => [
                                                                                                                        '[Lex-8]',
                                                                                                                        '[Lex-9]',
                                                                                                                        '[Lex-10]',
                                                                                                                        'adverb list'
                                                                                                                      ],
                                                                                                             'start' => 1285,
                                                                                                             'subkey' => 1,
                                                                                                             'xrlid' => 'lexeme default statement@1285+69'
                                                                                                           },
                                        'lexeme rule,[Lex-15],op declare match,symbol,adverb list' => {
                                                                                                        'action' => '[start,length,values]',
                                                                                                        'bless' => 'lexeme_rule',
                                                                                                        'length' => 69,
                                                                                                        'lhs' => 'lexeme rule',
                                                                                                        'mask' => [
                                                                                                                    0,
                                                                                                                    0,
                                                                                                                    1,
                                                                                                                    1
                                                                                                                  ],
                                                                                                        'name' => 'lexeme rule',
                                                                                                        'rhs' => [
                                                                                                                   '[Lex-15]',
                                                                                                                   'op declare match',
                                                                                                                   'symbol',
                                                                                                                   'adverb list'
                                                                                                                 ],
                                                                                                        'start' => 1684,
                                                                                                        'subkey' => 1,
                                                                                                        'xrlid' => 'lexeme rule@1684+69'
                                                                                                      },
                                        'lhs,symbol name' => {
                                                               'action' => '[start,length,values]',
                                                               'bless' => 'lhs',
                                                               'length' => 21,
                                                               'lhs' => 'lhs',
                                                               'mask' => [
                                                                           1
                                                                         ],
                                                               'name' => 'lhs',
                                                               'rhs' => [
                                                                          'symbol name'
                                                                        ],
                                                               'start' => 4265,
                                                               'subkey' => 1,
                                                               'xrlid' => 'lhs@4265+21'
                                                             },
                                        'naming,[Lex-68],[Lex-69],alternative name' => {
                                                                                         'action' => '[start,length,values]',
                                                                                         'bless' => 'naming',
                                                                                         'length' => 45,
                                                                                         'lhs' => 'naming',
                                                                                         'mask' => [
                                                                                                     0,
                                                                                                     0,
                                                                                                     1
                                                                                                   ],
                                                                                         'name' => 'naming',
                                                                                         'rhs' => [
                                                                                                    '[Lex-68]',
                                                                                                    '[Lex-69]',
                                                                                                    'alternative name'
                                                                                                  ],
                                                                                         'start' => 3950,
                                                                                         'subkey' => 1,
                                                                                         'xrlid' => 'naming@3950+45'
                                                                                       },
                                        'null adverb,[Lex-32]' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'null_adverb',
                                                                    'length' => 21,
                                                                    'lhs' => 'null adverb',
                                                                    'mask' => [
                                                                                1
                                                                              ],
                                                                    'name' => 'null adverb',
                                                                    'rhs' => [
                                                                               '[Lex-32]'
                                                                             ],
                                                                    'start' => 2875,
                                                                    'subkey' => 1,
                                                                    'xrlid' => 'null adverb@2875+21'
                                                                  },
                                        'null ranking constant,[Lex-55]' => {
                                                                              'action' => '[start,length,values]',
                                                                              'bless' => 'null_ranking_constant',
                                                                              'length' => 42,
                                                                              'lhs' => 'null ranking constant',
                                                                              'mask' => [
                                                                                          1
                                                                                        ],
                                                                              'name' => 'null ranking constant',
                                                                              'rhs' => [
                                                                                         '[Lex-55]'
                                                                                       ],
                                                                              'start' => 3406,
                                                                              'subkey' => 1,
                                                                              'xrlid' => 'null ranking constant@3406+42'
                                                                            },
                                        'null ranking constant,[Lex-56]' => {
                                                                              'action' => '[start,length,values]',
                                                                              'bless' => 'null_ranking_constant',
                                                                              'length' => 6,
                                                                              'lhs' => 'null ranking constant',
                                                                              'mask' => [
                                                                                          1
                                                                                        ],
                                                                              'name' => 'null ranking constant',
                                                                              'rhs' => [
                                                                                         '[Lex-56]'
                                                                                       ],
                                                                              'start' => 3442,
                                                                              'subkey' => 2,
                                                                              'xrlid' => 'null ranking constant@3406+42'
                                                                            },
                                        'null ranking specification,[Lex-50],[Lex-51],null ranking constant' => {
                                                                                                                  'action' => '[start,length,values]',
                                                                                                                  'bless' => 'null_ranking_specification',
                                                                                                                  'length' => 78,
                                                                                                                  'lhs' => 'null ranking specification',
                                                                                                                  'mask' => [
                                                                                                                              0,
                                                                                                                              0,
                                                                                                                              1
                                                                                                                            ],
                                                                                                                  'name' => 'null ranking specification',
                                                                                                                  'rhs' => [
                                                                                                                             '[Lex-50]',
                                                                                                                             '[Lex-51]',
                                                                                                                             'null ranking constant'
                                                                                                                           ],
                                                                                                                  'start' => 3249,
                                                                                                                  'subkey' => 1,
                                                                                                                  'xrlid' => 'null ranking specification@3249+78'
                                                                                                                },
                                        'null ranking specification,[Lex-52],[Lex-53],[Lex-54],null ranking constant' => {
                                                                                                                           'action' => '[start,length,values]',
                                                                                                                           'bless' => 'null_ranking_specification',
                                                                                                                           'length' => 77,
                                                                                                                           'lhs' => 'null ranking specification',
                                                                                                                           'mask' => [
                                                                                                                                       0,
                                                                                                                                       0,
                                                                                                                                       0,
                                                                                                                                       1
                                                                                                                                     ],
                                                                                                                           'name' => 'null ranking specification',
                                                                                                                           'rhs' => [
                                                                                                                                      '[Lex-52]',
                                                                                                                                      '[Lex-53]',
                                                                                                                                      '[Lex-54]',
                                                                                                                                      'null ranking constant'
                                                                                                                                    ],
                                                                                                                           'start' => 3328,
                                                                                                                           'subkey' => 1,
                                                                                                                           'xrlid' => 'null ranking specification@3328+77'
                                                                                                                         },
                                        'null statement,[Lex-0]' => {
                                                                      'action' => '[start,length,values]',
                                                                      'bless' => 'null_statement',
                                                                      'length' => 24,
                                                                      'lhs' => 'null statement',
                                                                      'mask' => [
                                                                                  1
                                                                                ],
                                                                      'name' => 'null statement',
                                                                      'rhs' => [
                                                                                 '[Lex-0]'
                                                                               ],
                                                                      'start' => 1056,
                                                                      'subkey' => 1,
                                                                      'xrlid' => 'null statement@1056+24'
                                                                    },
                                        'nulled event declaration,[Lex-19],event initialization,[Lex-20],[Lex-21],symbol name' => {
                                                                                                                                    'action' => '[start,length,values]',
                                                                                                                                    'bless' => 'nulled_event_declaration',
                                                                                                                                    'length' => 92,
                                                                                                                                    'lhs' => 'nulled event declaration',
                                                                                                                                    'mask' => [
                                                                                                                                                0,
                                                                                                                                                1,
                                                                                                                                                0,
                                                                                                                                                0,
                                                                                                                                                1
                                                                                                                                              ],
                                                                                                                                    'name' => 'nulled event declaration',
                                                                                                                                    'rhs' => [
                                                                                                                                               '[Lex-19]',
                                                                                                                                               'event initialization',
                                                                                                                                               '[Lex-20]',
                                                                                                                                               '[Lex-21]',
                                                                                                                                               'symbol name'
                                                                                                                                             ],
                                                                                                                                    'start' => 1854,
                                                                                                                                    'subkey' => 1,
                                                                                                                                    'xrlid' => 'nulled event declaration@1854+92'
                                                                                                                                  },
                                        'on or off,[Lex-64]' => {
                                                                  'action' => '[start,length,values]',
                                                                  'bless' => 'on_or_off',
                                                                  'length' => 28,
                                                                  'lhs' => 'on or off',
                                                                  'mask' => [
                                                                              1
                                                                            ],
                                                                  'name' => 'on or off',
                                                                  'rhs' => [
                                                                             '[Lex-64]'
                                                                           ],
                                                                  'start' => 3739,
                                                                  'subkey' => 1,
                                                                  'xrlid' => 'on or off@3739+28'
                                                                },
                                        'on or off,[Lex-65]' => {
                                                                  'action' => '[start,length,values]',
                                                                  'bless' => 'on_or_off',
                                                                  'length' => 5,
                                                                  'lhs' => 'on or off',
                                                                  'mask' => [
                                                                              1
                                                                            ],
                                                                  'name' => 'on or off',
                                                                  'rhs' => [
                                                                             '[Lex-65]'
                                                                           ],
                                                                  'start' => 3762,
                                                                  'subkey' => 2,
                                                                  'xrlid' => 'on or off@3739+28'
                                                                },
                                        'op declare,op declare bnf' => {
                                                                         'action' => '[start,length,values]',
                                                                         'bless' => 'op_declare',
                                                                         'length' => 54,
                                                                         'lhs' => 'op declare',
                                                                         'mask' => [
                                                                                     1
                                                                                   ],
                                                                         'name' => 'op declare',
                                                                         'rhs' => [
                                                                                    'op declare bnf'
                                                                                  ],
                                                                         'start' => 2195,
                                                                         'subkey' => 1,
                                                                         'xrlid' => 'op declare@2195+54'
                                                                       },
                                        'op declare,op declare match' => {
                                                                           'action' => '[start,length,values]',
                                                                           'bless' => 'op_declare',
                                                                           'length' => 18,
                                                                           'lhs' => 'op declare',
                                                                           'mask' => [
                                                                                       1
                                                                                     ],
                                                                           'name' => 'op declare',
                                                                           'rhs' => [
                                                                                      'op declare match'
                                                                                    ],
                                                                           'start' => 2231,
                                                                           'subkey' => 2,
                                                                           'xrlid' => 'op declare@2195+54'
                                                                         },
                                        'parenthesized rhs primary list,[Lex-70],rhs primary list,[Lex-71]' => {
                                                                                                                 'action' => '[start,length,values]',
                                                                                                                 'bless' => 'parenthesized_rhs_primary_list',
                                                                                                                 'length' => 67,
                                                                                                                 'lhs' => 'parenthesized rhs primary list',
                                                                                                                 'mask' => [
                                                                                                                             0,
                                                                                                                             1,
                                                                                                                             0
                                                                                                                           ],
                                                                                                                 'name' => 'parenthesized rhs primary list',
                                                                                                                 'rhs' => [
                                                                                                                            '[Lex-70]',
                                                                                                                            'rhs primary list',
                                                                                                                            '[Lex-71]'
                                                                                                                          ],
                                                                                                                 'start' => 4436,
                                                                                                                 'subkey' => 1,
                                                                                                                 'xrlid' => 'parenthesized rhs primary list@4436+67'
                                                                                                               },
                                        'pause specification,[Lex-59],[Lex-60],before or after' => {
                                                                                                     'action' => '[start,length,values]',
                                                                                                     'bless' => 'pause_specification',
                                                                                                     'length' => 58,
                                                                                                     'lhs' => 'pause specification',
                                                                                                     'mask' => [
                                                                                                                 0,
                                                                                                                 0,
                                                                                                                 1
                                                                                                               ],
                                                                                                     'name' => 'pause specification',
                                                                                                     'rhs' => [
                                                                                                                '[Lex-59]',
                                                                                                                '[Lex-60]',
                                                                                                                'before or after'
                                                                                                              ],
                                                                                                     'start' => 3513,
                                                                                                     'subkey' => 1,
                                                                                                     'xrlid' => 'pause specification@3513+58'
                                                                                                   },
                                        'prediction event declaration,[Lex-22],event initialization,[Lex-23],[Lex-24],symbol name' => {
                                                                                                                                        'action' => '[start,length,values]',
                                                                                                                                        'bless' => 'prediction_event_declaration',
                                                                                                                                        'length' => 99,
                                                                                                                                        'lhs' => 'prediction event declaration',
                                                                                                                                        'mask' => [
                                                                                                                                                    0,
                                                                                                                                                    1,
                                                                                                                                                    0,
                                                                                                                                                    0,
                                                                                                                                                    1
                                                                                                                                                  ],
                                                                                                                                        'name' => 'prediction event declaration',
                                                                                                                                        'rhs' => [
                                                                                                                                                   '[Lex-22]',
                                                                                                                                                   'event initialization',
                                                                                                                                                   '[Lex-23]',
                                                                                                                                                   '[Lex-24]',
                                                                                                                                                   'symbol name'
                                                                                                                                                 ],
                                                                                                                                        'start' => 1947,
                                                                                                                                        'subkey' => 1,
                                                                                                                                        'xrlid' => 'prediction event declaration@1947+99'
                                                                                                                                      },
                                        'priorities,alternatives' => {
                                                                       'action' => '[start,length,values]',
                                                                       'bless' => 'priorities',
                                                                       'length' => 69,
                                                                       'lhs' => 'priorities',
                                                                       'min' => 1,
                                                                       'name' => 'priorities',
                                                                       'proper' => '1',
                                                                       'rhs' => [
                                                                                  'alternatives'
                                                                                ],
                                                                       'separator' => 'op loosen',
                                                                       'start' => 2250,
                                                                       'subkey' => 0,
                                                                       'xrlid' => 'priorities@2250+69'
                                                                     },
                                        'priority rule,lhs,op declare,priorities' => {
                                                                                       'action' => '[start,length,values]',
                                                                                       'bless' => 'priority_rule',
                                                                                       'length' => 47,
                                                                                       'lhs' => 'priority rule',
                                                                                       'mask' => [
                                                                                                   1,
                                                                                                   1,
                                                                                                   1
                                                                                                 ],
                                                                                       'name' => 'priority rule',
                                                                                       'rhs' => [
                                                                                                  'lhs',
                                                                                                  'op declare',
                                                                                                  'priorities'
                                                                                                ],
                                                                                       'start' => 1427,
                                                                                       'subkey' => 1,
                                                                                       'xrlid' => 'priority rule@1427+47'
                                                                                     },
                                        'priority specification,[Lex-57],[Lex-58],signed integer' => {
                                                                                                       'action' => '[start,length,values]',
                                                                                                       'bless' => 'priority_specification',
                                                                                                       'length' => 63,
                                                                                                       'lhs' => 'priority specification',
                                                                                                       'mask' => [
                                                                                                                   0,
                                                                                                                   0,
                                                                                                                   1
                                                                                                                 ],
                                                                                                       'name' => 'priority specification',
                                                                                                       'rhs' => [
                                                                                                                  '[Lex-57]',
                                                                                                                  '[Lex-58]',
                                                                                                                  'signed integer'
                                                                                                                ],
                                                                                                       'start' => 3449,
                                                                                                       'subkey' => 1,
                                                                                                       'xrlid' => 'priority specification@3449+63'
                                                                                                     },
                                        'proper specification,[Lex-46],[Lex-47],boolean' => {
                                                                                              'action' => '[start,length,values]',
                                                                                              'bless' => 'proper_specification',
                                                                                              'length' => 50,
                                                                                              'lhs' => 'proper specification',
                                                                                              'mask' => [
                                                                                                          0,
                                                                                                          0,
                                                                                                          1
                                                                                                        ],
                                                                                              'name' => 'proper specification',
                                                                                              'rhs' => [
                                                                                                         '[Lex-46]',
                                                                                                         '[Lex-47]',
                                                                                                         'boolean'
                                                                                                       ],
                                                                                              'start' => 3142,
                                                                                              'subkey' => 1,
                                                                                              'xrlid' => 'proper specification@3142+50'
                                                                                            },
                                        'quantified rule,lhs,op declare,single symbol,quantifier,adverb list' => {
                                                                                                                   'action' => '[start,length,values]',
                                                                                                                   'bless' => 'quantified_rule',
                                                                                                                   'length' => 79,
                                                                                                                   'lhs' => 'quantified rule',
                                                                                                                   'mask' => [
                                                                                                                               1,
                                                                                                                               1,
                                                                                                                               1,
                                                                                                                               1,
                                                                                                                               1
                                                                                                                             ],
                                                                                                                   'name' => 'quantified rule',
                                                                                                                   'rhs' => [
                                                                                                                              'lhs',
                                                                                                                              'op declare',
                                                                                                                              'single symbol',
                                                                                                                              'quantifier',
                                                                                                                              'adverb list'
                                                                                                                            ],
                                                                                                                   'start' => 1523,
                                                                                                                   'subkey' => 1,
                                                                                                                   'xrlid' => 'quantified rule@1523+79'
                                                                                                                 },
                                        'quantifier,[Lex-72]' => {
                                                                   'action' => '[start,length,values]',
                                                                   'bless' => 'quantifier',
                                                                   'length' => 24,
                                                                   'lhs' => 'quantifier',
                                                                   'mask' => [
                                                                               1
                                                                             ],
                                                                   'name' => 'quantifier',
                                                                   'rhs' => [
                                                                              '[Lex-72]'
                                                                            ],
                                                                   'start' => 5359,
                                                                   'subkey' => 1,
                                                                   'xrlid' => 'quantifier@5359+24'
                                                                 },
                                        'quantifier,[Lex-73]' => {
                                                                   'action' => '[start,length,values]',
                                                                   'bless' => 'quantifier',
                                                                   'length' => 3,
                                                                   'lhs' => 'quantifier',
                                                                   'mask' => [
                                                                               1
                                                                             ],
                                                                   'name' => 'quantifier',
                                                                   'rhs' => [
                                                                              '[Lex-73]'
                                                                            ],
                                                                   'start' => 5380,
                                                                   'subkey' => 2,
                                                                   'xrlid' => 'quantifier@5359+24'
                                                                 },
                                        'rank specification,[Lex-48],[Lex-49],signed integer' => {
                                                                                                   'action' => '[start,length,values]',
                                                                                                   'bless' => 'rank_specification',
                                                                                                   'length' => 55,
                                                                                                   'lhs' => 'rank specification',
                                                                                                   'mask' => [
                                                                                                               0,
                                                                                                               0,
                                                                                                               1
                                                                                                             ],
                                                                                                   'name' => 'rank specification',
                                                                                                   'rhs' => [
                                                                                                              '[Lex-48]',
                                                                                                              '[Lex-49]',
                                                                                                              'signed integer'
                                                                                                            ],
                                                                                                   'start' => 3193,
                                                                                                   'subkey' => 1,
                                                                                                   'xrlid' => 'rank specification@3193+55'
                                                                                                 },
                                        'rhs primary list,rhs primary' => {
                                                                            'action' => '[start,length,values]',
                                                                            'bless' => 'rhs_primary_list',
                                                                            'length' => 37,
                                                                            'lhs' => 'rhs primary list',
                                                                            'min' => 1,
                                                                            'name' => 'rhs primary list',
                                                                            'rhs' => [
                                                                                       'rhs primary'
                                                                                     ],
                                                                            'start' => 4504,
                                                                            'subkey' => 0,
                                                                            'xrlid' => 'rhs primary list@4504+37'
                                                                          },
                                        'rhs primary,parenthesized rhs primary list' => {
                                                                                          'action' => '[start,length,values]',
                                                                                          'bless' => 'rhs_primary',
                                                                                          'length' => 50,
                                                                                          'lhs' => 'rhs primary',
                                                                                          'mask' => [
                                                                                                      1
                                                                                                    ],
                                                                                          'name' => 'rhs primary',
                                                                                          'rhs' => [
                                                                                                     'parenthesized rhs primary list'
                                                                                                   ],
                                                                                          'start' => 4385,
                                                                                          'subkey' => 1,
                                                                                          'xrlid' => 'rhs primary@4385+50'
                                                                                        },
                                        'rhs primary,single quoted string' => {
                                                                                'action' => '[start,length,values]',
                                                                                'bless' => 'rhs_primary',
                                                                                'length' => 40,
                                                                                'lhs' => 'rhs primary',
                                                                                'mask' => [
                                                                                            1
                                                                                          ],
                                                                                'name' => 'rhs primary',
                                                                                'rhs' => [
                                                                                           'single quoted string'
                                                                                         ],
                                                                                'start' => 4344,
                                                                                'subkey' => 1,
                                                                                'xrlid' => 'rhs primary@4344+40'
                                                                              },
                                        'rhs primary,single symbol' => {
                                                                         'action' => '[start,length,values]',
                                                                         'bless' => 'rhs_primary',
                                                                         'length' => 33,
                                                                         'lhs' => 'rhs primary',
                                                                         'mask' => [
                                                                                     1
                                                                                   ],
                                                                         'name' => 'rhs primary',
                                                                         'rhs' => [
                                                                                    'single symbol'
                                                                                  ],
                                                                         'start' => 4310,
                                                                         'subkey' => 1,
                                                                         'xrlid' => 'rhs primary@4310+33'
                                                                       },
                                        'rhs,rhs primary' => {
                                                               'action' => '[start,length,values]',
                                                               'bless' => 'rhs',
                                                               'length' => 22,
                                                               'lhs' => 'rhs',
                                                               'min' => 1,
                                                               'name' => 'rhs',
                                                               'rhs' => [
                                                                          'rhs primary'
                                                                        ],
                                                               'start' => 4287,
                                                               'subkey' => 0,
                                                               'xrlid' => 'rhs@4287+22'
                                                             },
                                        'right association,[Lex-38],[Lex-39],[Lex-40]' => {
                                                                                            'action' => '[start,length,values]',
                                                                                            'bless' => 'right_association',
                                                                                            'length' => 46,
                                                                                            'lhs' => 'right association',
                                                                                            'mask' => [
                                                                                                        0,
                                                                                                        0,
                                                                                                        0
                                                                                                      ],
                                                                                            'name' => 'right association',
                                                                                            'rhs' => [
                                                                                                       '[Lex-38]',
                                                                                                       '[Lex-39]',
                                                                                                       '[Lex-40]'
                                                                                                     ],
                                                                                            'start' => 2983,
                                                                                            'subkey' => 1,
                                                                                            'xrlid' => 'right association@2983+46'
                                                                                          },
                                        'separator specification,[Lex-44],[Lex-45],single symbol' => {
                                                                                                       'action' => '[start,length,values]',
                                                                                                       'bless' => 'separator_specification',
                                                                                                       'length' => 64,
                                                                                                       'lhs' => 'separator specification',
                                                                                                       'mask' => [
                                                                                                                   0,
                                                                                                                   0,
                                                                                                                   1
                                                                                                                 ],
                                                                                                       'name' => 'separator specification',
                                                                                                       'rhs' => [
                                                                                                                  '[Lex-44]',
                                                                                                                  '[Lex-45]',
                                                                                                                  'single symbol'
                                                                                                                ],
                                                                                                       'start' => 3077,
                                                                                                       'subkey' => 1,
                                                                                                       'xrlid' => 'separator specification@3077+64'
                                                                                                     },
                                        'single symbol,character class' => {
                                                                             'action' => '[start,length,values]',
                                                                             'bless' => 'single_symbol',
                                                                             'length' => 17,
                                                                             'lhs' => 'single symbol',
                                                                             'mask' => [
                                                                                         1
                                                                                       ],
                                                                             'name' => 'single symbol',
                                                                             'rhs' => [
                                                                                        'character class'
                                                                                      ],
                                                                             'start' => 4577,
                                                                             'subkey' => 2,
                                                                             'xrlid' => 'single symbol@4542+52'
                                                                           },
                                        'single symbol,symbol' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'single_symbol',
                                                                    'length' => 52,
                                                                    'lhs' => 'single symbol',
                                                                    'mask' => [
                                                                                1
                                                                              ],
                                                                    'name' => 'single symbol',
                                                                    'rhs' => [
                                                                               'symbol'
                                                                             ],
                                                                    'start' => 4542,
                                                                    'subkey' => 1,
                                                                    'xrlid' => 'single symbol@4542+52'
                                                                  },
                                        'start rule,[Lex-3],op declare bnf,symbol' => {
                                                                                        'action' => '[start,length,values]',
                                                                                        'bless' => 'start_rule',
                                                                                        'length' => 51,
                                                                                        'lhs' => 'start rule',
                                                                                        'mask' => [
                                                                                                    0,
                                                                                                    0,
                                                                                                    1
                                                                                                  ],
                                                                                        'name' => 'start rule',
                                                                                        'rhs' => [
                                                                                                   '[Lex-3]',
                                                                                                   'op declare bnf',
                                                                                                   'symbol'
                                                                                                 ],
                                                                                        'start' => 1124,
                                                                                        'subkey' => 1,
                                                                                        'xrlid' => 'start rule@1124+51'
                                                                                      },
                                        'start rule,[Lex-4],[Lex-5],[Lex-6],symbol' => {
                                                                                         'action' => '[start,length,values]',
                                                                                         'bless' => 'start_rule',
                                                                                         'length' => 47,
                                                                                         'lhs' => 'start rule',
                                                                                         'mask' => [
                                                                                                     0,
                                                                                                     0,
                                                                                                     0,
                                                                                                     1
                                                                                                   ],
                                                                                         'name' => 'start rule',
                                                                                         'rhs' => [
                                                                                                    '[Lex-4]',
                                                                                                    '[Lex-5]',
                                                                                                    '[Lex-6]',
                                                                                                    'symbol'
                                                                                                  ],
                                                                                         'start' => 1176,
                                                                                         'subkey' => 1,
                                                                                         'xrlid' => 'start rule@1176+47'
                                                                                       },
                                        'statement group,[Lex-1],statements,[Lex-2]' => {
                                                                                          'action' => '[start,length,values]',
                                                                                          'bless' => 'statement_group',
                                                                                          'length' => 42,
                                                                                          'lhs' => 'statement group',
                                                                                          'mask' => [
                                                                                                      0,
                                                                                                      1,
                                                                                                      1
                                                                                                    ],
                                                                                          'name' => 'statement group',
                                                                                          'rhs' => [
                                                                                                     '[Lex-1]',
                                                                                                     'statements',
                                                                                                     '[Lex-2]'
                                                                                                   ],
                                                                                          'start' => 1081,
                                                                                          'subkey' => 1,
                                                                                          'xrlid' => 'statement group@1081+42'
                                                                                        },
                                        'statement,completion event declaration' => {
                                                                                      'action' => '[start,length,values]',
                                                                                      'bless' => 'statement',
                                                                                      'length' => 30,
                                                                                      'lhs' => 'statement',
                                                                                      'mask' => [
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'statement',
                                                                                      'rhs' => [
                                                                                                 'completion event declaration'
                                                                                               ],
                                                                                      'start' => 929,
                                                                                      'subkey' => 12,
                                                                                      'xrlid' => 'statement@685+369'
                                                                                    },
                                        'statement,default rule' => {
                                                                      'action' => '[start,length,values]',
                                                                      'bless' => 'statement',
                                                                      'length' => 14,
                                                                      'lhs' => 'statement',
                                                                      'mask' => [
                                                                                  1
                                                                                ],
                                                                      'name' => 'statement',
                                                                      'rhs' => [
                                                                                 'default rule'
                                                                               ],
                                                                      'start' => 829,
                                                                      'subkey' => 8,
                                                                      'xrlid' => 'statement@685+369'
                                                                    },
                                        'statement,discard default statement' => {
                                                                                   'action' => '[start,length,values]',
                                                                                   'bless' => 'statement',
                                                                                   'length' => 27,
                                                                                   'lhs' => 'statement',
                                                                                   'mask' => [
                                                                                               1
                                                                                             ],
                                                                                   'name' => 'statement',
                                                                                   'rhs' => [
                                                                                              'discard default statement'
                                                                                            ],
                                                                                   'start' => 879,
                                                                                   'subkey' => 10,
                                                                                   'xrlid' => 'statement@685+369'
                                                                                 },
                                        'statement,discard rule' => {
                                                                      'action' => '[start,length,values]',
                                                                      'bless' => 'statement',
                                                                      'length' => 14,
                                                                      'lhs' => 'statement',
                                                                      'mask' => [
                                                                                  1
                                                                                ],
                                                                      'name' => 'statement',
                                                                      'rhs' => [
                                                                                 'discard rule'
                                                                               ],
                                                                      'start' => 812,
                                                                      'subkey' => 7,
                                                                      'xrlid' => 'statement@685+369'
                                                                    },
                                        'statement,empty rule' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'statement',
                                                                    'length' => 12,
                                                                    'lhs' => 'statement',
                                                                    'mask' => [
                                                                                1
                                                                              ],
                                                                    'name' => 'statement',
                                                                    'rhs' => [
                                                                               'empty rule'
                                                                             ],
                                                                    'start' => 714,
                                                                    'subkey' => 2,
                                                                    'xrlid' => 'statement@685+369'
                                                                  },
                                        'statement,inaccessible statement' => {
                                                                                'action' => '[start,length,values]',
                                                                                'bless' => 'statement',
                                                                                'length' => 24,
                                                                                'lhs' => 'statement',
                                                                                'mask' => [
                                                                                            1
                                                                                          ],
                                                                                'name' => 'statement',
                                                                                'rhs' => [
                                                                                           'inaccessible statement'
                                                                                         ],
                                                                                'start' => 1030,
                                                                                'subkey' => 15,
                                                                                'xrlid' => 'statement@685+369'
                                                                              },
                                        'statement,lexeme default statement' => {
                                                                                  'action' => '[start,length,values]',
                                                                                  'bless' => 'statement',
                                                                                  'length' => 26,
                                                                                  'lhs' => 'statement',
                                                                                  'mask' => [
                                                                                              1
                                                                                            ],
                                                                                  'name' => 'statement',
                                                                                  'rhs' => [
                                                                                             'lexeme default statement'
                                                                                           ],
                                                                                  'start' => 848,
                                                                                  'subkey' => 9,
                                                                                  'xrlid' => 'statement@685+369'
                                                                                },
                                        'statement,lexeme rule' => {
                                                                     'action' => '[start,length,values]',
                                                                     'bless' => 'statement',
                                                                     'length' => 13,
                                                                     'lhs' => 'statement',
                                                                     'mask' => [
                                                                                 1
                                                                               ],
                                                                     'name' => 'statement',
                                                                     'rhs' => [
                                                                                'lexeme rule'
                                                                              ],
                                                                     'start' => 911,
                                                                     'subkey' => 11,
                                                                     'xrlid' => 'statement@685+369'
                                                                   },
                                        'statement,null statement' => {
                                                                        'action' => '[start,length,values]',
                                                                        'bless' => 'statement',
                                                                        'length' => 16,
                                                                        'lhs' => 'statement',
                                                                        'mask' => [
                                                                                    1
                                                                                  ],
                                                                        'name' => 'statement',
                                                                        'rhs' => [
                                                                                   'null statement'
                                                                                 ],
                                                                        'start' => 731,
                                                                        'subkey' => 3,
                                                                        'xrlid' => 'statement@685+369'
                                                                      },
                                        'statement,nulled event declaration' => {
                                                                                  'action' => '[start,length,values]',
                                                                                  'bless' => 'statement',
                                                                                  'length' => 26,
                                                                                  'lhs' => 'statement',
                                                                                  'mask' => [
                                                                                              1
                                                                                            ],
                                                                                  'name' => 'statement',
                                                                                  'rhs' => [
                                                                                             'nulled event declaration'
                                                                                           ],
                                                                                  'start' => 964,
                                                                                  'subkey' => 13,
                                                                                  'xrlid' => 'statement@685+369'
                                                                                },
                                        'statement,prediction event declaration' => {
                                                                                      'action' => '[start,length,values]',
                                                                                      'bless' => 'statement',
                                                                                      'length' => 30,
                                                                                      'lhs' => 'statement',
                                                                                      'mask' => [
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'statement',
                                                                                      'rhs' => [
                                                                                                 'prediction event declaration'
                                                                                               ],
                                                                                      'start' => 995,
                                                                                      'subkey' => 14,
                                                                                      'xrlid' => 'statement@685+369'
                                                                                    },
                                        'statement,priority rule' => {
                                                                       'action' => '[start,length,values]',
                                                                       'bless' => 'statement',
                                                                       'length' => 15,
                                                                       'lhs' => 'statement',
                                                                       'mask' => [
                                                                                   1
                                                                                 ],
                                                                       'name' => 'statement',
                                                                       'rhs' => [
                                                                                  'priority rule'
                                                                                ],
                                                                       'start' => 772,
                                                                       'subkey' => 5,
                                                                       'xrlid' => 'statement@685+369'
                                                                     },
                                        'statement,quantified rule' => {
                                                                         'action' => '[start,length,values]',
                                                                         'bless' => 'statement',
                                                                         'length' => 17,
                                                                         'lhs' => 'statement',
                                                                         'mask' => [
                                                                                     1
                                                                                   ],
                                                                         'name' => 'statement',
                                                                         'rhs' => [
                                                                                    'quantified rule'
                                                                                  ],
                                                                         'start' => 790,
                                                                         'subkey' => 6,
                                                                         'xrlid' => 'statement@685+369'
                                                                       },
                                        'statement,start rule' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'statement',
                                                                    'length' => 369,
                                                                    'lhs' => 'statement',
                                                                    'mask' => [
                                                                                1
                                                                              ],
                                                                    'name' => 'statement',
                                                                    'rhs' => [
                                                                               'start rule'
                                                                             ],
                                                                    'start' => 685,
                                                                    'subkey' => 1,
                                                                    'xrlid' => 'statement@685+369'
                                                                  },
                                        'statement,statement group' => {
                                                                         'action' => '[start,length,values]',
                                                                         'bless' => 'statement',
                                                                         'length' => 17,
                                                                         'lhs' => 'statement',
                                                                         'mask' => [
                                                                                     1
                                                                                   ],
                                                                         'name' => 'statement',
                                                                         'rhs' => [
                                                                                    'statement group'
                                                                                  ],
                                                                         'start' => 750,
                                                                         'subkey' => 4,
                                                                         'xrlid' => 'statement@685+369'
                                                                       },
                                        'statements,statement' => {
                                                                    'action' => '[start,length,values]',
                                                                    'bless' => 'statements',
                                                                    'length' => 25,
                                                                    'lhs' => 'statements',
                                                                    'min' => 1,
                                                                    'name' => 'statements',
                                                                    'rhs' => [
                                                                               'statement'
                                                                             ],
                                                                    'start' => 659,
                                                                    'subkey' => 0,
                                                                    'xrlid' => 'statements@659+25'
                                                                  },
                                        'symbol name,bare name' => {
                                                                     'action' => '[start,length,values]',
                                                                     'bless' => 'symbol_name',
                                                                     'length' => 29,
                                                                     'lhs' => 'symbol name',
                                                                     'mask' => [
                                                                                 1
                                                                               ],
                                                                     'name' => 'symbol name',
                                                                     'rhs' => [
                                                                                'bare name'
                                                                              ],
                                                                     'start' => 4620,
                                                                     'subkey' => 1,
                                                                     'xrlid' => 'symbol name@4620+29'
                                                                   },
                                        'symbol name,bracketed name' => {
                                                                          'action' => '[start,length,values]',
                                                                          'bless' => 'symbol_name',
                                                                          'length' => 34,
                                                                          'lhs' => 'symbol name',
                                                                          'mask' => [
                                                                                      1
                                                                                    ],
                                                                          'name' => 'symbol name',
                                                                          'rhs' => [
                                                                                     'bracketed name'
                                                                                   ],
                                                                          'start' => 4650,
                                                                          'subkey' => 1,
                                                                          'xrlid' => 'symbol name@4650+34'
                                                                        },
                                        'symbol,symbol name' => {
                                                                  'action' => '[start,length,values]',
                                                                  'bless' => 'symbol',
                                                                  'length' => 24,
                                                                  'lhs' => 'symbol',
                                                                  'mask' => [
                                                                              1
                                                                            ],
                                                                  'name' => 'symbol',
                                                                  'rhs' => [
                                                                             'symbol name'
                                                                           ],
                                                                  'start' => 4595,
                                                                  'subkey' => 1,
                                                                  'xrlid' => 'symbol@4595+24'
                                                                }
                                      },
                              'L0' => {
                                        'Perl identifier,[[\\w]]' => {
                                                                       'length' => 25,
                                                                       'lhs' => 'Perl identifier',
                                                                       'min' => 1,
                                                                       'name' => 'Perl identifier',
                                                                       'rhs' => [
                                                                                  '[[\\w]]'
                                                                                ],
                                                                       'start' => 5868,
                                                                       'subkey' => 0,
                                                                       'xrlid' => 'Perl identifier@5868+25'
                                                                     },
                                        'Perl name,Perl identifier' => {
                                                                         'length' => 72,
                                                                         'lhs' => 'Perl name',
                                                                         'min' => 1,
                                                                         'name' => 'Perl name',
                                                                         'proper' => '1',
                                                                         'rhs' => [
                                                                                    'Perl identifier'
                                                                                  ],
                                                                         'separator' => 'double colon',
                                                                         'start' => 5916,
                                                                         'subkey' => 0,
                                                                         'xrlid' => 'Perl name@5916+72'
                                                                       },
                                        '[:discard],hash comment' => {
                                                                       'length' => 25,
                                                                       'lhs' => '[:discard]',
                                                                       'rhs' => [
                                                                                  'hash comment'
                                                                                ],
                                                                       'start' => 4853,
                                                                       'subkey' => 0,
                                                                       'symbol_as_event' => 'hash comment'
                                                                     },
                                        '[:discard],whitespace' => {
                                                                     'length' => 21,
                                                                     'lhs' => '[:discard]',
                                                                     'rhs' => [
                                                                                'whitespace'
                                                                              ],
                                                                     'start' => 4794,
                                                                     'subkey' => 0,
                                                                     'symbol_as_event' => 'whitespace'
                                                                   },
                                        '[Lex-0],[[\\;]]' => {
                                                               'length' => 3,
                                                               'lhs' => '[Lex-0]',
                                                               'mask' => [
                                                                           1
                                                                         ],
                                                               'rhs' => [
                                                                          '[[\\;]]'
                                                                        ],
                                                               'start' => 1077,
                                                               'subkey' => 0
                                                             },
                                        '[Lex-10],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-10]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 1336,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-11],[[d]],[[i]],[[s]],[[c]],[[a]],[[r]],[[d]]' => {
                                                                                                  'length' => 9,
                                                                                                  'lhs' => '[Lex-11]',
                                                                                                  'mask' => [
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1
                                                                                                            ],
                                                                                                  'rhs' => [
                                                                                                             '[[d]]',
                                                                                                             '[[i]]',
                                                                                                             '[[s]]',
                                                                                                             '[[c]]',
                                                                                                             '[[a]]',
                                                                                                             '[[r]]',
                                                                                                             '[[d]]'
                                                                                                           ],
                                                                                                  'start' => 1388,
                                                                                                  'subkey' => 0
                                                                                                },
                                        '[Lex-12],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]' => {
                                                                                                  'length' => 9,
                                                                                                  'lhs' => '[Lex-12]',
                                                                                                  'mask' => [
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1
                                                                                                            ],
                                                                                                  'rhs' => [
                                                                                                             '[[d]]',
                                                                                                             '[[e]]',
                                                                                                             '[[f]]',
                                                                                                             '[[a]]',
                                                                                                             '[[u]]',
                                                                                                             '[[l]]',
                                                                                                             '[[t]]'
                                                                                                           ],
                                                                                                  'start' => 1398,
                                                                                                  'subkey' => 0
                                                                                                },
                                        '[Lex-13],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-13]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 1408,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-14],[[\\:]],[[d]],[[i]],[[s]],[[c]],[[a]],[[r]],[[d]]' => {
                                                                                                          'length' => 10,
                                                                                                          'lhs' => '[Lex-14]',
                                                                                                          'mask' => [
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1
                                                                                                                    ],
                                                                                                          'rhs' => [
                                                                                                                     '[[\\:]]',
                                                                                                                     '[[d]]',
                                                                                                                     '[[i]]',
                                                                                                                     '[[s]]',
                                                                                                                     '[[c]]',
                                                                                                                     '[[a]]',
                                                                                                                     '[[r]]',
                                                                                                                     '[[d]]'
                                                                                                                   ],
                                                                                                          'start' => 1623,
                                                                                                          'subkey' => 0
                                                                                                        },
                                        '[Lex-15],[[\\:]],[[l]],[[e]],[[x]],[[e]],[[m]],[[e]]' => {
                                                                                                    'length' => 9,
                                                                                                    'lhs' => '[Lex-15]',
                                                                                                    'mask' => [
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1
                                                                                                              ],
                                                                                                    'rhs' => [
                                                                                                               '[[\\:]]',
                                                                                                               '[[l]]',
                                                                                                               '[[e]]',
                                                                                                               '[[x]]',
                                                                                                               '[[e]]',
                                                                                                               '[[m]]',
                                                                                                               '[[e]]'
                                                                                                             ],
                                                                                                    'start' => 1703,
                                                                                                    'subkey' => 0
                                                                                                  },
                                        '[Lex-16],[[e]],[[v]],[[e]],[[n]],[[t]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-16]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[e]]',
                                                                                                 '[[v]]',
                                                                                                 '[[e]]',
                                                                                                 '[[n]]',
                                                                                                 '[[t]]'
                                                                                               ],
                                                                                      'start' => 1790,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-17],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-17]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 1823,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-18],[[c]],[[o]],[[m]],[[p]],[[l]],[[e]],[[t]],[[e]],[[d]]' => {
                                                                                                              'length' => 11,
                                                                                                              'lhs' => '[Lex-18]',
                                                                                                              'mask' => [
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1
                                                                                                                        ],
                                                                                                              'rhs' => [
                                                                                                                         '[[c]]',
                                                                                                                         '[[o]]',
                                                                                                                         '[[m]]',
                                                                                                                         '[[p]]',
                                                                                                                         '[[l]]',
                                                                                                                         '[[e]]',
                                                                                                                         '[[t]]',
                                                                                                                         '[[e]]',
                                                                                                                         '[[d]]'
                                                                                                                       ],
                                                                                                              'start' => 1827,
                                                                                                              'subkey' => 0
                                                                                                            },
                                        '[Lex-19],[[e]],[[v]],[[e]],[[n]],[[t]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-19]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[e]]',
                                                                                                 '[[v]]',
                                                                                                 '[[e]]',
                                                                                                 '[[n]]',
                                                                                                 '[[t]]'
                                                                                               ],
                                                                                      'start' => 1886,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-1],[[\\{]]' => {
                                                               'length' => 3,
                                                               'lhs' => '[Lex-1]',
                                                               'mask' => [
                                                                           1
                                                                         ],
                                                               'rhs' => [
                                                                          '[[\\{]]'
                                                                        ],
                                                               'start' => 1104,
                                                               'subkey' => 0
                                                             },
                                        '[Lex-20],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-20]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 1919,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-21],[[n]],[[u]],[[l]],[[l]],[[e]],[[d]]' => {
                                                                                            'length' => 8,
                                                                                            'lhs' => '[Lex-21]',
                                                                                            'mask' => [
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1
                                                                                                      ],
                                                                                            'rhs' => [
                                                                                                       '[[n]]',
                                                                                                       '[[u]]',
                                                                                                       '[[l]]',
                                                                                                       '[[l]]',
                                                                                                       '[[e]]',
                                                                                                       '[[d]]'
                                                                                                     ],
                                                                                            'start' => 1923,
                                                                                            'subkey' => 0
                                                                                          },
                                        '[Lex-22],[[e]],[[v]],[[e]],[[n]],[[t]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-22]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[e]]',
                                                                                                 '[[v]]',
                                                                                                 '[[e]]',
                                                                                                 '[[n]]',
                                                                                                 '[[t]]'
                                                                                               ],
                                                                                      'start' => 1983,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-23],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-23]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 2016,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-24],[[p]],[[r]],[[e]],[[d]],[[i]],[[c]],[[t]],[[e]],[[d]]' => {
                                                                                                              'length' => 11,
                                                                                                              'lhs' => '[Lex-24]',
                                                                                                              'mask' => [
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1
                                                                                                                        ],
                                                                                                              'rhs' => [
                                                                                                                         '[[p]]',
                                                                                                                         '[[r]]',
                                                                                                                         '[[e]]',
                                                                                                                         '[[d]]',
                                                                                                                         '[[i]]',
                                                                                                                         '[[c]]',
                                                                                                                         '[[t]]',
                                                                                                                         '[[e]]',
                                                                                                                         '[[d]]'
                                                                                                                       ],
                                                                                                              'start' => 2020,
                                                                                                              'subkey' => 0
                                                                                                            },
                                        '[Lex-25],[[i]],[[n]],[[a]],[[c]],[[c]],[[e]],[[s]],[[s]],[[i]],[[b]],[[l]],[[e]]' => {
                                                                                                                                'length' => 14,
                                                                                                                                'lhs' => '[Lex-25]',
                                                                                                                                'mask' => [
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1,
                                                                                                                                            1
                                                                                                                                          ],
                                                                                                                                'rhs' => [
                                                                                                                                           '[[i]]',
                                                                                                                                           '[[n]]',
                                                                                                                                           '[[a]]',
                                                                                                                                           '[[c]]',
                                                                                                                                           '[[c]]',
                                                                                                                                           '[[e]]',
                                                                                                                                           '[[s]]',
                                                                                                                                           '[[s]]',
                                                                                                                                           '[[i]]',
                                                                                                                                           '[[b]]',
                                                                                                                                           '[[l]]',
                                                                                                                                           '[[e]]'
                                                                                                                                         ],
                                                                                                                                'start' => 2078,
                                                                                                                                'subkey' => 0
                                                                                                                              },
                                        '[Lex-26],[[i]],[[s]]' => {
                                                                    'length' => 4,
                                                                    'lhs' => '[Lex-26]',
                                                                    'mask' => [
                                                                                1,
                                                                                1
                                                                              ],
                                                                    'rhs' => [
                                                                               '[[i]]',
                                                                               '[[s]]'
                                                                             ],
                                                                    'start' => 2093,
                                                                    'subkey' => 0
                                                                  },
                                        '[Lex-27],[[b]],[[y]]' => {
                                                                    'length' => 4,
                                                                    'lhs' => '[Lex-27]',
                                                                    'mask' => [
                                                                                1,
                                                                                1
                                                                              ],
                                                                    'rhs' => [
                                                                               '[[b]]',
                                                                               '[[y]]'
                                                                             ],
                                                                    'start' => 2125,
                                                                    'subkey' => 0
                                                                  },
                                        '[Lex-28],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]' => {
                                                                                                  'length' => 9,
                                                                                                  'lhs' => '[Lex-28]',
                                                                                                  'mask' => [
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1,
                                                                                                              1
                                                                                                            ],
                                                                                                  'rhs' => [
                                                                                                             '[[d]]',
                                                                                                             '[[e]]',
                                                                                                             '[[f]]',
                                                                                                             '[[a]]',
                                                                                                             '[[u]]',
                                                                                                             '[[l]]',
                                                                                                             '[[t]]'
                                                                                                           ],
                                                                                                  'start' => 2130,
                                                                                                  'subkey' => 0
                                                                                                },
                                        '[Lex-29],[[w]],[[a]],[[r]],[[n]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-29]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[w]]',
                                                                                           '[[a]]',
                                                                                           '[[r]]',
                                                                                           '[[n]]'
                                                                                         ],
                                                                                'start' => 2170,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-2],[[\\}]]' => {
                                                               'length' => 3,
                                                               'lhs' => '[Lex-2]',
                                                               'mask' => [
                                                                           1
                                                                         ],
                                                               'rhs' => [
                                                                          '[[\\}]]'
                                                                        ],
                                                               'start' => 1120,
                                                               'subkey' => 0
                                                             },
                                        '[Lex-30],[[o]],[[k]]' => {
                                                                    'length' => 4,
                                                                    'lhs' => '[Lex-30]',
                                                                    'mask' => [
                                                                                1,
                                                                                1
                                                                              ],
                                                                    'rhs' => [
                                                                               '[[o]]',
                                                                               '[[k]]'
                                                                             ],
                                                                    'start' => 2179,
                                                                    'subkey' => 0
                                                                  },
                                        '[Lex-31],[[f]],[[a]],[[t]],[[a]],[[l]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-31]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[f]]',
                                                                                                 '[[a]]',
                                                                                                 '[[t]]',
                                                                                                 '[[a]]',
                                                                                                 '[[l]]'
                                                                                               ],
                                                                                      'start' => 2186,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-32],[[\\,]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-32]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\,]]'
                                                                         ],
                                                                'start' => 2893,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-33],[[a]],[[c]],[[t]],[[i]],[[o]],[[n]]' => {
                                                                                            'length' => 8,
                                                                                            'lhs' => '[Lex-33]',
                                                                                            'mask' => [
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1
                                                                                                      ],
                                                                                            'rhs' => [
                                                                                                       '[[a]]',
                                                                                                       '[[c]]',
                                                                                                       '[[t]]',
                                                                                                       '[[i]]',
                                                                                                       '[[o]]',
                                                                                                       '[[n]]'
                                                                                                     ],
                                                                                            'start' => 2909,
                                                                                            'subkey' => 0
                                                                                          },
                                        '[Lex-34],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-34]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 2918,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-35],[[a]],[[s]],[[s]],[[o]],[[c]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-35]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[a]]',
                                                                                                 '[[s]]',
                                                                                                 '[[s]]',
                                                                                                 '[[o]]',
                                                                                                 '[[c]]'
                                                                                               ],
                                                                                      'start' => 2962,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-36],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-36]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 2970,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-37],[[l]],[[e]],[[f]],[[t]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-37]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[l]]',
                                                                                           '[[e]]',
                                                                                           '[[f]]',
                                                                                           '[[t]]'
                                                                                         ],
                                                                                'start' => 2975,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-38],[[a]],[[s]],[[s]],[[o]],[[c]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-38]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[a]]',
                                                                                                 '[[s]]',
                                                                                                 '[[s]]',
                                                                                                 '[[o]]',
                                                                                                 '[[c]]'
                                                                                               ],
                                                                                      'start' => 3008,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-39],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-39]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3016,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-3],[[\\:]],[[s]],[[t]],[[a]],[[r]],[[t]]' => {
                                                                                             'length' => 8,
                                                                                             'lhs' => '[Lex-3]',
                                                                                             'mask' => [
                                                                                                         1,
                                                                                                         1,
                                                                                                         1,
                                                                                                         1,
                                                                                                         1,
                                                                                                         1
                                                                                                       ],
                                                                                             'rhs' => [
                                                                                                        '[[\\:]]',
                                                                                                        '[[s]]',
                                                                                                        '[[t]]',
                                                                                                        '[[a]]',
                                                                                                        '[[r]]',
                                                                                                        '[[t]]'
                                                                                                      ],
                                                                                             'start' => 1142,
                                                                                             'subkey' => 0
                                                                                           },
                                        '[Lex-40],[[r]],[[i]],[[g]],[[h]],[[t]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-40]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[r]]',
                                                                                                 '[[i]]',
                                                                                                 '[[g]]',
                                                                                                 '[[h]]',
                                                                                                 '[[t]]'
                                                                                               ],
                                                                                      'start' => 3021,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-41],[[a]],[[s]],[[s]],[[o]],[[c]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-41]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[a]]',
                                                                                                 '[[s]]',
                                                                                                 '[[s]]',
                                                                                                 '[[o]]',
                                                                                                 '[[c]]'
                                                                                               ],
                                                                                      'start' => 3055,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-42],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-42]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3063,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-43],[[g]],[[r]],[[o]],[[u]],[[p]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-43]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[g]]',
                                                                                                 '[[r]]',
                                                                                                 '[[o]]',
                                                                                                 '[[u]]',
                                                                                                 '[[p]]'
                                                                                               ],
                                                                                      'start' => 3068,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-44],[[s]],[[e]],[[p]],[[a]],[[r]],[[a]],[[t]],[[o]],[[r]]' => {
                                                                                                              'length' => 11,
                                                                                                              'lhs' => '[Lex-44]',
                                                                                                              'mask' => [
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1,
                                                                                                                          1
                                                                                                                        ],
                                                                                                              'rhs' => [
                                                                                                                         '[[s]]',
                                                                                                                         '[[e]]',
                                                                                                                         '[[p]]',
                                                                                                                         '[[a]]',
                                                                                                                         '[[r]]',
                                                                                                                         '[[a]]',
                                                                                                                         '[[t]]',
                                                                                                                         '[[o]]',
                                                                                                                         '[[r]]'
                                                                                                                       ],
                                                                                                              'start' => 3108,
                                                                                                              'subkey' => 0
                                                                                                            },
                                        '[Lex-45],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-45]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3120,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-46],[[p]],[[r]],[[o]],[[p]],[[e]],[[r]]' => {
                                                                                            'length' => 8,
                                                                                            'lhs' => '[Lex-46]',
                                                                                            'mask' => [
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1,
                                                                                                        1
                                                                                                      ],
                                                                                            'rhs' => [
                                                                                                       '[[p]]',
                                                                                                       '[[r]]',
                                                                                                       '[[o]]',
                                                                                                       '[[p]]',
                                                                                                       '[[e]]',
                                                                                                       '[[r]]'
                                                                                                     ],
                                                                                            'start' => 3170,
                                                                                            'subkey' => 0
                                                                                          },
                                        '[Lex-47],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-47]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3179,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-48],[[r]],[[a]],[[n]],[[k]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-48]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[r]]',
                                                                                           '[[a]]',
                                                                                           '[[n]]',
                                                                                           '[[k]]'
                                                                                         ],
                                                                                'start' => 3219,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-49],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-49]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3226,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-4],[[s]],[[t]],[[a]],[[r]],[[t]]' => {
                                                                                     'length' => 7,
                                                                                     'lhs' => '[Lex-4]',
                                                                                     'mask' => [
                                                                                                 1,
                                                                                                 1,
                                                                                                 1,
                                                                                                 1,
                                                                                                 1
                                                                                               ],
                                                                                     'rhs' => [
                                                                                                '[[s]]',
                                                                                                '[[t]]',
                                                                                                '[[a]]',
                                                                                                '[[r]]',
                                                                                                '[[t]]'
                                                                                              ],
                                                                                     'start' => 1194,
                                                                                     'subkey' => 0
                                                                                   },
                                        '[Lex-50],[[n]],[[u]],[[l]],[[l]],[[\\-]],[[r]],[[a]],[[n]],[[k]],[[i]],[[n]],[[g]]' => {
                                                                                                                                  'length' => 14,
                                                                                                                                  'lhs' => '[Lex-50]',
                                                                                                                                  'mask' => [
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1,
                                                                                                                                              1
                                                                                                                                            ],
                                                                                                                                  'rhs' => [
                                                                                                                                             '[[n]]',
                                                                                                                                             '[[u]]',
                                                                                                                                             '[[l]]',
                                                                                                                                             '[[l]]',
                                                                                                                                             '[[\\-]]',
                                                                                                                                             '[[r]]',
                                                                                                                                             '[[a]]',
                                                                                                                                             '[[n]]',
                                                                                                                                             '[[k]]',
                                                                                                                                             '[[i]]',
                                                                                                                                             '[[n]]',
                                                                                                                                             '[[g]]'
                                                                                                                                           ],
                                                                                                                                  'start' => 3283,
                                                                                                                                  'subkey' => 0
                                                                                                                                },
                                        '[Lex-51],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-51]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3298,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-52],[[n]],[[u]],[[l]],[[l]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-52]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[n]]',
                                                                                           '[[u]]',
                                                                                           '[[l]]',
                                                                                           '[[l]]'
                                                                                         ],
                                                                                'start' => 3362,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-53],[[r]],[[a]],[[n]],[[k]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-53]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[r]]',
                                                                                           '[[a]]',
                                                                                           '[[n]]',
                                                                                           '[[k]]'
                                                                                         ],
                                                                                'start' => 3369,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-54],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-54]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3376,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-55],[[l]],[[o]],[[w]]' => {
                                                                          'length' => 5,
                                                                          'lhs' => '[Lex-55]',
                                                                          'mask' => [
                                                                                      1,
                                                                                      1,
                                                                                      1
                                                                                    ],
                                                                          'rhs' => [
                                                                                     '[[l]]',
                                                                                     '[[o]]',
                                                                                     '[[w]]'
                                                                                   ],
                                                                          'start' => 3434,
                                                                          'subkey' => 0
                                                                        },
                                        '[Lex-56],[[h]],[[i]],[[g]],[[h]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-56]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[h]]',
                                                                                           '[[i]]',
                                                                                           '[[g]]',
                                                                                           '[[h]]'
                                                                                         ],
                                                                                'start' => 3442,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-57],[[p]],[[r]],[[i]],[[o]],[[r]],[[i]],[[t]],[[y]]' => {
                                                                                                        'length' => 10,
                                                                                                        'lhs' => '[Lex-57]',
                                                                                                        'mask' => [
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1,
                                                                                                                    1
                                                                                                                  ],
                                                                                                        'rhs' => [
                                                                                                                   '[[p]]',
                                                                                                                   '[[r]]',
                                                                                                                   '[[i]]',
                                                                                                                   '[[o]]',
                                                                                                                   '[[r]]',
                                                                                                                   '[[i]]',
                                                                                                                   '[[t]]',
                                                                                                                   '[[y]]'
                                                                                                                 ],
                                                                                                        'start' => 3479,
                                                                                                        'subkey' => 0
                                                                                                      },
                                        '[Lex-58],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-58]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3490,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-59],[[p]],[[a]],[[u]],[[s]],[[e]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-59]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[p]]',
                                                                                                 '[[a]]',
                                                                                                 '[[u]]',
                                                                                                 '[[s]]',
                                                                                                 '[[e]]'
                                                                                               ],
                                                                                      'start' => 3540,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-5],[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]' => {
                                                                                           'length' => 8,
                                                                                           'lhs' => '[Lex-5]',
                                                                                           'mask' => [
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1
                                                                                                     ],
                                                                                           'rhs' => [
                                                                                                      '[[s]]',
                                                                                                      '[[y]]',
                                                                                                      '[[m]]',
                                                                                                      '[[b]]',
                                                                                                      '[[o]]',
                                                                                                      '[[l]]'
                                                                                                    ],
                                                                                           'start' => 1202,
                                                                                           'subkey' => 0
                                                                                         },
                                        '[Lex-60],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-60]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3548,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-61],[[e]],[[v]],[[e]],[[n]],[[t]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-61]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[e]]',
                                                                                                 '[[v]]',
                                                                                                 '[[e]]',
                                                                                                 '[[n]]',
                                                                                                 '[[t]]'
                                                                                               ],
                                                                                      'start' => 3600,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-62],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-62]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3608,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-63],[[\\=]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-63]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\=]]'
                                                                         ],
                                                                'start' => 3722,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-64],[[o]],[[n]]' => {
                                                                    'length' => 4,
                                                                    'lhs' => '[Lex-64]',
                                                                    'mask' => [
                                                                                1,
                                                                                1
                                                                              ],
                                                                    'rhs' => [
                                                                               '[[o]]',
                                                                               '[[n]]'
                                                                             ],
                                                                    'start' => 3755,
                                                                    'subkey' => 0
                                                                  },
                                        '[Lex-65],[[o]],[[f]],[[f]]' => {
                                                                          'length' => 5,
                                                                          'lhs' => '[Lex-65]',
                                                                          'mask' => [
                                                                                      1,
                                                                                      1,
                                                                                      1
                                                                                    ],
                                                                          'rhs' => [
                                                                                     '[[o]]',
                                                                                     '[[f]]',
                                                                                     '[[f]]'
                                                                                   ],
                                                                          'start' => 3762,
                                                                          'subkey' => 0
                                                                        },
                                        '[Lex-66],[[b]],[[l]],[[e]],[[s]],[[s]]' => {
                                                                                      'length' => 7,
                                                                                      'lhs' => '[Lex-66]',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'rhs' => [
                                                                                                 '[[b]]',
                                                                                                 '[[l]]',
                                                                                                 '[[e]]',
                                                                                                 '[[s]]',
                                                                                                 '[[s]]'
                                                                                               ],
                                                                                      'start' => 3920,
                                                                                      'subkey' => 0
                                                                                    },
                                        '[Lex-67],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-67]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3928,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-68],[[n]],[[a]],[[m]],[[e]]' => {
                                                                                'length' => 6,
                                                                                'lhs' => '[Lex-68]',
                                                                                'mask' => [
                                                                                            1,
                                                                                            1,
                                                                                            1,
                                                                                            1
                                                                                          ],
                                                                                'rhs' => [
                                                                                           '[[n]]',
                                                                                           '[[a]]',
                                                                                           '[[m]]',
                                                                                           '[[e]]'
                                                                                         ],
                                                                                'start' => 3964,
                                                                                'subkey' => 0
                                                                              },
                                        '[Lex-69],[[\\=]],[[\\>]]' => {
                                                                        'length' => 4,
                                                                        'lhs' => '[Lex-69]',
                                                                        'mask' => [
                                                                                    1,
                                                                                    1
                                                                                  ],
                                                                        'rhs' => [
                                                                                   '[[\\=]]',
                                                                                   '[[\\>]]'
                                                                                 ],
                                                                        'start' => 3971,
                                                                        'subkey' => 0
                                                                      },
                                        '[Lex-6],[[i]],[[s]]' => {
                                                                   'length' => 4,
                                                                   'lhs' => '[Lex-6]',
                                                                   'mask' => [
                                                                               1,
                                                                               1
                                                                             ],
                                                                   'rhs' => [
                                                                              '[[i]]',
                                                                              '[[s]]'
                                                                            ],
                                                                   'start' => 1211,
                                                                   'subkey' => 0
                                                                 },
                                        '[Lex-70],[[\\(]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-70]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\(]]'
                                                                         ],
                                                                'start' => 4474,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-71],[[\\)]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-71]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\)]]'
                                                                         ],
                                                                'start' => 4499,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-72],[[\\*]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-72]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\*]]'
                                                                         ],
                                                                'start' => 5374,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-73],[[\\+]]' => {
                                                                'length' => 3,
                                                                'lhs' => '[Lex-73]',
                                                                'mask' => [
                                                                            1
                                                                          ],
                                                                'rhs' => [
                                                                           '[[\\+]]'
                                                                         ],
                                                                'start' => 5380,
                                                                'subkey' => 0
                                                              },
                                        '[Lex-7],[[\\:]],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]' => {
                                                                                                         'length' => 10,
                                                                                                         'lhs' => '[Lex-7]',
                                                                                                         'mask' => [
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1,
                                                                                                                     1
                                                                                                                   ],
                                                                                                         'rhs' => [
                                                                                                                    '[[\\:]]',
                                                                                                                    '[[d]]',
                                                                                                                    '[[e]]',
                                                                                                                    '[[f]]',
                                                                                                                    '[[a]]',
                                                                                                                    '[[u]]',
                                                                                                                    '[[l]]',
                                                                                                                    '[[t]]'
                                                                                                                  ],
                                                                                                         'start' => 1243,
                                                                                                         'subkey' => 0
                                                                                                       },
                                        '[Lex-8],[[l]],[[e]],[[x]],[[e]],[[m]],[[e]]' => {
                                                                                           'length' => 8,
                                                                                           'lhs' => '[Lex-8]',
                                                                                           'mask' => [
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1,
                                                                                                       1
                                                                                                     ],
                                                                                           'rhs' => [
                                                                                                      '[[l]]',
                                                                                                      '[[e]]',
                                                                                                      '[[x]]',
                                                                                                      '[[e]]',
                                                                                                      '[[m]]',
                                                                                                      '[[e]]'
                                                                                                    ],
                                                                                           'start' => 1317,
                                                                                           'subkey' => 0
                                                                                         },
                                        '[Lex-9],[[d]],[[e]],[[f]],[[a]],[[u]],[[l]],[[t]]' => {
                                                                                                 'length' => 9,
                                                                                                 'lhs' => '[Lex-9]',
                                                                                                 'mask' => [
                                                                                                             1,
                                                                                                             1,
                                                                                                             1,
                                                                                                             1,
                                                                                                             1,
                                                                                                             1,
                                                                                                             1
                                                                                                           ],
                                                                                                 'rhs' => [
                                                                                                            '[[d]]',
                                                                                                            '[[e]]',
                                                                                                            '[[f]]',
                                                                                                            '[[a]]',
                                                                                                            '[[u]]',
                                                                                                            '[[l]]',
                                                                                                            '[[t]]'
                                                                                                          ],
                                                                                                 'start' => 1326,
                                                                                                 'subkey' => 0
                                                                                               },
                                        'array descriptor left bracket,[[\\[]]' => {
                                                                                     'length' => 37,
                                                                                     'lhs' => 'array descriptor left bracket',
                                                                                     'mask' => [
                                                                                                 1
                                                                                               ],
                                                                                     'name' => 'array descriptor left bracket',
                                                                                     'rhs' => [
                                                                                                '[[\\[]]'
                                                                                              ],
                                                                                     'start' => 6270,
                                                                                     'subkey' => 1,
                                                                                     'xrlid' => 'array descriptor left bracket@6270+37'
                                                                                   },
                                        'array descriptor left bracket,[[\\[]],whitespace' => {
                                                                                                'length' => 48,
                                                                                                'lhs' => 'array descriptor left bracket',
                                                                                                'mask' => [
                                                                                                            1,
                                                                                                            1
                                                                                                          ],
                                                                                                'name' => 'array descriptor left bracket',
                                                                                                'rhs' => [
                                                                                                           '[[\\[]]',
                                                                                                           'whitespace'
                                                                                                         ],
                                                                                                'start' => 6309,
                                                                                                'subkey' => 1,
                                                                                                'xrlid' => 'array descriptor left bracket@6309+48'
                                                                                              },
                                        'array descriptor right bracket,[[\\]]]' => {
                                                                                      'length' => 38,
                                                                                      'lhs' => 'array descriptor right bracket',
                                                                                      'mask' => [
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'array descriptor right bracket',
                                                                                      'rhs' => [
                                                                                                 '[[\\]]]'
                                                                                               ],
                                                                                      'start' => 6358,
                                                                                      'subkey' => 1,
                                                                                      'xrlid' => 'array descriptor right bracket@6358+38'
                                                                                    },
                                        'array descriptor right bracket,whitespace,[[\\]]]' => {
                                                                                                 'length' => 49,
                                                                                                 'lhs' => 'array descriptor right bracket',
                                                                                                 'mask' => [
                                                                                                             1,
                                                                                                             1
                                                                                                           ],
                                                                                                 'name' => 'array descriptor right bracket',
                                                                                                 'rhs' => [
                                                                                                            'whitespace',
                                                                                                            '[[\\]]]'
                                                                                                          ],
                                                                                                 'start' => 6398,
                                                                                                 'subkey' => 1,
                                                                                                 'xrlid' => 'array descriptor right bracket@6398+49'
                                                                                               },
                                        'array descriptor,array descriptor left bracket,result item descriptor list,array descriptor right bracket' => {
                                                                                                                                                         'length' => 115,
                                                                                                                                                         'lhs' => 'array descriptor',
                                                                                                                                                         'mask' => [
                                                                                                                                                                     1,
                                                                                                                                                                     1,
                                                                                                                                                                     1
                                                                                                                                                                   ],
                                                                                                                                                         'name' => 'array descriptor',
                                                                                                                                                         'rhs' => [
                                                                                                                                                                    'array descriptor left bracket',
                                                                                                                                                                    'result item descriptor list',
                                                                                                                                                                    'array descriptor right bracket'
                                                                                                                                                                  ],
                                                                                                                                                         'start' => 6154,
                                                                                                                                                         'subkey' => 1,
                                                                                                                                                         'xrlid' => 'array descriptor@6154+115'
                                                                                                                                                       },
                                        'bare name,[[\\w]]' => {
                                                                 'length' => 19,
                                                                 'lhs' => 'bare name',
                                                                 'min' => 1,
                                                                 'name' => 'bare name',
                                                                 'rhs' => [
                                                                            '[[\\w]]'
                                                                          ],
                                                                 'start' => 5990,
                                                                 'subkey' => 0,
                                                                 'xrlid' => 'bare name@5990+19'
                                                               },
                                        'before or after,[[a]],[[f]],[[t]],[[e]],[[r]]' => {
                                                                                             'length' => 7,
                                                                                             'lhs' => 'before or after',
                                                                                             'mask' => [
                                                                                                         1,
                                                                                                         1,
                                                                                                         1,
                                                                                                         1,
                                                                                                         1
                                                                                                       ],
                                                                                             'name' => 'before or after',
                                                                                             'rhs' => [
                                                                                                        '[[a]]',
                                                                                                        '[[f]]',
                                                                                                        '[[t]]',
                                                                                                        '[[e]]',
                                                                                                        '[[r]]'
                                                                                                      ],
                                                                                             'start' => 5415,
                                                                                             'subkey' => 2,
                                                                                             'xrlid' => 'before or after@5384+38'
                                                                                           },
                                        'before or after,[[b]],[[e]],[[f]],[[o]],[[r]],[[e]]' => {
                                                                                                   'length' => 38,
                                                                                                   'lhs' => 'before or after',
                                                                                                   'mask' => [
                                                                                                               1,
                                                                                                               1,
                                                                                                               1,
                                                                                                               1,
                                                                                                               1,
                                                                                                               1
                                                                                                             ],
                                                                                                   'name' => 'before or after',
                                                                                                   'rhs' => [
                                                                                                              '[[b]]',
                                                                                                              '[[e]]',
                                                                                                              '[[f]]',
                                                                                                              '[[o]]',
                                                                                                              '[[r]]',
                                                                                                              '[[e]]'
                                                                                                            ],
                                                                                                   'start' => 5384,
                                                                                                   'subkey' => 1,
                                                                                                   'xrlid' => 'before or after@5384+38'
                                                                                                 },
                                        'boolean,[[01]]' => {
                                                              'length' => 14,
                                                              'lhs' => 'boolean',
                                                              'mask' => [
                                                                          1
                                                                        ],
                                                              'name' => 'boolean',
                                                              'rhs' => [
                                                                         '[[01]]'
                                                                       ],
                                                              'start' => 5505,
                                                              'subkey' => 1,
                                                              'xrlid' => 'boolean@5505+14'
                                                            },
                                        'bracketed name string,[[\\s\\w]]' => {
                                                                                'length' => 33,
                                                                                'lhs' => 'bracketed name string',
                                                                                'min' => 1,
                                                                                'name' => 'bracketed name string',
                                                                                'rhs' => [
                                                                                           '[[\\s\\w]]'
                                                                                         ],
                                                                                'start' => 6119,
                                                                                'subkey' => 0,
                                                                                'xrlid' => 'bracketed name string@6119+33'
                                                                              },
                                        'bracketed name,[[\\<]],bracketed name string,[[\\>]]' => {
                                                                                                    'length' => 50,
                                                                                                    'lhs' => 'bracketed name',
                                                                                                    'mask' => [
                                                                                                                1,
                                                                                                                1,
                                                                                                                1
                                                                                                              ],
                                                                                                    'name' => 'bracketed name',
                                                                                                    'rhs' => [
                                                                                                               '[[\\<]]',
                                                                                                               'bracketed name string',
                                                                                                               '[[\\>]]'
                                                                                                             ],
                                                                                                    'start' => 6068,
                                                                                                    'subkey' => 1,
                                                                                                    'xrlid' => 'bracketed name@6068+50'
                                                                                                  },
                                        'cc element,escaped cc character' => {
                                                                               'length' => 37,
                                                                               'lhs' => 'cc element',
                                                                               'mask' => [
                                                                                           1
                                                                                         ],
                                                                               'name' => 'cc element',
                                                                               'rhs' => [
                                                                                          'escaped cc character'
                                                                                        ],
                                                                               'start' => 7438,
                                                                               'subkey' => 1,
                                                                               'xrlid' => 'cc element@7438+37'
                                                                             },
                                        'cc element,negated posix char class' => {
                                                                                   'length' => 41,
                                                                                   'lhs' => 'cc element',
                                                                                   'mask' => [
                                                                                               1
                                                                                             ],
                                                                                   'name' => 'cc element',
                                                                                   'rhs' => [
                                                                                              'negated posix char class'
                                                                                            ],
                                                                                   'start' => 7562,
                                                                                   'subkey' => 1,
                                                                                   'xrlid' => 'cc element@7562+41'
                                                                                 },
                                        'cc element,posix char class' => {
                                                                           'length' => 33,
                                                                           'lhs' => 'cc element',
                                                                           'mask' => [
                                                                                       1
                                                                                     ],
                                                                           'name' => 'cc element',
                                                                           'rhs' => [
                                                                                      'posix char class'
                                                                                    ],
                                                                           'start' => 7528,
                                                                           'subkey' => 1,
                                                                           'xrlid' => 'cc element@7528+33'
                                                                         },
                                        'cc element,safe cc character' => {
                                                                            'length' => 34,
                                                                            'lhs' => 'cc element',
                                                                            'mask' => [
                                                                                        1
                                                                                      ],
                                                                            'name' => 'cc element',
                                                                            'rhs' => [
                                                                                       'safe cc character'
                                                                                     ],
                                                                            'start' => 7290,
                                                                            'subkey' => 1,
                                                                            'xrlid' => 'cc element@7290+34'
                                                                          },
                                        'cc elements,cc element' => {
                                                                      'length' => 29,
                                                                      'lhs' => 'cc elements',
                                                                      'min' => 1,
                                                                      'name' => 'cc elements',
                                                                      'rhs' => [
                                                                                 'cc element'
                                                                               ],
                                                                      'start' => 7260,
                                                                      'subkey' => 0,
                                                                      'xrlid' => 'cc elements@7260+29'
                                                                    },
                                        'character class modifier,[[\\:]],[[i]]' => {
                                                                                      'length' => 33,
                                                                                      'lhs' => 'character class modifier',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'character class modifier',
                                                                                      'rhs' => [
                                                                                                 '[[\\:]]',
                                                                                                 '[[i]]'
                                                                                               ],
                                                                                      'start' => 7697,
                                                                                      'subkey' => 1,
                                                                                      'xrlid' => 'character class modifier@7697+33'
                                                                                    },
                                        'character class modifier,[[\\:]],[[i]],[[c]]' => {
                                                                                            'length' => 34,
                                                                                            'lhs' => 'character class modifier',
                                                                                            'mask' => [
                                                                                                        1,
                                                                                                        1,
                                                                                                        1
                                                                                                      ],
                                                                                            'name' => 'character class modifier',
                                                                                            'rhs' => [
                                                                                                       '[[\\:]]',
                                                                                                       '[[i]]',
                                                                                                       '[[c]]'
                                                                                                     ],
                                                                                            'start' => 7662,
                                                                                            'subkey' => 1,
                                                                                            'xrlid' => 'character class modifier@7662+34'
                                                                                          },
                                        'character class modifiers,character class modifier' => {
                                                                                                  'length' => 57,
                                                                                                  'lhs' => 'character class modifiers',
                                                                                                  'min' => 0,
                                                                                                  'name' => 'character class modifiers',
                                                                                                  'rhs' => [
                                                                                                             'character class modifier'
                                                                                                           ],
                                                                                                  'start' => 7604,
                                                                                                  'subkey' => 0,
                                                                                                  'xrlid' => 'character class modifiers@7604+57'
                                                                                                },
                                        'character class,[[\\[]],cc elements,[[\\]]],character class modifiers' => {
                                                                                                                     'length' => 69,
                                                                                                                     'lhs' => 'character class',
                                                                                                                     'mask' => [
                                                                                                                                 1,
                                                                                                                                 1,
                                                                                                                                 1,
                                                                                                                                 1
                                                                                                                               ],
                                                                                                                     'name' => 'character class',
                                                                                                                     'rhs' => [
                                                                                                                                '[[\\[]]',
                                                                                                                                'cc elements',
                                                                                                                                '[[\\]]]',
                                                                                                                                'character class modifiers'
                                                                                                                              ],
                                                                                                                     'start' => 7190,
                                                                                                                     'subkey' => 1,
                                                                                                                     'xrlid' => 'character class@7190+69'
                                                                                                                   },
                                        'double colon,[[\\:]],[[\\:]]' => {
                                                                            'length' => 21,
                                                                            'lhs' => 'double colon',
                                                                            'mask' => [
                                                                                        1,
                                                                                        1
                                                                                      ],
                                                                            'name' => 'double colon',
                                                                            'rhs' => [
                                                                                       '[[\\:]]',
                                                                                       '[[\\:]]'
                                                                                     ],
                                                                            'start' => 5894,
                                                                            'subkey' => 1,
                                                                            'xrlid' => 'double colon@5894+21'
                                                                          },
                                        'escaped cc character,[[\\\\]],horizontal character' => {
                                                                                                  'length' => 51,
                                                                                                  'lhs' => 'escaped cc character',
                                                                                                  'mask' => [
                                                                                                              1,
                                                                                                              1
                                                                                                            ],
                                                                                                  'name' => 'escaped cc character',
                                                                                                  'rhs' => [
                                                                                                             '[[\\\\]]',
                                                                                                             'horizontal character'
                                                                                                           ],
                                                                                                  'start' => 7476,
                                                                                                  'subkey' => 1,
                                                                                                  'xrlid' => 'escaped cc character@7476+51'
                                                                                                },
                                        'hash comment body,hash comment char' => {
                                                                                   'length' => 42,
                                                                                   'lhs' => 'hash comment body',
                                                                                   'min' => 0,
                                                                                   'name' => 'hash comment body',
                                                                                   'rhs' => [
                                                                                              'hash comment char'
                                                                                            ],
                                                                                   'start' => 5095,
                                                                                   'subkey' => 0,
                                                                                   'xrlid' => 'hash comment body@5095+42'
                                                                                 },
                                        'hash comment char,[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                                                 'length' => 61,
                                                                                                                 'lhs' => 'hash comment char',
                                                                                                                 'mask' => [
                                                                                                                             1
                                                                                                                           ],
                                                                                                                 'name' => 'hash comment char',
                                                                                                                 'rhs' => [
                                                                                                                            '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                                                                                          ],
                                                                                                                 'start' => 5201,
                                                                                                                 'subkey' => 1,
                                                                                                                 'xrlid' => 'hash comment char@5201+61'
                                                                                                               },
                                        'hash comment,terminated hash comment' => {
                                                                                    'length' => 81,
                                                                                    'lhs' => 'hash comment',
                                                                                    'mask' => [
                                                                                                1
                                                                                              ],
                                                                                    'name' => 'hash comment',
                                                                                    'rhs' => [
                                                                                               'terminated hash comment'
                                                                                             ],
                                                                                    'start' => 4879,
                                                                                    'subkey' => 1,
                                                                                    'xrlid' => 'hash comment@4879+81'
                                                                                  },
                                        'hash comment,unterminated final hash comment' => {
                                                                                            'length' => 36,
                                                                                            'lhs' => 'hash comment',
                                                                                            'mask' => [
                                                                                                        1
                                                                                                      ],
                                                                                            'name' => 'hash comment',
                                                                                            'rhs' => [
                                                                                                       'unterminated final hash comment'
                                                                                                     ],
                                                                                            'start' => 4924,
                                                                                            'subkey' => 2,
                                                                                            'xrlid' => 'hash comment@4879+81'
                                                                                          },
                                        'horizontal character,[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                                                    'length' => 64,
                                                                                                                    'lhs' => 'horizontal character',
                                                                                                                    'mask' => [
                                                                                                                                1
                                                                                                                              ],
                                                                                                                    'name' => 'horizontal character',
                                                                                                                    'rhs' => [
                                                                                                                               '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                                                                                             ],
                                                                                                                    'start' => 8138,
                                                                                                                    'subkey' => 1,
                                                                                                                    'xrlid' => 'horizontal character@8138+64'
                                                                                                                  },
                                        'integer,[[\\d]]' => {
                                                               'length' => 17,
                                                               'lhs' => 'integer',
                                                               'min' => 1,
                                                               'name' => 'integer',
                                                               'rhs' => [
                                                                          '[[\\d]]'
                                                                        ],
                                                               'start' => 5486,
                                                               'subkey' => 0,
                                                               'xrlid' => 'integer@5486+17'
                                                             },
                                        'negated posix char class,[[\\[]],[[\\:]],[[\\^]],posix char class name,[[\\:]],[[\\]]]' => {
                                                                                                                                      'length' => 63,
                                                                                                                                      'lhs' => 'negated posix char class',
                                                                                                                                      'mask' => [
                                                                                                                                                  1,
                                                                                                                                                  1,
                                                                                                                                                  1,
                                                                                                                                                  1,
                                                                                                                                                  1,
                                                                                                                                                  1
                                                                                                                                                ],
                                                                                                                                      'name' => 'negated posix char class',
                                                                                                                                      'rhs' => [
                                                                                                                                                 '[[\\[]]',
                                                                                                                                                 '[[\\:]]',
                                                                                                                                                 '[[\\^]]',
                                                                                                                                                 'posix char class name',
                                                                                                                                                 '[[\\:]]',
                                                                                                                                                 '[[\\]]]'
                                                                                                                                               ],
                                                                                                                                      'start' => 7965,
                                                                                                                                      'subkey' => 1,
                                                                                                                                      'xrlid' => 'negated posix char class@7965+63'
                                                                                                                                    },
                                        'one or more word characters,[[\\w]]' => {
                                                                                   'length' => 37,
                                                                                   'lhs' => 'one or more word characters',
                                                                                   'min' => 1,
                                                                                   'name' => 'one or more word characters',
                                                                                   'rhs' => [
                                                                                              '[[\\w]]'
                                                                                            ],
                                                                                   'start' => 5643,
                                                                                   'subkey' => 0,
                                                                                   'xrlid' => 'one or more word characters@5643+37'
                                                                                 },
                                        'op declare bnf,[[\\:]],[[\\:]],[[\\=]]' => {
                                                                                      'length' => 24,
                                                                                      'lhs' => 'op declare bnf',
                                                                                      'mask' => [
                                                                                                  1,
                                                                                                  1,
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'op declare bnf',
                                                                                      'rhs' => [
                                                                                                 '[[\\:]]',
                                                                                                 '[[\\:]]',
                                                                                                 '[[\\=]]'
                                                                                               ],
                                                                                      'start' => 5264,
                                                                                      'subkey' => 1,
                                                                                      'xrlid' => 'op declare bnf@5264+24'
                                                                                    },
                                        'op declare match,[[\\~]]' => {
                                                                        'length' => 24,
                                                                        'lhs' => 'op declare match',
                                                                        'mask' => [
                                                                                    1
                                                                                  ],
                                                                        'name' => 'op declare match',
                                                                        'rhs' => [
                                                                                   '[[\\~]]'
                                                                                 ],
                                                                        'start' => 5289,
                                                                        'subkey' => 1,
                                                                        'xrlid' => 'op declare match@5289+24'
                                                                      },
                                        'op equal priority,[[\\|]]' => {
                                                                         'length' => 25,
                                                                         'lhs' => 'op equal priority',
                                                                         'mask' => [
                                                                                     1
                                                                                   ],
                                                                         'name' => 'op equal priority',
                                                                         'rhs' => [
                                                                                    '[[\\|]]'
                                                                                  ],
                                                                         'start' => 5333,
                                                                         'subkey' => 1,
                                                                         'xrlid' => 'op equal priority@5333+25'
                                                                       },
                                        'op loosen,[[\\|]],[[\\|]]' => {
                                                                         'length' => 18,
                                                                         'lhs' => 'op loosen',
                                                                         'mask' => [
                                                                                     1,
                                                                                     1
                                                                                   ],
                                                                         'name' => 'op loosen',
                                                                         'rhs' => [
                                                                                    '[[\\|]]',
                                                                                    '[[\\|]]'
                                                                                  ],
                                                                         'start' => 5314,
                                                                         'subkey' => 1,
                                                                         'xrlid' => 'op loosen@5314+18'
                                                                       },
                                        'posix char class name,[[[:alnum:]]]' => {
                                                                                   'length' => 38,
                                                                                   'lhs' => 'posix char class name',
                                                                                   'min' => 1,
                                                                                   'name' => 'posix char class name',
                                                                                   'rhs' => [
                                                                                              '[[[:alnum:]]]'
                                                                                            ],
                                                                                   'start' => 8029,
                                                                                   'subkey' => 0,
                                                                                   'xrlid' => 'posix char class name@8029+38'
                                                                                 },
                                        'posix char class,[[\\[]],[[\\:]],posix char class name,[[\\:]],[[\\]]]' => {
                                                                                                                      'length' => 54,
                                                                                                                      'lhs' => 'posix char class',
                                                                                                                      'mask' => [
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1
                                                                                                                                ],
                                                                                                                      'name' => 'posix char class',
                                                                                                                      'rhs' => [
                                                                                                                                 '[[\\[]]',
                                                                                                                                 '[[\\:]]',
                                                                                                                                 'posix char class name',
                                                                                                                                 '[[\\:]]',
                                                                                                                                 '[[\\]]]'
                                                                                                                               ],
                                                                                                                      'start' => 7910,
                                                                                                                      'subkey' => 1,
                                                                                                                      'xrlid' => 'posix char class@7910+54'
                                                                                                                    },
                                        'reserved action name,[[\\:]],[[\\:]],one or more word characters' => {
                                                                                                                'length' => 59,
                                                                                                                'lhs' => 'reserved action name',
                                                                                                                'mask' => [
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1
                                                                                                                          ],
                                                                                                                'name' => 'reserved action name',
                                                                                                                'rhs' => [
                                                                                                                           '[[\\:]]',
                                                                                                                           '[[\\:]]',
                                                                                                                           'one or more word characters'
                                                                                                                         ],
                                                                                                                'start' => 5521,
                                                                                                                'subkey' => 1,
                                                                                                                'xrlid' => 'reserved action name@5521+59'
                                                                                                              },
                                        'reserved blessing name,[[\\:]],[[\\:]],one or more word characters' => {
                                                                                                                  'length' => 61,
                                                                                                                  'lhs' => 'reserved blessing name',
                                                                                                                  'mask' => [
                                                                                                                              1,
                                                                                                                              1,
                                                                                                                              1
                                                                                                                            ],
                                                                                                                  'name' => 'reserved blessing name',
                                                                                                                  'rhs' => [
                                                                                                                             '[[\\:]]',
                                                                                                                             '[[\\:]]',
                                                                                                                             'one or more word characters'
                                                                                                                           ],
                                                                                                                  'start' => 5581,
                                                                                                                  'subkey' => 1,
                                                                                                                  'xrlid' => 'reserved blessing name@5581+61'
                                                                                                                },
                                        'reserved event name,[[\\:]],[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]' => {
                                                                                                               'length' => 33,
                                                                                                               'lhs' => 'reserved event name',
                                                                                                               'mask' => [
                                                                                                                           1,
                                                                                                                           1,
                                                                                                                           1,
                                                                                                                           1,
                                                                                                                           1,
                                                                                                                           1,
                                                                                                                           1
                                                                                                                         ],
                                                                                                               'name' => 'reserved event name',
                                                                                                               'rhs' => [
                                                                                                                          '[[\\:]]',
                                                                                                                          '[[s]]',
                                                                                                                          '[[y]]',
                                                                                                                          '[[m]]',
                                                                                                                          '[[b]]',
                                                                                                                          '[[o]]',
                                                                                                                          '[[l]]'
                                                                                                                        ],
                                                                                                               'start' => 4148,
                                                                                                               'subkey' => 1,
                                                                                                               'xrlid' => 'reserved event name@4148+33'
                                                                                                             },
                                        'result item descriptor list,result item descriptor' => {
                                                                                                  'length' => 105,
                                                                                                  'lhs' => 'result item descriptor list',
                                                                                                  'min' => 0,
                                                                                                  'name' => 'result item descriptor list',
                                                                                                  'rhs' => [
                                                                                                             'result item descriptor'
                                                                                                           ],
                                                                                                  'separator' => 'result item descriptor separator',
                                                                                                  'start' => 6449,
                                                                                                  'subkey' => 0,
                                                                                                  'xrlid' => 'result item descriptor list@6449+105'
                                                                                                },
                                        'result item descriptor separator,[[,]]' => {
                                                                                      'length' => 40,
                                                                                      'lhs' => 'result item descriptor separator',
                                                                                      'mask' => [
                                                                                                  1
                                                                                                ],
                                                                                      'name' => 'result item descriptor separator',
                                                                                      'rhs' => [
                                                                                                 '[[,]]'
                                                                                               ],
                                                                                      'start' => 6555,
                                                                                      'subkey' => 1,
                                                                                      'xrlid' => 'result item descriptor separator@6555+40'
                                                                                    },
                                        'result item descriptor separator,[[,]],whitespace' => {
                                                                                                 'length' => 51,
                                                                                                 'lhs' => 'result item descriptor separator',
                                                                                                 'mask' => [
                                                                                                             1,
                                                                                                             1
                                                                                                           ],
                                                                                                 'name' => 'result item descriptor separator',
                                                                                                 'rhs' => [
                                                                                                            '[[,]]',
                                                                                                            'whitespace'
                                                                                                          ],
                                                                                                 'start' => 6596,
                                                                                                 'subkey' => 1,
                                                                                                 'xrlid' => 'result item descriptor separator@6596+51'
                                                                                               },
                                        'result item descriptor,[[g]],[[1]],[[l]],[[e]],[[n]],[[g]],[[t]],[[h]]' => {
                                                                                                                      'length' => 10,
                                                                                                                      'lhs' => 'result item descriptor',
                                                                                                                      'mask' => [
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1,
                                                                                                                                  1
                                                                                                                                ],
                                                                                                                      'name' => 'result item descriptor',
                                                                                                                      'rhs' => [
                                                                                                                                 '[[g]]',
                                                                                                                                 '[[1]]',
                                                                                                                                 '[[l]]',
                                                                                                                                 '[[e]]',
                                                                                                                                 '[[n]]',
                                                                                                                                 '[[g]]',
                                                                                                                                 '[[t]]',
                                                                                                                                 '[[h]]'
                                                                                                                               ],
                                                                                                                      'start' => 6710,
                                                                                                                      'subkey' => 4,
                                                                                                                      'xrlid' => 'result item descriptor@6648+135'
                                                                                                                    },
                                        'result item descriptor,[[g]],[[1]],[[s]],[[t]],[[a]],[[r]],[[t]]' => {
                                                                                                                'length' => 9,
                                                                                                                'lhs' => 'result item descriptor',
                                                                                                                'mask' => [
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1,
                                                                                                                            1
                                                                                                                          ],
                                                                                                                'name' => 'result item descriptor',
                                                                                                                'rhs' => [
                                                                                                                           '[[g]]',
                                                                                                                           '[[1]]',
                                                                                                                           '[[s]]',
                                                                                                                           '[[t]]',
                                                                                                                           '[[a]]',
                                                                                                                           '[[r]]',
                                                                                                                           '[[t]]'
                                                                                                                         ],
                                                                                                                'start' => 6698,
                                                                                                                'subkey' => 3,
                                                                                                                'xrlid' => 'result item descriptor@6648+135'
                                                                                                              },
                                        'result item descriptor,[[l]],[[e]],[[n]],[[g]],[[t]],[[h]]' => {
                                                                                                          'length' => 8,
                                                                                                          'lhs' => 'result item descriptor',
                                                                                                          'mask' => [
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1
                                                                                                                    ],
                                                                                                          'name' => 'result item descriptor',
                                                                                                          'rhs' => [
                                                                                                                     '[[l]]',
                                                                                                                     '[[e]]',
                                                                                                                     '[[n]]',
                                                                                                                     '[[g]]',
                                                                                                                     '[[t]]',
                                                                                                                     '[[h]]'
                                                                                                                   ],
                                                                                                          'start' => 6685,
                                                                                                          'subkey' => 2,
                                                                                                          'xrlid' => 'result item descriptor@6648+135'
                                                                                                        },
                                        'result item descriptor,[[l]],[[h]],[[s]]' => {
                                                                                        'length' => 5,
                                                                                        'lhs' => 'result item descriptor',
                                                                                        'mask' => [
                                                                                                    1,
                                                                                                    1,
                                                                                                    1
                                                                                                  ],
                                                                                        'name' => 'result item descriptor',
                                                                                        'rhs' => [
                                                                                                   '[[l]]',
                                                                                                   '[[h]]',
                                                                                                   '[[s]]'
                                                                                                 ],
                                                                                        'start' => 6734,
                                                                                        'subkey' => 6,
                                                                                        'xrlid' => 'result item descriptor@6648+135'
                                                                                      },
                                        'result item descriptor,[[n]],[[a]],[[m]],[[e]]' => {
                                                                                              'length' => 6,
                                                                                              'lhs' => 'result item descriptor',
                                                                                              'mask' => [
                                                                                                          1,
                                                                                                          1,
                                                                                                          1,
                                                                                                          1
                                                                                                        ],
                                                                                              'name' => 'result item descriptor',
                                                                                              'rhs' => [
                                                                                                         '[[n]]',
                                                                                                         '[[a]]',
                                                                                                         '[[m]]',
                                                                                                         '[[e]]'
                                                                                                       ],
                                                                                              'start' => 6725,
                                                                                              'subkey' => 5,
                                                                                              'xrlid' => 'result item descriptor@6648+135'
                                                                                            },
                                        'result item descriptor,[[r]],[[u]],[[l]],[[e]]' => {
                                                                                              'length' => 6,
                                                                                              'lhs' => 'result item descriptor',
                                                                                              'mask' => [
                                                                                                          1,
                                                                                                          1,
                                                                                                          1,
                                                                                                          1
                                                                                                        ],
                                                                                              'name' => 'result item descriptor',
                                                                                              'rhs' => [
                                                                                                         '[[r]]',
                                                                                                         '[[u]]',
                                                                                                         '[[l]]',
                                                                                                         '[[e]]'
                                                                                                       ],
                                                                                              'start' => 6753,
                                                                                              'subkey' => 8,
                                                                                              'xrlid' => 'result item descriptor@6648+135'
                                                                                            },
                                        'result item descriptor,[[s]],[[t]],[[a]],[[r]],[[t]]' => {
                                                                                                    'length' => 135,
                                                                                                    'lhs' => 'result item descriptor',
                                                                                                    'mask' => [
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1
                                                                                                              ],
                                                                                                    'name' => 'result item descriptor',
                                                                                                    'rhs' => [
                                                                                                               '[[s]]',
                                                                                                               '[[t]]',
                                                                                                               '[[a]]',
                                                                                                               '[[r]]',
                                                                                                               '[[t]]'
                                                                                                             ],
                                                                                                    'start' => 6648,
                                                                                                    'subkey' => 1,
                                                                                                    'xrlid' => 'result item descriptor@6648+135'
                                                                                                  },
                                        'result item descriptor,[[s]],[[y]],[[m]],[[b]],[[o]],[[l]]' => {
                                                                                                          'length' => 8,
                                                                                                          'lhs' => 'result item descriptor',
                                                                                                          'mask' => [
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1
                                                                                                                    ],
                                                                                                          'name' => 'result item descriptor',
                                                                                                          'rhs' => [
                                                                                                                     '[[s]]',
                                                                                                                     '[[y]]',
                                                                                                                     '[[m]]',
                                                                                                                     '[[b]]',
                                                                                                                     '[[o]]',
                                                                                                                     '[[l]]'
                                                                                                                   ],
                                                                                                          'start' => 6742,
                                                                                                          'subkey' => 7,
                                                                                                          'xrlid' => 'result item descriptor@6648+135'
                                                                                                        },
                                        'result item descriptor,[[v]],[[a]],[[l]],[[u]],[[e]]' => {
                                                                                                    'length' => 7,
                                                                                                    'lhs' => 'result item descriptor',
                                                                                                    'mask' => [
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1,
                                                                                                                1
                                                                                                              ],
                                                                                                    'name' => 'result item descriptor',
                                                                                                    'rhs' => [
                                                                                                               '[[v]]',
                                                                                                               '[[a]]',
                                                                                                               '[[l]]',
                                                                                                               '[[u]]',
                                                                                                               '[[e]]'
                                                                                                             ],
                                                                                                    'start' => 6765,
                                                                                                    'subkey' => 9,
                                                                                                    'xrlid' => 'result item descriptor@6648+135'
                                                                                                  },
                                        'result item descriptor,[[v]],[[a]],[[l]],[[u]],[[e]],[[s]]' => {
                                                                                                          'length' => 8,
                                                                                                          'lhs' => 'result item descriptor',
                                                                                                          'mask' => [
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1,
                                                                                                                      1
                                                                                                                    ],
                                                                                                          'name' => 'result item descriptor',
                                                                                                          'rhs' => [
                                                                                                                     '[[v]]',
                                                                                                                     '[[a]]',
                                                                                                                     '[[l]]',
                                                                                                                     '[[u]]',
                                                                                                                     '[[e]]',
                                                                                                                     '[[s]]'
                                                                                                                   ],
                                                                                                          'start' => 6775,
                                                                                                          'subkey' => 10,
                                                                                                          'xrlid' => 'result item descriptor@6648+135'
                                                                                                        },
                                        'safe cc character,[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                                                     'length' => 79,
                                                                                                                                     'lhs' => 'safe cc character',
                                                                                                                                     'mask' => [
                                                                                                                                                 1
                                                                                                                                               ],
                                                                                                                                     'name' => 'safe cc character',
                                                                                                                                     'rhs' => [
                                                                                                                                                '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                                                                                                              ],
                                                                                                                                     'start' => 7358,
                                                                                                                                     'subkey' => 1,
                                                                                                                                     'xrlid' => 'safe cc character@7358+79'
                                                                                                                                   },
                                        'sign,[[+-]]' => {
                                                           'length' => 13,
                                                           'lhs' => 'sign',
                                                           'mask' => [
                                                                       1
                                                                     ],
                                                           'name' => 'sign',
                                                           'rhs' => [
                                                                      '[[+-]]'
                                                                    ],
                                                           'start' => 5472,
                                                           'subkey' => 1,
                                                           'xrlid' => 'sign@5472+13'
                                                         },
                                        'signed integer,integer' => {
                                                                      'length' => 47,
                                                                      'lhs' => 'signed integer',
                                                                      'mask' => [
                                                                                  1
                                                                                ],
                                                                      'name' => 'signed integer',
                                                                      'rhs' => [
                                                                                 'integer'
                                                                               ],
                                                                      'start' => 5424,
                                                                      'subkey' => 1,
                                                                      'xrlid' => 'signed integer@5424+47'
                                                                    },
                                        'signed integer,sign,integer' => {
                                                                           'length' => 16,
                                                                           'lhs' => 'signed integer',
                                                                           'mask' => [
                                                                                       1,
                                                                                       1
                                                                                     ],
                                                                           'name' => 'signed integer',
                                                                           'rhs' => [
                                                                                      'sign',
                                                                                      'integer'
                                                                                    ],
                                                                           'start' => 5455,
                                                                           'subkey' => 2,
                                                                           'xrlid' => 'signed integer@5424+47'
                                                                         },
                                        'single quoted name,[[\']],string without single quote or vertical space,[[\']]' => {
                                                                                                                              'length' => 78,
                                                                                                                              'lhs' => 'single quoted name',
                                                                                                                              'mask' => [
                                                                                                                                          1,
                                                                                                                                          1,
                                                                                                                                          1
                                                                                                                                        ],
                                                                                                                              'name' => 'single quoted name',
                                                                                                                              'rhs' => [
                                                                                                                                         '[[\']]',
                                                                                                                                         'string without single quote or vertical space',
                                                                                                                                         '[[\']]'
                                                                                                                                       ],
                                                                                                                              'start' => 7006,
                                                                                                                              'subkey' => 1,
                                                                                                                              'xrlid' => 'single quoted name@7006+78'
                                                                                                                            },
                                        'single quoted string,[[\']],string without single quote or vertical space,[[\']],character class modifiers' => {
                                                                                                                                                          'length' => 108,
                                                                                                                                                          'lhs' => 'single quoted string',
                                                                                                                                                          'mask' => [
                                                                                                                                                                      1,
                                                                                                                                                                      1,
                                                                                                                                                                      1,
                                                                                                                                                                      1
                                                                                                                                                                    ],
                                                                                                                                                          'name' => 'single quoted string',
                                                                                                                                                          'rhs' => [
                                                                                                                                                                     '[[\']]',
                                                                                                                                                                     'string without single quote or vertical space',
                                                                                                                                                                     '[[\']]',
                                                                                                                                                                     'character class modifiers'
                                                                                                                                                                   ],
                                                                                                                                                          'start' => 6897,
                                                                                                                                                          'subkey' => 1,
                                                                                                                                                          'xrlid' => 'single quoted string@6897+108'
                                                                                                                                                        },
                                        'standard name,[[a-zA-Z]],zero or more word characters' => {
                                                                                                     'length' => 57,
                                                                                                     'lhs' => 'standard name',
                                                                                                     'mask' => [
                                                                                                                 1,
                                                                                                                 1
                                                                                                               ],
                                                                                                     'name' => 'standard name',
                                                                                                     'rhs' => [
                                                                                                                '[[a-zA-Z]]',
                                                                                                                'zero or more word characters'
                                                                                                              ],
                                                                                                     'start' => 6010,
                                                                                                     'subkey' => 1,
                                                                                                     'xrlid' => 'standard name@6010+57'
                                                                                                   },
                                        'string without single quote or vertical space,[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                                                                            'length' => 103,
                                                                                                                                                            'lhs' => 'string without single quote or vertical space',
                                                                                                                                                            'min' => 1,
                                                                                                                                                            'name' => 'string without single quote or vertical space',
                                                                                                                                                            'rhs' => [
                                                                                                                                                                       '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]'
                                                                                                                                                                     ],
                                                                                                                                                            'start' => 7085,
                                                                                                                                                            'subkey' => 0,
                                                                                                                                                            'xrlid' => 'string without single quote or vertical space@7085+103'
                                                                                                                                                          },
                                        'terminated hash comment,[[\\#]],hash comment body,vertical space char' => {
                                                                                                                     'length' => 73,
                                                                                                                     'lhs' => 'terminated hash comment',
                                                                                                                     'mask' => [
                                                                                                                                 1,
                                                                                                                                 1,
                                                                                                                                 1
                                                                                                                               ],
                                                                                                                     'name' => 'terminated hash comment',
                                                                                                                     'rhs' => [
                                                                                                                                '[[\\#]]',
                                                                                                                                'hash comment body',
                                                                                                                                'vertical space char'
                                                                                                                              ],
                                                                                                                     'start' => 4961,
                                                                                                                     'subkey' => 1,
                                                                                                                     'xrlid' => 'terminated hash comment@4961+73'
                                                                                                                   },
                                        'unterminated final hash comment,[[\\#]],hash comment body' => {
                                                                                                         'length' => 59,
                                                                                                         'lhs' => 'unterminated final hash comment',
                                                                                                         'mask' => [
                                                                                                                     1,
                                                                                                                     1
                                                                                                                   ],
                                                                                                         'name' => 'unterminated final hash comment',
                                                                                                         'rhs' => [
                                                                                                                    '[[\\#]]',
                                                                                                                    'hash comment body'
                                                                                                                  ],
                                                                                                         'start' => 5035,
                                                                                                         'subkey' => 1,
                                                                                                         'xrlid' => 'unterminated final hash comment@5035+59'
                                                                                                       },
                                        'vertical space char,[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                                                  'length' => 62,
                                                                                                                  'lhs' => 'vertical space char',
                                                                                                                  'mask' => [
                                                                                                                              1
                                                                                                                            ],
                                                                                                                  'name' => 'vertical space char',
                                                                                                                  'rhs' => [
                                                                                                                             '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]'
                                                                                                                           ],
                                                                                                                  'start' => 5138,
                                                                                                                  'subkey' => 1,
                                                                                                                  'xrlid' => 'vertical space char@5138+62'
                                                                                                                },
                                        'whitespace,[[\\s]]' => {
                                                                  'length' => 18,
                                                                  'lhs' => 'whitespace',
                                                                  'min' => 1,
                                                                  'name' => 'whitespace',
                                                                  'rhs' => [
                                                                             '[[\\s]]'
                                                                           ],
                                                                  'start' => 4816,
                                                                  'subkey' => 0,
                                                                  'xrlid' => 'whitespace@4816+18'
                                                                },
                                        'zero or more word characters,[[\\w]]' => {
                                                                                    'length' => 38,
                                                                                    'lhs' => 'zero or more word characters',
                                                                                    'min' => 0,
                                                                                    'name' => 'zero or more word characters',
                                                                                    'rhs' => [
                                                                                               '[[\\w]]'
                                                                                             ],
                                                                                    'start' => 5681,
                                                                                    'subkey' => 0,
                                                                                    'xrlid' => 'zero or more word characters@5681+38'
                                                                                  }
                                      }
                            },
                  'xrl' => {
                             'Perl identifier@5868+25' => {
                                                            'length' => 25,
                                                            'lhs' => 'Perl identifier',
                                                            'precedence_count' => 1,
                                                            'start' => 5868
                                                          },
                             'Perl name@5916+72' => {
                                                      'length' => 72,
                                                      'lhs' => 'Perl name',
                                                      'precedence_count' => 1,
                                                      'start' => 5916
                                                    },
                             'action name@4685+29' => {
                                                        'length' => 29,
                                                        'lhs' => 'action name',
                                                        'precedence_count' => 1,
                                                        'start' => 4685
                                                      },
                             'action name@4715+40' => {
                                                        'length' => 40,
                                                        'lhs' => 'action name',
                                                        'precedence_count' => 1,
                                                        'start' => 4715
                                                      },
                             'action name@4756+36' => {
                                                        'length' => 36,
                                                        'lhs' => 'action name',
                                                        'precedence_count' => 1,
                                                        'start' => 4756
                                                      },
                             'action@2897+40' => {
                                                   'length' => 40,
                                                   'lhs' => 'action',
                                                   'precedence_count' => 1,
                                                   'start' => 2897
                                                 },
                             'adverb item@2511+362' => {
                                                         'length' => 362,
                                                         'lhs' => 'adverb item',
                                                         'precedence_count' => 1,
                                                         'start' => 2511
                                                       },
                             'adverb list items@2472+38' => {
                                                              'length' => 38,
                                                              'lhs' => 'adverb list items',
                                                              'precedence_count' => 1,
                                                              'start' => 2472
                                                            },
                             'adverb list@2434+37' => {
                                                        'length' => 37,
                                                        'lhs' => 'adverb list',
                                                        'precedence_count' => 1,
                                                        'start' => 2434
                                                      },
                             'alternative name@3997+61' => {
                                                             'length' => 61,
                                                             'lhs' => 'alternative name',
                                                             'precedence_count' => 1,
                                                             'start' => 3997
                                                           },
                             'alternative@2399+33' => {
                                                        'length' => 33,
                                                        'lhs' => 'alternative',
                                                        'precedence_count' => 1,
                                                        'start' => 2399
                                                      },
                             'alternatives@2320+78' => {
                                                         'length' => 78,
                                                         'lhs' => 'alternatives',
                                                         'precedence_count' => 1,
                                                         'start' => 2320
                                                       },
                             'array descriptor left bracket@6270+37' => {
                                                                          'length' => 37,
                                                                          'lhs' => 'array descriptor left bracket',
                                                                          'precedence_count' => 1,
                                                                          'start' => 6270
                                                                        },
                             'array descriptor left bracket@6309+48' => {
                                                                          'length' => 48,
                                                                          'lhs' => 'array descriptor left bracket',
                                                                          'precedence_count' => 1,
                                                                          'start' => 6309
                                                                        },
                             'array descriptor right bracket@6358+38' => {
                                                                           'length' => 38,
                                                                           'lhs' => 'array descriptor right bracket',
                                                                           'precedence_count' => 1,
                                                                           'start' => 6358
                                                                         },
                             'array descriptor right bracket@6398+49' => {
                                                                           'length' => 49,
                                                                           'lhs' => 'array descriptor right bracket',
                                                                           'precedence_count' => 1,
                                                                           'start' => 6398
                                                                         },
                             'array descriptor@6154+115' => {
                                                              'length' => 115,
                                                              'lhs' => 'array descriptor',
                                                              'precedence_count' => 1,
                                                              'start' => 6154
                                                            },
                             'bare name@5990+19' => {
                                                      'length' => 19,
                                                      'lhs' => 'bare name',
                                                      'precedence_count' => 1,
                                                      'start' => 5990
                                                    },
                             'before or after@5384+38' => {
                                                            'length' => 38,
                                                            'lhs' => 'before or after',
                                                            'precedence_count' => 1,
                                                            'start' => 5384
                                                          },
                             'blessing name@4183+35' => {
                                                          'length' => 35,
                                                          'lhs' => 'blessing name',
                                                          'precedence_count' => 1,
                                                          'start' => 4183
                                                        },
                             'blessing name@4219+44' => {
                                                          'length' => 44,
                                                          'lhs' => 'blessing name',
                                                          'precedence_count' => 1,
                                                          'start' => 4219
                                                        },
                             'blessing@3904+45' => {
                                                     'length' => 45,
                                                     'lhs' => 'blessing',
                                                     'precedence_count' => 1,
                                                     'start' => 3904
                                                   },
                             'boolean@5505+14' => {
                                                    'length' => 14,
                                                    'lhs' => 'boolean',
                                                    'precedence_count' => 1,
                                                    'start' => 5505
                                                  },
                             'bracketed name string@6119+33' => {
                                                                  'length' => 33,
                                                                  'lhs' => 'bracketed name string',
                                                                  'precedence_count' => 1,
                                                                  'start' => 6119
                                                                },
                             'bracketed name@6068+50' => {
                                                           'length' => 50,
                                                           'lhs' => 'bracketed name',
                                                           'precedence_count' => 1,
                                                           'start' => 6068
                                                         },
                             'cc element@7290+34' => {
                                                       'length' => 34,
                                                       'lhs' => 'cc element',
                                                       'precedence_count' => 1,
                                                       'start' => 7290
                                                     },
                             'cc element@7438+37' => {
                                                       'length' => 37,
                                                       'lhs' => 'cc element',
                                                       'precedence_count' => 1,
                                                       'start' => 7438
                                                     },
                             'cc element@7528+33' => {
                                                       'length' => 33,
                                                       'lhs' => 'cc element',
                                                       'precedence_count' => 1,
                                                       'start' => 7528
                                                     },
                             'cc element@7562+41' => {
                                                       'length' => 41,
                                                       'lhs' => 'cc element',
                                                       'precedence_count' => 1,
                                                       'start' => 7562
                                                     },
                             'cc elements@7260+29' => {
                                                        'length' => 29,
                                                        'lhs' => 'cc elements',
                                                        'precedence_count' => 1,
                                                        'start' => 7260
                                                      },
                             'character class modifier@7662+34' => {
                                                                     'length' => 34,
                                                                     'lhs' => 'character class modifier',
                                                                     'precedence_count' => 1,
                                                                     'start' => 7662
                                                                   },
                             'character class modifier@7697+33' => {
                                                                     'length' => 33,
                                                                     'lhs' => 'character class modifier',
                                                                     'precedence_count' => 1,
                                                                     'start' => 7697
                                                                   },
                             'character class modifiers@7604+57' => {
                                                                      'length' => 57,
                                                                      'lhs' => 'character class modifiers',
                                                                      'precedence_count' => 1,
                                                                      'start' => 7604
                                                                    },
                             'character class@7190+69' => {
                                                            'length' => 69,
                                                            'lhs' => 'character class',
                                                            'precedence_count' => 1,
                                                            'start' => 7190
                                                          },
                             'completion event declaration@1754+99' => {
                                                                         'length' => 99,
                                                                         'lhs' => 'completion event declaration',
                                                                         'precedence_count' => 1,
                                                                         'start' => 1754
                                                                       },
                             'default rule@1224+60' => {
                                                         'length' => 60,
                                                         'lhs' => 'default rule',
                                                         'precedence_count' => 1,
                                                         'start' => 1224
                                                       },
                             'discard default statement@1355+71' => {
                                                                      'length' => 71,
                                                                      'lhs' => 'discard default statement',
                                                                      'precedence_count' => 1,
                                                                      'start' => 1355
                                                                    },
                             'discard rule@1603+80' => {
                                                         'length' => 80,
                                                         'lhs' => 'discard rule',
                                                         'precedence_count' => 1,
                                                         'start' => 1603
                                                       },
                             'double colon@5894+21' => {
                                                         'length' => 21,
                                                         'lhs' => 'double colon',
                                                         'precedence_count' => 1,
                                                         'start' => 5894
                                                       },
                             'empty rule@1475+47' => {
                                                       'length' => 47,
                                                       'lhs' => 'empty rule',
                                                       'precedence_count' => 1,
                                                       'start' => 1475
                                                     },
                             'escaped cc character@7476+51' => {
                                                                 'length' => 51,
                                                                 'lhs' => 'escaped cc character',
                                                                 'precedence_count' => 1,
                                                                 'start' => 7476
                                                               },
                             'event initialization@3637+59' => {
                                                                 'length' => 59,
                                                                 'lhs' => 'event initialization',
                                                                 'precedence_count' => 1,
                                                                 'start' => 3637
                                                               },
                             'event initializer@3697+41' => {
                                                              'length' => 41,
                                                              'lhs' => 'event initializer',
                                                              'precedence_count' => 1,
                                                              'start' => 3697
                                                            },
                             'event initializer@3768+23' => {
                                                              'length' => 23,
                                                              'lhs' => 'event initializer',
                                                              'precedence_count' => 1,
                                                              'start' => 3768
                                                            },
                             'event name@4060+87' => {
                                                       'length' => 87,
                                                       'lhs' => 'event name',
                                                       'precedence_count' => 1,
                                                       'start' => 4060
                                                     },
                             'event specification@3573+63' => {
                                                                'length' => 63,
                                                                'lhs' => 'event specification',
                                                                'precedence_count' => 1,
                                                                'start' => 3573
                                                              },
                             'group association@3030+46' => {
                                                              'length' => 46,
                                                              'lhs' => 'group association',
                                                              'precedence_count' => 1,
                                                              'start' => 3030
                                                            },
                             'hash comment body@5095+42' => {
                                                              'length' => 42,
                                                              'lhs' => 'hash comment body',
                                                              'precedence_count' => 1,
                                                              'start' => 5095
                                                            },
                             'hash comment char@5201+61' => {
                                                              'length' => 61,
                                                              'lhs' => 'hash comment char',
                                                              'precedence_count' => 1,
                                                              'start' => 5201
                                                            },
                             'hash comment@4879+81' => {
                                                         'length' => 81,
                                                         'lhs' => 'hash comment',
                                                         'precedence_count' => 1,
                                                         'start' => 4879
                                                       },
                             'horizontal character@8138+64' => {
                                                                 'length' => 64,
                                                                 'lhs' => 'horizontal character',
                                                                 'precedence_count' => 1,
                                                                 'start' => 8138
                                                               },
                             'inaccessible statement@2048+92' => {
                                                                   'length' => 92,
                                                                   'lhs' => 'inaccessible statement',
                                                                   'precedence_count' => 1,
                                                                   'start' => 2048
                                                                 },
                             'inaccessible treatment@2141+52' => {
                                                                   'length' => 52,
                                                                   'lhs' => 'inaccessible treatment',
                                                                   'precedence_count' => 1,
                                                                   'start' => 2141
                                                                 },
                             'integer@5486+17' => {
                                                    'length' => 17,
                                                    'lhs' => 'integer',
                                                    'precedence_count' => 1,
                                                    'start' => 5486
                                                  },
                             'left association@2938+44' => {
                                                             'length' => 44,
                                                             'lhs' => 'left association',
                                                             'precedence_count' => 1,
                                                             'start' => 2938
                                                           },
                             'lexeme default statement@1285+69' => {
                                                                     'length' => 69,
                                                                     'lhs' => 'lexeme default statement',
                                                                     'precedence_count' => 1,
                                                                     'start' => 1285
                                                                   },
                             'lexeme rule@1684+69' => {
                                                        'length' => 69,
                                                        'lhs' => 'lexeme rule',
                                                        'precedence_count' => 1,
                                                        'start' => 1684
                                                      },
                             'lhs@4265+21' => {
                                                'length' => 21,
                                                'lhs' => 'lhs',
                                                'precedence_count' => 1,
                                                'start' => 4265
                                              },
                             'naming@3950+45' => {
                                                   'length' => 45,
                                                   'lhs' => 'naming',
                                                   'precedence_count' => 1,
                                                   'start' => 3950
                                                 },
                             'negated posix char class@7965+63' => {
                                                                     'length' => 63,
                                                                     'lhs' => 'negated posix char class',
                                                                     'precedence_count' => 1,
                                                                     'start' => 7965
                                                                   },
                             'null adverb@2875+21' => {
                                                        'length' => 21,
                                                        'lhs' => 'null adverb',
                                                        'precedence_count' => 1,
                                                        'start' => 2875
                                                      },
                             'null ranking constant@3406+42' => {
                                                                  'length' => 42,
                                                                  'lhs' => 'null ranking constant',
                                                                  'precedence_count' => 1,
                                                                  'start' => 3406
                                                                },
                             'null ranking specification@3249+78' => {
                                                                       'length' => 78,
                                                                       'lhs' => 'null ranking specification',
                                                                       'precedence_count' => 1,
                                                                       'start' => 3249
                                                                     },
                             'null ranking specification@3328+77' => {
                                                                       'length' => 77,
                                                                       'lhs' => 'null ranking specification',
                                                                       'precedence_count' => 1,
                                                                       'start' => 3328
                                                                     },
                             'null statement@1056+24' => {
                                                           'length' => 24,
                                                           'lhs' => 'null statement',
                                                           'precedence_count' => 1,
                                                           'start' => 1056
                                                         },
                             'nulled event declaration@1854+92' => {
                                                                     'length' => 92,
                                                                     'lhs' => 'nulled event declaration',
                                                                     'precedence_count' => 1,
                                                                     'start' => 1854
                                                                   },
                             'on or off@3739+28' => {
                                                      'length' => 28,
                                                      'lhs' => 'on or off',
                                                      'precedence_count' => 1,
                                                      'start' => 3739
                                                    },
                             'one or more word characters@5643+37' => {
                                                                        'length' => 37,
                                                                        'lhs' => 'one or more word characters',
                                                                        'precedence_count' => 1,
                                                                        'start' => 5643
                                                                      },
                             'op declare bnf@5264+24' => {
                                                           'length' => 24,
                                                           'lhs' => 'op declare bnf',
                                                           'precedence_count' => 1,
                                                           'start' => 5264
                                                         },
                             'op declare match@5289+24' => {
                                                             'length' => 24,
                                                             'lhs' => 'op declare match',
                                                             'precedence_count' => 1,
                                                             'start' => 5289
                                                           },
                             'op declare@2195+54' => {
                                                       'length' => 54,
                                                       'lhs' => 'op declare',
                                                       'precedence_count' => 1,
                                                       'start' => 2195
                                                     },
                             'op equal priority@5333+25' => {
                                                              'length' => 25,
                                                              'lhs' => 'op equal priority',
                                                              'precedence_count' => 1,
                                                              'start' => 5333
                                                            },
                             'op loosen@5314+18' => {
                                                      'length' => 18,
                                                      'lhs' => 'op loosen',
                                                      'precedence_count' => 1,
                                                      'start' => 5314
                                                    },
                             'parenthesized rhs primary list@4436+67' => {
                                                                           'length' => 67,
                                                                           'lhs' => 'parenthesized rhs primary list',
                                                                           'precedence_count' => 1,
                                                                           'start' => 4436
                                                                         },
                             'pause specification@3513+58' => {
                                                                'length' => 58,
                                                                'lhs' => 'pause specification',
                                                                'precedence_count' => 1,
                                                                'start' => 3513
                                                              },
                             'posix char class name@8029+38' => {
                                                                  'length' => 38,
                                                                  'lhs' => 'posix char class name',
                                                                  'precedence_count' => 1,
                                                                  'start' => 8029
                                                                },
                             'posix char class@7910+54' => {
                                                             'length' => 54,
                                                             'lhs' => 'posix char class',
                                                             'precedence_count' => 1,
                                                             'start' => 7910
                                                           },
                             'prediction event declaration@1947+99' => {
                                                                         'length' => 99,
                                                                         'lhs' => 'prediction event declaration',
                                                                         'precedence_count' => 1,
                                                                         'start' => 1947
                                                                       },
                             'priorities@2250+69' => {
                                                       'length' => 69,
                                                       'lhs' => 'priorities',
                                                       'precedence_count' => 1,
                                                       'start' => 2250
                                                     },
                             'priority rule@1427+47' => {
                                                          'length' => 47,
                                                          'lhs' => 'priority rule',
                                                          'precedence_count' => 1,
                                                          'start' => 1427
                                                        },
                             'priority specification@3449+63' => {
                                                                   'length' => 63,
                                                                   'lhs' => 'priority specification',
                                                                   'precedence_count' => 1,
                                                                   'start' => 3449
                                                                 },
                             'proper specification@3142+50' => {
                                                                 'length' => 50,
                                                                 'lhs' => 'proper specification',
                                                                 'precedence_count' => 1,
                                                                 'start' => 3142
                                                               },
                             'quantified rule@1523+79' => {
                                                            'length' => 79,
                                                            'lhs' => 'quantified rule',
                                                            'precedence_count' => 1,
                                                            'start' => 1523
                                                          },
                             'quantifier@5359+24' => {
                                                       'length' => 24,
                                                       'lhs' => 'quantifier',
                                                       'precedence_count' => 1,
                                                       'start' => 5359
                                                     },
                             'rank specification@3193+55' => {
                                                               'length' => 55,
                                                               'lhs' => 'rank specification',
                                                               'precedence_count' => 1,
                                                               'start' => 3193
                                                             },
                             'reserved action name@5521+59' => {
                                                                 'length' => 59,
                                                                 'lhs' => 'reserved action name',
                                                                 'precedence_count' => 1,
                                                                 'start' => 5521
                                                               },
                             'reserved blessing name@5581+61' => {
                                                                   'length' => 61,
                                                                   'lhs' => 'reserved blessing name',
                                                                   'precedence_count' => 1,
                                                                   'start' => 5581
                                                                 },
                             'reserved event name@4148+33' => {
                                                                'length' => 33,
                                                                'lhs' => 'reserved event name',
                                                                'precedence_count' => 1,
                                                                'start' => 4148
                                                              },
                             'result item descriptor list@6449+105' => {
                                                                         'length' => 105,
                                                                         'lhs' => 'result item descriptor list',
                                                                         'precedence_count' => 1,
                                                                         'start' => 6449
                                                                       },
                             'result item descriptor separator@6555+40' => {
                                                                             'length' => 40,
                                                                             'lhs' => 'result item descriptor separator',
                                                                             'precedence_count' => 1,
                                                                             'start' => 6555
                                                                           },
                             'result item descriptor separator@6596+51' => {
                                                                             'length' => 51,
                                                                             'lhs' => 'result item descriptor separator',
                                                                             'precedence_count' => 1,
                                                                             'start' => 6596
                                                                           },
                             'result item descriptor@6648+135' => {
                                                                    'length' => 135,
                                                                    'lhs' => 'result item descriptor',
                                                                    'precedence_count' => 1,
                                                                    'start' => 6648
                                                                  },
                             'rhs primary list@4504+37' => {
                                                             'length' => 37,
                                                             'lhs' => 'rhs primary list',
                                                             'precedence_count' => 1,
                                                             'start' => 4504
                                                           },
                             'rhs primary@4310+33' => {
                                                        'length' => 33,
                                                        'lhs' => 'rhs primary',
                                                        'precedence_count' => 1,
                                                        'start' => 4310
                                                      },
                             'rhs primary@4344+40' => {
                                                        'length' => 40,
                                                        'lhs' => 'rhs primary',
                                                        'precedence_count' => 1,
                                                        'start' => 4344
                                                      },
                             'rhs primary@4385+50' => {
                                                        'length' => 50,
                                                        'lhs' => 'rhs primary',
                                                        'precedence_count' => 1,
                                                        'start' => 4385
                                                      },
                             'rhs@4287+22' => {
                                                'length' => 22,
                                                'lhs' => 'rhs',
                                                'precedence_count' => 1,
                                                'start' => 4287
                                              },
                             'right association@2983+46' => {
                                                              'length' => 46,
                                                              'lhs' => 'right association',
                                                              'precedence_count' => 1,
                                                              'start' => 2983
                                                            },
                             'safe cc character@7358+79' => {
                                                              'length' => 79,
                                                              'lhs' => 'safe cc character',
                                                              'precedence_count' => 1,
                                                              'start' => 7358
                                                            },
                             'separator specification@3077+64' => {
                                                                    'length' => 64,
                                                                    'lhs' => 'separator specification',
                                                                    'precedence_count' => 1,
                                                                    'start' => 3077
                                                                  },
                             'sign@5472+13' => {
                                                 'length' => 13,
                                                 'lhs' => 'sign',
                                                 'precedence_count' => 1,
                                                 'start' => 5472
                                               },
                             'signed integer@5424+47' => {
                                                           'length' => 47,
                                                           'lhs' => 'signed integer',
                                                           'precedence_count' => 1,
                                                           'start' => 5424
                                                         },
                             'single quoted name@7006+78' => {
                                                               'length' => 78,
                                                               'lhs' => 'single quoted name',
                                                               'precedence_count' => 1,
                                                               'start' => 7006
                                                             },
                             'single quoted string@6897+108' => {
                                                                  'length' => 108,
                                                                  'lhs' => 'single quoted string',
                                                                  'precedence_count' => 1,
                                                                  'start' => 6897
                                                                },
                             'single symbol@4542+52' => {
                                                          'length' => 52,
                                                          'lhs' => 'single symbol',
                                                          'precedence_count' => 1,
                                                          'start' => 4542
                                                        },
                             'standard name@6010+57' => {
                                                          'length' => 57,
                                                          'lhs' => 'standard name',
                                                          'precedence_count' => 1,
                                                          'start' => 6010
                                                        },
                             'start rule@1124+51' => {
                                                       'length' => 51,
                                                       'lhs' => 'start rule',
                                                       'precedence_count' => 1,
                                                       'start' => 1124
                                                     },
                             'start rule@1176+47' => {
                                                       'length' => 47,
                                                       'lhs' => 'start rule',
                                                       'precedence_count' => 1,
                                                       'start' => 1176
                                                     },
                             'statement group@1081+42' => {
                                                            'length' => 42,
                                                            'lhs' => 'statement group',
                                                            'precedence_count' => 1,
                                                            'start' => 1081
                                                          },
                             'statement@685+369' => {
                                                      'length' => 369,
                                                      'lhs' => 'statement',
                                                      'precedence_count' => 1,
                                                      'start' => 685
                                                    },
                             'statements@659+25' => {
                                                      'length' => 25,
                                                      'lhs' => 'statements',
                                                      'precedence_count' => 1,
                                                      'start' => 659
                                                    },
                             'string without single quote or vertical space@7085+103' => {
                                                                                           'length' => 103,
                                                                                           'lhs' => 'string without single quote or vertical space',
                                                                                           'precedence_count' => 1,
                                                                                           'start' => 7085
                                                                                         },
                             'symbol name@4620+29' => {
                                                        'length' => 29,
                                                        'lhs' => 'symbol name',
                                                        'precedence_count' => 1,
                                                        'start' => 4620
                                                      },
                             'symbol name@4650+34' => {
                                                        'length' => 34,
                                                        'lhs' => 'symbol name',
                                                        'precedence_count' => 1,
                                                        'start' => 4650
                                                      },
                             'symbol@4595+24' => {
                                                   'length' => 24,
                                                   'lhs' => 'symbol',
                                                   'precedence_count' => 1,
                                                   'start' => 4595
                                                 },
                             'terminated hash comment@4961+73' => {
                                                                    'length' => 73,
                                                                    'lhs' => 'terminated hash comment',
                                                                    'precedence_count' => 1,
                                                                    'start' => 4961
                                                                  },
                             'unterminated final hash comment@5035+59' => {
                                                                            'length' => 59,
                                                                            'lhs' => 'unterminated final hash comment',
                                                                            'precedence_count' => 1,
                                                                            'start' => 5035
                                                                          },
                             'vertical space char@5138+62' => {
                                                                'length' => 62,
                                                                'lhs' => 'vertical space char',
                                                                'precedence_count' => 1,
                                                                'start' => 5138
                                                              },
                             'whitespace@4816+18' => {
                                                       'length' => 18,
                                                       'lhs' => 'whitespace',
                                                       'precedence_count' => 1,
                                                       'start' => 4816
                                                     },
                             'zero or more word characters@5681+38' => {
                                                                         'length' => 38,
                                                                         'lhs' => 'zero or more word characters',
                                                                         'precedence_count' => 1,
                                                                         'start' => 5681
                                                                       }
                           },
                  'xsy' => {
                             'Perl identifier' => {
                                                    'dsl_form' => 'Perl identifier',
                                                    'name_source' => 'lexical'
                                                  },
                             'Perl name' => {
                                              'dsl_form' => 'Perl name',
                                              'name_source' => 'lexical'
                                            },
                             '[Lex-0]' => {
                                            'dsl_form' => '\';\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-10]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-11]' => {
                                             'dsl_form' => '\'discard\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-12]' => {
                                             'dsl_form' => '\'default\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-13]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-14]' => {
                                             'dsl_form' => '\':discard\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-15]' => {
                                             'dsl_form' => '\':lexeme\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-16]' => {
                                             'dsl_form' => '\'event\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-17]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-18]' => {
                                             'dsl_form' => '\'completed\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-19]' => {
                                             'dsl_form' => '\'event\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-1]' => {
                                            'dsl_form' => '\'{\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-20]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-21]' => {
                                             'dsl_form' => '\'nulled\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-22]' => {
                                             'dsl_form' => '\'event\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-23]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-24]' => {
                                             'dsl_form' => '\'predicted\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-25]' => {
                                             'dsl_form' => '\'inaccessible\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-26]' => {
                                             'dsl_form' => '\'is\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-27]' => {
                                             'dsl_form' => '\'by\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-28]' => {
                                             'dsl_form' => '\'default\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-29]' => {
                                             'dsl_form' => '\'warn\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-2]' => {
                                            'dsl_form' => '\'}\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-30]' => {
                                             'dsl_form' => '\'ok\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-31]' => {
                                             'dsl_form' => '\'fatal\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-32]' => {
                                             'dsl_form' => '\',\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-33]' => {
                                             'dsl_form' => '\'action\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-34]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-35]' => {
                                             'dsl_form' => '\'assoc\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-36]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-37]' => {
                                             'dsl_form' => '\'left\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-38]' => {
                                             'dsl_form' => '\'assoc\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-39]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-3]' => {
                                            'dsl_form' => '\':start\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-40]' => {
                                             'dsl_form' => '\'right\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-41]' => {
                                             'dsl_form' => '\'assoc\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-42]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-43]' => {
                                             'dsl_form' => '\'group\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-44]' => {
                                             'dsl_form' => '\'separator\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-45]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-46]' => {
                                             'dsl_form' => '\'proper\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-47]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-48]' => {
                                             'dsl_form' => '\'rank\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-49]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-4]' => {
                                            'dsl_form' => '\'start\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-50]' => {
                                             'dsl_form' => '\'null-ranking\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-51]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-52]' => {
                                             'dsl_form' => '\'null\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-53]' => {
                                             'dsl_form' => '\'rank\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-54]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-55]' => {
                                             'dsl_form' => '\'low\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-56]' => {
                                             'dsl_form' => '\'high\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-57]' => {
                                             'dsl_form' => '\'priority\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-58]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-59]' => {
                                             'dsl_form' => '\'pause\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-5]' => {
                                            'dsl_form' => '\'symbol\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-60]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-61]' => {
                                             'dsl_form' => '\'event\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-62]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-63]' => {
                                             'dsl_form' => '\'=\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-64]' => {
                                             'dsl_form' => '\'on\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-65]' => {
                                             'dsl_form' => '\'off\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-66]' => {
                                             'dsl_form' => '\'bless\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-67]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-68]' => {
                                             'dsl_form' => '\'name\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-69]' => {
                                             'dsl_form' => '\'=>\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-6]' => {
                                            'dsl_form' => '\'is\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-70]' => {
                                             'dsl_form' => '\'(\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-71]' => {
                                             'dsl_form' => '\')\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-72]' => {
                                             'dsl_form' => '\'*\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-73]' => {
                                             'dsl_form' => '\'+\'',
                                             'name_source' => 'internal'
                                           },
                             '[Lex-7]' => {
                                            'dsl_form' => '\':default\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-8]' => {
                                            'dsl_form' => '\'lexeme\'',
                                            'name_source' => 'internal'
                                          },
                             '[Lex-9]' => {
                                            'dsl_form' => '\'default\'',
                                            'name_source' => 'internal'
                                          },
                             '[[\']]' => {
                                           'dsl_form' => '[\']',
                                           'name_source' => 'internal'
                                         },
                             '[[+-]]' => {
                                           'dsl_form' => '[+-]',
                                           'name_source' => 'internal'
                                         },
                             '[[,]]' => {
                                          'dsl_form' => '[,]',
                                          'name_source' => 'internal'
                                        },
                             '[[01]]' => {
                                           'dsl_form' => '[01]',
                                           'name_source' => 'internal'
                                         },
                             '[[1]]' => {
                                          'dsl_form' => '[1]',
                                          'name_source' => 'internal'
                                        },
                             '[[[:alnum:]]]' => {
                                                  'dsl_form' => '[[:alnum:]]',
                                                  'name_source' => 'internal'
                                                },
                             '[[\\#]]' => {
                                            'dsl_form' => '[\\#]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\(]]' => {
                                            'dsl_form' => '[\\(]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\)]]' => {
                                            'dsl_form' => '[\\)]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\*]]' => {
                                            'dsl_form' => '[\\*]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\+]]' => {
                                            'dsl_form' => '[\\+]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\,]]' => {
                                            'dsl_form' => '[\\,]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\-]]' => {
                                            'dsl_form' => '[\\-]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\:]]' => {
                                            'dsl_form' => '[\\:]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\;]]' => {
                                            'dsl_form' => '[\\;]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\<]]' => {
                                            'dsl_form' => '[\\<]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\=]]' => {
                                            'dsl_form' => '[\\=]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\>]]' => {
                                            'dsl_form' => '[\\>]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\[]]' => {
                                            'dsl_form' => '[\\[]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\\\]]' => {
                                             'dsl_form' => '[\\\\]',
                                             'name_source' => 'internal'
                                           },
                             '[[\\]]]' => {
                                            'dsl_form' => '[\\]]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\^]]' => {
                                            'dsl_form' => '[\\^]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\d]]' => {
                                            'dsl_form' => '[\\d]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\s\\w]]' => {
                                               'dsl_form' => '[\\s\\w]',
                                               'name_source' => 'internal'
                                             },
                             '[[\\s]]' => {
                                            'dsl_form' => '[\\s]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\w]]' => {
                                            'dsl_form' => '[\\w]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                   'dsl_form' => '[\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]',
                                                                                   'name_source' => 'internal'
                                                                                 },
                             '[[\\{]]' => {
                                            'dsl_form' => '[\\{]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\|]]' => {
                                            'dsl_form' => '[\\|]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\}]]' => {
                                            'dsl_form' => '[\\}]',
                                            'name_source' => 'internal'
                                          },
                             '[[\\~]]' => {
                                            'dsl_form' => '[\\~]',
                                            'name_source' => 'internal'
                                          },
                             '[[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                   'dsl_form' => '[^\'\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]',
                                                                                                   'name_source' => 'internal'
                                                                                                 },
                             '[[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]]' => {
                                                                                                        'dsl_form' => '[^\\x{5d}\\x{0A}\\x{0B}\\x{0C}\\x{0D}\\x{0085}\\x{2028}\\x{2029}]',
                                                                                                        'name_source' => 'internal'
                                                                                                      },
                             '[[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]]' => {
                                                                                    'dsl_form' => '[^\\x{A}\\x{B}\\x{C}\\x{D}\\x{2028}\\x{2029}]',
                                                                                    'name_source' => 'internal'
                                                                                  },
                             '[[a-zA-Z]]' => {
                                               'dsl_form' => '[a-zA-Z]',
                                               'name_source' => 'internal'
                                             },
                             '[[a]]' => {
                                          'dsl_form' => '[a]',
                                          'name_source' => 'internal'
                                        },
                             '[[b]]' => {
                                          'dsl_form' => '[b]',
                                          'name_source' => 'internal'
                                        },
                             '[[c]]' => {
                                          'dsl_form' => '[c]',
                                          'name_source' => 'internal'
                                        },
                             '[[d]]' => {
                                          'dsl_form' => '[d]',
                                          'name_source' => 'internal'
                                        },
                             '[[e]]' => {
                                          'dsl_form' => '[e]',
                                          'name_source' => 'internal'
                                        },
                             '[[f]]' => {
                                          'dsl_form' => '[f]',
                                          'name_source' => 'internal'
                                        },
                             '[[g]]' => {
                                          'dsl_form' => '[g]',
                                          'name_source' => 'internal'
                                        },
                             '[[h]]' => {
                                          'dsl_form' => '[h]',
                                          'name_source' => 'internal'
                                        },
                             '[[i]]' => {
                                          'dsl_form' => '[i]',
                                          'name_source' => 'internal'
                                        },
                             '[[k]]' => {
                                          'dsl_form' => '[k]',
                                          'name_source' => 'internal'
                                        },
                             '[[l]]' => {
                                          'dsl_form' => '[l]',
                                          'name_source' => 'internal'
                                        },
                             '[[m]]' => {
                                          'dsl_form' => '[m]',
                                          'name_source' => 'internal'
                                        },
                             '[[n]]' => {
                                          'dsl_form' => '[n]',
                                          'name_source' => 'internal'
                                        },
                             '[[o]]' => {
                                          'dsl_form' => '[o]',
                                          'name_source' => 'internal'
                                        },
                             '[[p]]' => {
                                          'dsl_form' => '[p]',
                                          'name_source' => 'internal'
                                        },
                             '[[r]]' => {
                                          'dsl_form' => '[r]',
                                          'name_source' => 'internal'
                                        },
                             '[[s]]' => {
                                          'dsl_form' => '[s]',
                                          'name_source' => 'internal'
                                        },
                             '[[t]]' => {
                                          'dsl_form' => '[t]',
                                          'name_source' => 'internal'
                                        },
                             '[[u]]' => {
                                          'dsl_form' => '[u]',
                                          'name_source' => 'internal'
                                        },
                             '[[v]]' => {
                                          'dsl_form' => '[v]',
                                          'name_source' => 'internal'
                                        },
                             '[[w]]' => {
                                          'dsl_form' => '[w]',
                                          'name_source' => 'internal'
                                        },
                             '[[x]]' => {
                                          'dsl_form' => '[x]',
                                          'name_source' => 'internal'
                                        },
                             '[[y]]' => {
                                          'dsl_form' => '[y]',
                                          'name_source' => 'internal'
                                        },
                             'action' => {
                                           'dsl_form' => 'action',
                                           'name_source' => 'lexical'
                                         },
                             'action name' => {
                                                'dsl_form' => 'action name',
                                                'name_source' => 'lexical'
                                              },
                             'adverb item' => {
                                                'dsl_form' => 'adverb item',
                                                'name_source' => 'lexical'
                                              },
                             'adverb list' => {
                                                'dsl_form' => 'adverb list',
                                                'name_source' => 'lexical'
                                              },
                             'adverb list items' => {
                                                      'dsl_form' => 'adverb list items',
                                                      'name_source' => 'lexical'
                                                    },
                             'alternative' => {
                                                'dsl_form' => 'alternative',
                                                'name_source' => 'lexical'
                                              },
                             'alternative name' => {
                                                     'dsl_form' => 'alternative name',
                                                     'name_source' => 'lexical'
                                                   },
                             'alternatives' => {
                                                 'dsl_form' => 'alternatives',
                                                 'name_source' => 'lexical'
                                               },
                             'array descriptor' => {
                                                     'dsl_form' => 'array descriptor',
                                                     'name_source' => 'lexical'
                                                   },
                             'array descriptor left bracket' => {
                                                                  'dsl_form' => 'array descriptor left bracket',
                                                                  'name_source' => 'lexical'
                                                                },
                             'array descriptor right bracket' => {
                                                                   'dsl_form' => 'array descriptor right bracket',
                                                                   'name_source' => 'lexical'
                                                                 },
                             'bare name' => {
                                              'dsl_form' => 'bare name',
                                              'name_source' => 'lexical'
                                            },
                             'before or after' => {
                                                    'dsl_form' => 'before or after',
                                                    'name_source' => 'lexical'
                                                  },
                             'blessing' => {
                                             'dsl_form' => 'blessing',
                                             'name_source' => 'lexical'
                                           },
                             'blessing name' => {
                                                  'dsl_form' => 'blessing name',
                                                  'name_source' => 'lexical'
                                                },
                             'boolean' => {
                                            'dsl_form' => 'boolean',
                                            'name_source' => 'lexical'
                                          },
                             'bracketed name' => {
                                                   'dsl_form' => 'bracketed name',
                                                   'name_source' => 'lexical'
                                                 },
                             'bracketed name string' => {
                                                          'dsl_form' => 'bracketed name string',
                                                          'name_source' => 'lexical'
                                                        },
                             'cc element' => {
                                               'dsl_form' => 'cc element',
                                               'name_source' => 'lexical'
                                             },
                             'cc elements' => {
                                                'dsl_form' => 'cc elements',
                                                'name_source' => 'lexical'
                                              },
                             'character class' => {
                                                    'dsl_form' => 'character class',
                                                    'name_source' => 'lexical'
                                                  },
                             'character class modifier' => {
                                                             'dsl_form' => 'character class modifier',
                                                             'name_source' => 'lexical'
                                                           },
                             'character class modifiers' => {
                                                              'dsl_form' => 'character class modifiers',
                                                              'name_source' => 'lexical'
                                                            },
                             'completion event declaration' => {
                                                                 'dsl_form' => 'completion event declaration',
                                                                 'name_source' => 'lexical'
                                                               },
                             'default rule' => {
                                                 'dsl_form' => 'default rule',
                                                 'name_source' => 'lexical'
                                               },
                             'discard default statement' => {
                                                              'dsl_form' => 'discard default statement',
                                                              'name_source' => 'lexical'
                                                            },
                             'discard rule' => {
                                                 'dsl_form' => 'discard rule',
                                                 'name_source' => 'lexical'
                                               },
                             'double colon' => {
                                                 'dsl_form' => 'double colon',
                                                 'name_source' => 'lexical'
                                               },
                             'empty rule' => {
                                               'dsl_form' => 'empty rule',
                                               'name_source' => 'lexical'
                                             },
                             'escaped cc character' => {
                                                         'dsl_form' => 'escaped cc character',
                                                         'name_source' => 'lexical'
                                                       },
                             'event initialization' => {
                                                         'dsl_form' => 'event initialization',
                                                         'name_source' => 'lexical'
                                                       },
                             'event initializer' => {
                                                      'dsl_form' => 'event initializer',
                                                      'name_source' => 'lexical'
                                                    },
                             'event name' => {
                                               'dsl_form' => 'event name',
                                               'name_source' => 'lexical'
                                             },
                             'event specification' => {
                                                        'dsl_form' => 'event specification',
                                                        'name_source' => 'lexical'
                                                      },
                             'group association' => {
                                                      'dsl_form' => 'group association',
                                                      'name_source' => 'lexical'
                                                    },
                             'hash comment' => {
                                                 'dsl_form' => 'hash comment',
                                                 'name_source' => 'lexical'
                                               },
                             'hash comment body' => {
                                                      'dsl_form' => 'hash comment body',
                                                      'name_source' => 'lexical'
                                                    },
                             'hash comment char' => {
                                                      'dsl_form' => 'hash comment char',
                                                      'name_source' => 'lexical'
                                                    },
                             'horizontal character' => {
                                                         'dsl_form' => 'horizontal character',
                                                         'name_source' => 'lexical'
                                                       },
                             'inaccessible statement' => {
                                                           'dsl_form' => 'inaccessible statement',
                                                           'name_source' => 'lexical'
                                                         },
                             'inaccessible treatment' => {
                                                           'dsl_form' => 'inaccessible treatment',
                                                           'name_source' => 'lexical'
                                                         },
                             'integer' => {
                                            'dsl_form' => 'integer',
                                            'name_source' => 'lexical'
                                          },
                             'left association' => {
                                                     'dsl_form' => 'left association',
                                                     'name_source' => 'lexical'
                                                   },
                             'lexeme default statement' => {
                                                             'dsl_form' => 'lexeme default statement',
                                                             'name_source' => 'lexical'
                                                           },
                             'lexeme rule' => {
                                                'dsl_form' => 'lexeme rule',
                                                'name_source' => 'lexical'
                                              },
                             'lhs' => {
                                        'dsl_form' => 'lhs',
                                        'name_source' => 'lexical'
                                      },
                             'naming' => {
                                           'dsl_form' => 'naming',
                                           'name_source' => 'lexical'
                                         },
                             'negated posix char class' => {
                                                             'dsl_form' => 'negated posix char class',
                                                             'name_source' => 'lexical'
                                                           },
                             'null adverb' => {
                                                'dsl_form' => 'null adverb',
                                                'name_source' => 'lexical'
                                              },
                             'null ranking constant' => {
                                                          'dsl_form' => 'null ranking constant',
                                                          'name_source' => 'lexical'
                                                        },
                             'null ranking specification' => {
                                                               'dsl_form' => 'null ranking specification',
                                                               'name_source' => 'lexical'
                                                             },
                             'null statement' => {
                                                   'dsl_form' => 'null statement',
                                                   'name_source' => 'lexical'
                                                 },
                             'nulled event declaration' => {
                                                             'dsl_form' => 'nulled event declaration',
                                                             'name_source' => 'lexical'
                                                           },
                             'on or off' => {
                                              'dsl_form' => 'on or off',
                                              'name_source' => 'lexical'
                                            },
                             'one or more word characters' => {
                                                                'dsl_form' => 'one or more word characters',
                                                                'name_source' => 'lexical'
                                                              },
                             'op declare' => {
                                               'dsl_form' => 'op declare',
                                               'name_source' => 'lexical'
                                             },
                             'op declare bnf' => {
                                                   'dsl_form' => 'op declare bnf',
                                                   'name_source' => 'lexical'
                                                 },
                             'op declare match' => {
                                                     'dsl_form' => 'op declare match',
                                                     'name_source' => 'lexical'
                                                   },
                             'op equal priority' => {
                                                      'dsl_form' => 'op equal priority',
                                                      'name_source' => 'lexical'
                                                    },
                             'op loosen' => {
                                              'dsl_form' => 'op loosen',
                                              'name_source' => 'lexical'
                                            },
                             'parenthesized rhs primary list' => {
                                                                   'dsl_form' => 'parenthesized rhs primary list',
                                                                   'name_source' => 'lexical'
                                                                 },
                             'pause specification' => {
                                                        'dsl_form' => 'pause specification',
                                                        'name_source' => 'lexical'
                                                      },
                             'posix char class' => {
                                                     'dsl_form' => 'posix char class',
                                                     'name_source' => 'lexical'
                                                   },
                             'posix char class name' => {
                                                          'dsl_form' => 'posix char class name',
                                                          'name_source' => 'lexical'
                                                        },
                             'prediction event declaration' => {
                                                                 'dsl_form' => 'prediction event declaration',
                                                                 'name_source' => 'lexical'
                                                               },
                             'priorities' => {
                                               'dsl_form' => 'priorities',
                                               'name_source' => 'lexical'
                                             },
                             'priority rule' => {
                                                  'dsl_form' => 'priority rule',
                                                  'name_source' => 'lexical'
                                                },
                             'priority specification' => {
                                                           'dsl_form' => 'priority specification',
                                                           'name_source' => 'lexical'
                                                         },
                             'proper specification' => {
                                                         'dsl_form' => 'proper specification',
                                                         'name_source' => 'lexical'
                                                       },
                             'quantified rule' => {
                                                    'dsl_form' => 'quantified rule',
                                                    'name_source' => 'lexical'
                                                  },
                             'quantifier' => {
                                               'dsl_form' => 'quantifier',
                                               'name_source' => 'lexical'
                                             },
                             'rank specification' => {
                                                       'dsl_form' => 'rank specification',
                                                       'name_source' => 'lexical'
                                                     },
                             'reserved action name' => {
                                                         'dsl_form' => 'reserved action name',
                                                         'name_source' => 'lexical'
                                                       },
                             'reserved blessing name' => {
                                                           'dsl_form' => 'reserved blessing name',
                                                           'name_source' => 'lexical'
                                                         },
                             'reserved event name' => {
                                                        'dsl_form' => 'reserved event name',
                                                        'name_source' => 'lexical'
                                                      },
                             'result item descriptor' => {
                                                           'dsl_form' => 'result item descriptor',
                                                           'name_source' => 'lexical'
                                                         },
                             'result item descriptor list' => {
                                                                'dsl_form' => 'result item descriptor list',
                                                                'name_source' => 'lexical'
                                                              },
                             'result item descriptor separator' => {
                                                                     'dsl_form' => 'result item descriptor separator',
                                                                     'name_source' => 'lexical'
                                                                   },
                             'rhs' => {
                                        'dsl_form' => 'rhs',
                                        'name_source' => 'lexical'
                                      },
                             'rhs primary' => {
                                                'dsl_form' => 'rhs primary',
                                                'name_source' => 'lexical'
                                              },
                             'rhs primary list' => {
                                                     'dsl_form' => 'rhs primary list',
                                                     'name_source' => 'lexical'
                                                   },
                             'right association' => {
                                                      'dsl_form' => 'right association',
                                                      'name_source' => 'lexical'
                                                    },
                             'safe cc character' => {
                                                      'dsl_form' => 'safe cc character',
                                                      'name_source' => 'lexical'
                                                    },
                             'separator specification' => {
                                                            'dsl_form' => 'separator specification',
                                                            'name_source' => 'lexical'
                                                          },
                             'sign' => {
                                         'dsl_form' => 'sign',
                                         'name_source' => 'lexical'
                                       },
                             'signed integer' => {
                                                   'dsl_form' => 'signed integer',
                                                   'name_source' => 'lexical'
                                                 },
                             'single quoted name' => {
                                                       'dsl_form' => 'single quoted name',
                                                       'name_source' => 'lexical'
                                                     },
                             'single quoted string' => {
                                                         'dsl_form' => 'single quoted string',
                                                         'name_source' => 'lexical'
                                                       },
                             'single symbol' => {
                                                  'dsl_form' => 'single symbol',
                                                  'name_source' => 'lexical'
                                                },
                             'standard name' => {
                                                  'dsl_form' => 'standard name',
                                                  'name_source' => 'lexical'
                                                },
                             'start rule' => {
                                               'dsl_form' => 'start rule',
                                               'name_source' => 'lexical'
                                             },
                             'statement' => {
                                              'dsl_form' => 'statement',
                                              'name_source' => 'lexical'
                                            },
                             'statement group' => {
                                                    'dsl_form' => 'statement group',
                                                    'name_source' => 'lexical'
                                                  },
                             'statements' => {
                                               'dsl_form' => 'statements',
                                               'name_source' => 'lexical'
                                             },
                             'string without single quote or vertical space' => {
                                                                                  'dsl_form' => 'string without single quote or vertical space',
                                                                                  'name_source' => 'lexical'
                                                                                },
                             'symbol' => {
                                           'dsl_form' => 'symbol',
                                           'name_source' => 'lexical'
                                         },
                             'symbol name' => {
                                                'dsl_form' => 'symbol name',
                                                'name_source' => 'lexical'
                                              },
                             'terminated hash comment' => {
                                                            'dsl_form' => 'terminated hash comment',
                                                            'name_source' => 'lexical'
                                                          },
                             'unterminated final hash comment' => {
                                                                    'dsl_form' => 'unterminated final hash comment',
                                                                    'name_source' => 'lexical'
                                                                  },
                             'vertical space char' => {
                                                        'dsl_form' => 'vertical space char',
                                                        'name_source' => 'lexical'
                                                      },
                             'whitespace' => {
                                               'dsl_form' => 'whitespace',
                                               'name_source' => 'lexical'
                                             },
                             'zero or more word characters' => {
                                                                 'dsl_form' => 'zero or more word characters',
                                                                 'name_source' => 'lexical'
                                                               }
                           }
                };
## The code before this line was automatically generated by sl_to_hash.pl

##use critic

    return $hashed_metag;

} ## end sub meta_grammar

1;

# vim: expandtab shiftwidth=4:
